/*
 * Copyright (c) 1995, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*----------------------------------------------------------------------------
 *
 * example showing how to use the porting layer for arcs
 *
 * Author : Yusuf Attarwala
 *          SGI - Applications
 * Date   : Apr 95
 *
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Xm/Xm.h> 
#include <Xm/Frame.h>               /* for frame widgets */
#include <Xm/Form.h>                /* for frame widgets */
#include <X11/keysym.h>             /* keyboard translations */
#include <X11/StringDefs.h>

#include <GL/gl.h>                  /* gl includes */
#include <GL/glu.h>                 /* utility library includes */
#include <GL/GLwMDrawA.h>           /* include for the drawing area widget */

#include "twodwrap.h"

/* function declarations */

void 
    createToplevel(void),
    drawScene(void),
    setMatrix(void),
    exposeCB(Widget,XtPointer,XtPointer),
    resizeCB(Widget,XtPointer,XtPointer),
    initCB(Widget,XtPointer,XtPointer);
 

/* global variables */
            
Display       *display;             /* current display */
XtAppContext  appContext;           /* X application context */

Widget        toplevel,       /* toplevel shell */
              glw;            /* current widget */
             
GLXContext    glxc;           /* current glx context */

Arg args[20];
int acnt;

void 
main(int argc, char** argv)
{
    XtToolkitInitialize();
    appContext = XtCreateApplicationContext();
    display    = XtOpenDisplay(appContext, NULL, "Olight","olight",NULL,0,
                              &argc,argv);
    createToplevel();             /* create widget hierarchy */
    XtAppMainLoop(appContext);    /* loop forever */
}


void
createToplevel(void)
{
    Widget frame;

    acnt = 0;
    XtSetArg(args[acnt],XmNminHeight, 300);acnt++;
    XtSetArg(args[acnt],XmNminWidth,  300);acnt++;
    XtSetArg(args[acnt],XmNminAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNminAspectY,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectY,  1);acnt++;
    toplevel  = XtAppCreateShell("openGL arc","oarc",
                                  applicationShellWidgetClass,
                                  display,args,acnt);

    /* create a frame to hold glx widget */
    frame   = XtVaCreateManagedWidget("frame", 
			      xmFrameWidgetClass, toplevel, 
			      NULL);

    /* create a double buffer widget, in rgb mode and manage it */
    acnt = 0;
    XtSetArg(args[acnt], GLwNrgba,               TRUE); acnt++;
    XtSetArg(args[acnt], GLwNredSize,            1); acnt++;
    XtSetArg(args[acnt], GLwNgreenSize,          1); acnt++;
    XtSetArg(args[acnt], GLwNblueSize,           1); acnt++;
    XtSetArg(args[acnt], GLwNdoublebuffer,       TRUE); acnt++;
    XtSetArg(args[acnt], GLwNallocateBackground, TRUE); acnt++;
    glw = GLwCreateMDrawingArea(frame, "glw", args, acnt);
    XtManageChild(glw);

    /* register callbacks */
    XtAddCallback(glw, GLwNginitCallback,  initCB,   NULL);

    /* realize widget hierarchy */
    XtRealizeWidget(toplevel);

    /* additional callbacks */
    XtAddCallback(glw, GLwNexposeCallback, exposeCB, NULL);
    XtAddCallback(glw, GLwNresizeCallback, resizeCB, NULL);

}

void
drawScene(void)
{
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    glColor3f(1.0,1.0,0);
    arcf(-5.,-5.6,4.0,2700,0);

    glColor3f(0,1.0,1.0);
    arc(5.,5.,3.0,1800,900);

    glColor3f(1.0,1.0,1.0);
    circf(-5.,5.,3.0);

    glColor3f(1.0,0.0,0.0);
    arci(5,-5,3,1000,2000);

    glColor3f(1.0,1.0,0.0);
    rect(-1.,-1.,2.0,2.0);

    glColor3f(0.0,1.0,0.0);
    sboxf(2.,2.,3.5,3.5);

    glFlush();
    glXSwapBuffers(XtDisplay(glw), XtWindow(glw));

}

void
setMatrix(void)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(-10.0,10.0,-10.0,10.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void 
resizeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;


    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    setMatrix();
    drawScene();
}

void 
exposeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;

    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    drawScene();
}

void
initCB(Widget w, XtPointer client_data, XtPointer call)
{
    XVisualInfo *vi;

    XtSetArg(args[0], GLwNvisualInfo, &vi);
    XtGetValues(w, args, 1);

    glxc = glXCreateContext(XtDisplay(w), vi, 0, GL_TRUE);

    glClearColor(0.0, 0.0, 0.0, 0.0);
}

