/*
 * Copyright (c) 1995, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*----------------------------------------------------------------------------
 *
 * file   : lm.h
 *
 * Author : Yusuf Attarwala
 * Date   : Apr 95
 *
 *---------------------------------------------------------------------------*/

/* some material, lights and light models */

/* materials */

#define MAT_BRASS      10
#define MAT_REDPLASTIC 11
#define MAT_SILVER     12
#define MAT_GRNPLASTIC 13

static float mat_brass[]  =      {AMBIENT,  0.25,0.15,0.0,
                                  DIFFUSE,  0.65,0.5, 0.35,
                                  SPECULAR, 0.9, 0.6, 0.0,
                                  SHININESS,10.0,
                                  LMNULL };
 
static float mat_silver[] =      {AMBIENT,  0.4, 0.4, 0.4,
				  DIFFUSE,  0.3, 0.3, 0.3,
				  SPECULAR, 0.9, 0.9, 0.95,
				  SHININESS,25.0,
				  LMNULL };


static float mat_redplastic[] =  {AMBIENT,  0.3, 0.1, 0.1,
				  DIFFUSE,  0.5, 0.1, 0.1,
				  SPECULAR, 0.45,0.45,0.45,
				  SHININESS,30.0,
				  LMNULL };

static float mat_grnplastic[] =  {AMBIENT,  0.1, 0.3, 0.1,
			          DIFFUSE,  0.1, 0.5, 0.1,
			          SPECULAR, 0.45,0.45,0.45,
			          SHININESS,30.0,
			          LMNULL };



/* light models */

#define LTM_INFINITE   20
#define LTM_LOCAL      21
#define LTM_INF_TWO    22

static float infinite[] =         {AMBIENT, 0.3, 0.3, 0.3,
                                   LOCALVIEWER,0.0,
                                   LMNULL };

static float local[] =            {AMBIENT, 0.3, 0.3, 0.3,
                                   LOCALVIEWER,1.0,
                                   ATTENUATION,1.0,0.0,
                                   LMNULL };

static float inf_twosided[] =     {AMBIENT, 0.3, 0.3, 0.3,
                                   LOCALVIEWER,0.0,
                                   TWOSIDE,1.0,
                                   LMNULL };


/* and some lights */

#define LT_INF_WHITE   30
#define LT_INF_RED     31

static float white_inf_light[] = {LCOLOR,   1.0,1.0,1.0,
                                  POSITION, 0.0,0.0,1.0,0.0,
                                  LMNULL };

static float red_inf_light[] =   {LCOLOR,   0.9,0.6,0.6,
                                  POSITION, 1.0,0.0,1.0,0.0,
                                  LMNULL };

