/*
 * Copyright (c) 1993-94, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*----------------------------------------------------------------------------
 *
 * oedgeflag.c : openGL (motif) example showing use of edge flag.
 *
 * Author : Yusuf Attarwala
 *          SGI - Applications
 * Date   : Sep 93
 *
 *    press  left   button to move object 
 *           middle button to turn off edge flaging
 *           right  button to turn on  edge flaging
 *
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include <Xm/Xm.h> 
#include <Xm/Frame.h>               /* for frame widgets */
#include <Xm/Form.h>               /* for frame widgets */
#include <X11/keysym.h>             /* keyboard translations */
#include <X11/StringDefs.h>

#include <GL/gl.h>                  /* gl includes */
#include <GL/glu.h>                 /* utility library includes */
#include <GL/GLwMDrawA.h>           /* include for the drawing area widget */

/* function declarations */

void 
    createToplevel(void),
    drawScene(void),
    setMatrix(void),
    animation(void),
    exposeCB(Widget,XtPointer,XtPointer),
    resizeCB(Widget,XtPointer,XtPointer),
    initCB(Widget,XtPointer,XtPointer),
    inputCB(Widget,XtPointer,XtPointer);
 

/* global variables */
            
Display       *display;             /* current display */
XtAppContext  appContext;           /* X application context */
float         ax,ay,az;             /* angles for animation */

Widget        toplevel,       /* toplevel shell */
              glw;            /* current widget */
             
GLXContext    glxc;           /* current glx context */

/* coordinates of star */
static GLfloat   starv[10][3] = {3.5,5.0,0.0,
				 2.0,2.0,0.0,
				 5.0,4.0,0.0,
				 8.0,2.0,0.0,
				 6.5,5.0,0.0,
				 8.0,6.0,0.0,
				 6.0,6.0,0.0,
				 5.0,8.0,0.0,
				 4.0,6.0,0.0,
				 2.0,6.0,0.0};

#define NTRI 8   /* number of triangles in the star */

/* connectivity data */
static int       stari[NTRI][3] = {0,1,2,
				   2,3,4,
				   4,5,6,
				   6,7,8,
				   8,9,0,
				   0,2,8,
				   8,2,6,
				   6,2,4};

static GLfloat starcolor[NTRI][3] = {1.0,0.0,0.0,
				     0.0,1.0,0.0,
				     0.0,0.0,1.0,
				     1.0,0.0,1.0,
				     0.0,1.0,1.0,
				     1.0,1.0,0.0,
				     1.0,1.0,1.0,
				     1.0,0.5,0.5};


int doEdgeFlag = 0;

/* edge flags */
static GLboolean stare[NTRI][3] = {TRUE,TRUE,FALSE,
                                   TRUE,TRUE,FALSE,
                                   TRUE,TRUE,FALSE,
                                   TRUE,TRUE,FALSE,
                                   TRUE,TRUE,FALSE,
                                   FALSE,FALSE,FALSE,
                                   FALSE,FALSE,FALSE,
                                   FALSE,FALSE,FALSE};

Arg args[20];
int acnt;

void 
main(int argc, char** argv)
{
    XtToolkitInitialize();
    appContext = XtCreateApplicationContext();
    display    = XtOpenDisplay(appContext, NULL, "Oedge","oedge",NULL,0,
                              &argc,argv);
    if (!display) {
        printf("%s : Unable to open display\n",argv[0]);
        exit(0);
    }

    printf("\n---------------------------------------------\n");
    printf("OpenGL edge flag example \n\n");
    printf("Press:  left   button for animation\n\
        middle button to turn OFF edgeflag (default)\n\
        right  button to turn ON  edgeflag \n");

    createToplevel();             /* create widget hierarchy */
    XtAppMainLoop(appContext);    /* loop forever */
}


void
createToplevel(void)
{
    Widget frame;

    acnt = 0;
    XtSetArg(args[acnt],XmNminHeight, 300);acnt++;
    XtSetArg(args[acnt],XmNminWidth,  300);acnt++;
    XtSetArg(args[acnt],XmNminAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNminAspectY,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectY,  1);acnt++;
    toplevel  = XtAppCreateShell("openGL edge","xtext",
                                  applicationShellWidgetClass,
                                  display,args,acnt);

    /* create a frame to hold glx widget */
    frame   = XtVaCreateManagedWidget("frame", 
			      xmFrameWidgetClass, toplevel, 
			      NULL);

    /* create a double buffer widget, in rgb mode and manage it */
    acnt = 0;
    XtSetArg(args[acnt], GLwNrgba,               TRUE); acnt++;
    XtSetArg(args[acnt], GLwNdoublebuffer,       TRUE); acnt++;
    XtSetArg(args[acnt], GLwNallocateBackground, TRUE); acnt++;
    glw = GLwCreateMDrawingArea(frame, "glw", args, acnt);
    XtManageChild(glw);

    /* register callbacks */
    XtAddCallback(glw, GLwNginitCallback,  initCB,   NULL);

    /* realize widget hierarchy */
    XtRealizeWidget(toplevel);

    /* additional callbacks */
    XtAddCallback(glw, GLwNexposeCallback, exposeCB, NULL);
    XtAddCallback(glw, GLwNresizeCallback, resizeCB, NULL);
    XtAddCallback(glw, GLwNinputCallback,  inputCB,  NULL);

}

void
drawScene(void)
{
    int i;
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT);

    glPushMatrix();

    glTranslatef(4.0,4.0,0.0);
    glRotatef (ax,1.0,0.0,0.0);       /* animation angles */
    glRotatef (-ay,0.0, 1.0, 0.0);
    glRotatef (az,0.0, 0.0, 1.0);
    glTranslatef(-4.0,-4.0,0.0);

    /* render the 8 triangles */
    for (i=0;i<NTRI;i++) {
        glColor3fv(starcolor[i]);
        if (doEdgeFlag) {
	    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	    glBegin(GL_POLYGON);
        }
        else {
            glBegin(GL_LINE_LOOP);
        }
	if (doEdgeFlag) glEdgeFlag(stare[i][0]);
	glVertex3fv(starv[stari[i][0]]);
	if (doEdgeFlag) glEdgeFlag(stare[i][1]);
	glVertex3fv(starv[stari[i][1]]);
	if (doEdgeFlag) glEdgeFlag(stare[i][2]);
	glVertex3fv(starv[stari[i][2]]);
	glEnd();
    }

    glPopMatrix();

    glFlush();
    glXSwapBuffers(XtDisplay(glw), XtWindow(glw));

}

void
setMatrix(void)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-0.5,9.5,-0.5,9.5,-10.0,10.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void
animation(void)
{
    register int i;
    /* animate the star */

    for (i=0;i<60;i++) {
	/*
        ax += 5.0;
        ay -= 2.0;
	*/
	az += 3.0;
        if (ax >= 360)  ax = 0.0;
        if (ay <= -360) ay = 0.0;
        if (az >= 360)  az = 0.0;
        drawScene();
    }
}

void 
inputCB(Widget w, XtPointer client_data, XtPointer call)
{
    char            string[31];
    XComposeStatus  composeStatus;
    KeySym keysym;
    GLwDrawingAreaCallbackStruct *call_data;

    call_data = (GLwDrawingAreaCallbackStruct *) call;

    switch(call_data->event->type) {
    case ButtonPress:
        switch (call_data->event->xbutton.button) {
        case Button1:
	    animation();
            break;
        case Button2 :
	    doEdgeFlag = 0;
	    XtVaSetValues(toplevel,XmNtitle, "Edge Flag OFF",NULL);
	    drawScene();
            break;
        case Button3 :
	    doEdgeFlag = 1;
	    XtVaSetValues(toplevel,XmNtitle, "Edge Flag ON",NULL);
	    drawScene();
            break;
        }
        break;
    case KeyPress :
        XLookupString((XKeyEvent *)call_data->event,string,
                      30,&keysym,&composeStatus);
        switch(keysym) {
        case XK_Escape :
            exit(0);
            break;
        }
	break;
    default:
        break;
    }
}

void 
resizeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;


    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    setMatrix();
    drawScene();
}

void 
exposeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;


    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    drawScene();
}

void
initCB(Widget w, XtPointer client_data, XtPointer call)
{
    XVisualInfo *vi;


    XtSetArg(args[0], GLwNvisualInfo, &vi);
    XtGetValues(w, args, 1);

    glxc = glXCreateContext(XtDisplay(w), vi, 0, GL_TRUE);

    ax = 10.0;
    ay = -10.0;
    az = 0.0;
}
