/*
 * Copyright (c) 1993-94, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*----------------------------------------------------------------------------
 *
 * ohidden.c : openGL (motif) example showing how to use stencils
 *             to draw outlined polygons.
 *
 * Author : Yusuf Attarwala
 *          SGI - Applications
 * Date   : Jul 93
 *
 *    note : the main intent of this program is to demo the stencil
 *           plane functionality, hence the rendering is kept
 *           simple (wireframe).
 *
 *    press  left   button for animation
 *    press  middle button to turn OFF stenciling
 *    press  right  button to turn ON  stenciling
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Xm/Xm.h> 
#include <Xm/Frame.h>               /* for frame widgets */
#include <Xm/Form.h>               /* for frame widgets */
#include <X11/keysym.h>             /* keyboard translations */
#include <X11/StringDefs.h>

#include <GL/gl.h>                  /* gl includes */
#include <GL/glu.h>                 /* utility library includes */
#include <GL/GLwMDrawA.h>           /* include for the drawing area widget */

static int stencilOn = 1;

/* function declarations */

void 
    createToplevel(void),
    drawScene(void),
    setMatrix(void),
    animation(void),
    exposeCB(Widget,XtPointer,XtPointer),
    resizeCB(Widget,XtPointer,XtPointer),
    initCB(Widget,XtPointer,XtPointer),
    inputCB(Widget,XtPointer,XtPointer),
    drawWireframe(int face),
    drawFilled(int face);

/* global variables */
            
Display       *display;             /* current display */
XtAppContext  appContext;           /* X application context */
float         ax,ay,az;             /* angles for animation */

Widget        toplevel,       /* toplevel shell */
              glw;            /* current widget */
             
GLXContext    glxc;           /* current glx context */

Arg args[20];
int acnt;

/* coords of a cube */
static GLfloat cube[8][3] = {0.0,0.0,0.0,
			    1.0,0.0,0.0,
			    1.0,0.0,1.0,
			    0.0,0.0,1.0,
			    1.0,1.0,0.0,
			    1.0,1.0,1.0,
			    0.0,1.0,1.0,
			    0.0,1.0,0.0};

static int faceIndex[6][4] = {0,1,2,3,
			      1,4,5,2,
			      4,7,6,5,
			      7,0,3,6,
			      3,2,5,6,
			      7,4,1,0};
void 
main(int argc, char** argv)
{
    XtToolkitInitialize();
    appContext = XtCreateApplicationContext();
    display    = XtOpenDisplay(appContext, NULL, "Ostencil","ostencil",NULL,0,
                              &argc,argv);
    if (!display) {
        printf("%s : Unable to open display\n",argv[0]);
        exit(0);
    }

    printf("\n---------------------------------------------\n");
    printf("OpenGL stencil example \n\n");
    printf("Press:  left   button for animation\n\
        middle button to turn OFF stenciling\n\
        right  button to turn ON  stenciling (default)\n");

    createToplevel();             /* create widget hierarchy */
    XtAppMainLoop(appContext);    /* loop forever */
}


void
createToplevel(void)
{
    Widget frame;

    acnt = 0;
    XtSetArg(args[acnt],XmNminHeight, 300);acnt++;
    XtSetArg(args[acnt],XmNminWidth,  300);acnt++;
    XtSetArg(args[acnt],XmNminAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNminAspectY,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectX,  1);acnt++;
    XtSetArg(args[acnt],XmNmaxAspectY,  1);acnt++;
    toplevel  = XtAppCreateShell("openGL stencil","ostencil",
                                  applicationShellWidgetClass,
                                  display,args,acnt);

    /* create a frame to hold glx widget */
    frame   = XtVaCreateManagedWidget("frame", 
			      xmFrameWidgetClass, toplevel, 
			      NULL);

    /* create a double buffer widget, in rgb mode and manage it */
    acnt = 0;
    XtSetArg(args[acnt], GLwNrgba,               TRUE); acnt++;
    XtSetArg(args[acnt], GLwNdoublebuffer,       TRUE); acnt++;
    XtSetArg(args[acnt], GLwNstencilSize,        1); acnt++;
    XtSetArg(args[acnt], GLwNallocateBackground, TRUE); acnt++;
    glw = GLwCreateMDrawingArea(frame, "glw", args, acnt);
    XtManageChild(glw);

    /* register callbacks */
    XtAddCallback(glw, GLwNginitCallback,  initCB,   NULL);

    /* realize widget hierarchy */
    XtRealizeWidget(toplevel);

    /* additional callbacks */
    XtAddCallback(glw, GLwNexposeCallback, exposeCB, NULL);
    XtAddCallback(glw, GLwNresizeCallback, resizeCB, NULL);
    XtAddCallback(glw, GLwNinputCallback,  inputCB,  NULL);

}

static float p0[] = {-5.0,-5.0,0.0};
static float p1[] = {5.0,-5.0,0.0};
static float p2[] = {5.0,5.0,0.0};
static float p3[] = {-5.0,5.0,0.0};

void
drawWireframe(face)
int face;
{
    int i;
    glBegin(GL_LINE_LOOP);
	for (i=0;i<4;i++)
	    glVertex3fv((GLfloat *)cube[faceIndex[face][i]]);
    glEnd();
}

void
drawFilled(face)
int face;
{
    int i;
    glBegin(GL_POLYGON);
	for (i=0;i<4;i++)
	    glVertex3fv((GLfloat *)cube[faceIndex[face][i]]);
    glEnd();
}

void
drawScene(void)
{

    int i;
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

    glPushMatrix();

    glRotatef (ax,1.0,0.0,0.0);
    glRotatef (-ay,0.0, 1.0, 0.0);

    /* all the good stuff follows */

    if (stencilOn) { 
	glEnable(GL_STENCIL_TEST);
        glClear(GL_STENCIL_BUFFER_BIT);
        glStencilMask(1);
        glStencilFunc(GL_ALWAYS,0,1);
        glStencilOp(GL_INVERT, GL_INVERT, GL_INVERT);
    }

    glColor3f (1.0, 1.0, 0.0);
    for (i=0;i<6;i++) {
	drawWireframe(i);
        if (stencilOn) {
            glStencilFunc(GL_EQUAL, 0, 1);
            glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
        }

        glColor3f (0.0, 0.0, 0.0);
	drawFilled(i);

        glColor3f (1.0, 1.0, 0.0);
        if (stencilOn) {
            glStencilFunc(GL_ALWAYS,0,1);
            glStencilOp(GL_INVERT, GL_INVERT, GL_INVERT);
        }
        glColor3f (1.0, 1.0, 0.0);
	drawWireframe(i);
    }
    glPopMatrix();

    if (stencilOn) glDisable(GL_STENCIL_TEST);

    /* end of good stuff */

    glFlush();
    glXSwapBuffers(XtDisplay(glw), XtWindow(glw));

}

void
setMatrix(void)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-2.0,2.0,-2.0,2.0,-2.0,2.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void
animation(void)
{
    register int i;
    /* animate the cone */

    for (i=0;i<60;i++) {
        ax += 5.0;
        ay -= 2.0;
        az += 5.0;
        if (ax >= 360)  ax = 0.0;
        if (ay <= -360) ay = 0.0;
        if (az >= 360)  az = 0.0;
        drawScene();
    }
}

void 
inputCB(Widget w, XtPointer client_data, XtPointer call)
{
    char            string[31];
    XComposeStatus  composeStatus;
    KeySym keysym;
    GLwDrawingAreaCallbackStruct *call_data;

    call_data = (GLwDrawingAreaCallbackStruct *) call;

    switch(call_data->event->type) {
    case ButtonPress:
        switch (call_data->event->xbutton.button) {
        case Button1:
	    animation();
            break;
        case Button2 :
	    stencilOn = 0;
	    XtVaSetValues(toplevel,XmNtitle, "Stencil Disabled",NULL);
	    drawScene();
            break;
        case Button3 :
	    stencilOn = 1;
	    XtVaSetValues(toplevel,XmNtitle, "Stencil Enabled",NULL);
	    drawScene();
            break;
        }
        break;
    case KeyPress :
        XLookupString((XKeyEvent *)call_data->event,string,
                      30,&keysym,&composeStatus);
        switch(keysym) {
        case XK_Escape :
            exit(0);
            break;
        }
	break;
    default:
        break;
    }
}

void 
resizeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;

    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    setMatrix();

    drawScene();
}

void 
exposeCB(Widget w, XtPointer client_data, XtPointer call)
{
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;


    GLwDrawingAreaMakeCurrent(w, glxc);
    glViewport(0, 0, call_data->width, call_data->height);
    drawScene();
}

void
initCB(Widget w, XtPointer client_data, XtPointer call)
{
    XVisualInfo *vi;

    XtSetArg(args[0], GLwNvisualInfo, &vi);
    XtGetValues(w, args, 1);

    glxc = glXCreateContext(XtDisplay(w), vi, 0, GL_TRUE);

    ax = 10.0;
    ay = -10.0;
    az = 0.0;
}
