/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _BASICDEFS_H
#define _BASICDEFS_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Fri Jan  7 18:08:43 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:17:06 1994 by George Kyriazis
 *
 * General defines to be used almost everywhere.
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include <stream.h>
#include <string.h>
#include <ctype.h>
#include <GL/gl.h>

#ifndef TRUE
#define	TRUE	1
#endif

#ifndef FALSE
#define FALSE	!TRUE
#endif

typedef	char	Boolean;	// so we don't conflict with X

const	float	Pi = 3.1415926535;

// misc structure defines
typedef struct {
    float x;
    float y;
} Point2;

typedef struct {
    float x;
    float y;
    float z;
} Point3;

typedef struct {
    float r;
    float g;
    float b;
} RGBColor;

typedef struct {
    float r;
    float g;
    float b;
    float a;
} RGBAColor;

typedef float Matrix4[4][4];

// misc defines

#ifndef MAX
#define MAX(x,y)	(((x)<(y))?(y):(x))
#endif

#ifndef	MIN
#define	MIN(x,y) 	(((x)<(y))?(x):(y))
#endif

#ifndef	ABS
#define	ABS(x)		(((x)<0)?-(x):(x))
#endif

#ifndef	SGN
#define	SGN(x)		(((x)>0)?1:(((x)<0)?-1:0))
#endif

//
// here's a typical inline that is used everywhere
//
inline void
switch_flag( GLenum thing, Boolean state )
{
    switch( state ) {
      case FALSE:
	glDisable( thing );
	break;

      case TRUE:
	glEnable( thing );
	break;
    }
}

//
// create a new string and copy the arg to it.
inline char *
new_strcpy( char *arg )
{
    char *str = new char[ strlen(arg)+1 ];

    strcpy( str, arg );

    return str;
}

//
// make a string from an int
//
extern	char	___tempstr[300];  // can't include it in the inline defn. NI.
inline char *
itoa( int arg )
{
    sprintf( ___tempstr, "%d", arg );

    return new_strcpy( ___tempstr );
}

//
// make a string from a float
//
inline char *
ftoa( float arg )
{
    sprintf( ___tempstr, "%f", arg );

    return new_strcpy( ___tempstr );
}

//
// make a hex string from an int
//
inline char *
itox( int arg )
{
    sprintf( ___tempstr, "%x", arg );

    return new_strcpy( ___tempstr );
}

//
// convert a hex string to an int
//
extern char *___hexconv;
inline int
xtoi( char *hex )
{
    int	i;
    int	n = strlen(hex);
    int	val = 0;

    /* starting from the LSB going to the MSB */
    for( i = 0; i < n; i++ ) {
	/* first add the contribution */
	val += strchr( ___hexconv, tolower( hex[i] ) ) - ___hexconv;
	/* then shift by one hex digit if we are not the last one */
	if( i != n-1 ) {
	    val <<= 4;
	}
    }

    return val;
}

#endif /* _BASICDEFS_H */
