/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Tue Jan 18 17:08:16 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:12:25 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "ColorChooser.h"

#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Sgm/ColorC.h>

#include <Inventor/SbColor.h>
#include "MyColorEditor.h"

#include "GlWindow.h"

#include <GL/glx.h>

const char * const ColorChooser::newValue = "NewValue";

ColorChooser::ColorChooser( char *Name, Widget Parent, GlWindow *dwa )
: VkComponent( Name ),

  toplev_window( Name )
{
    glWindow = dwa;
    
    create_widget( Name, Parent );
}

ColorChooser::~ColorChooser()
{
}

void
ColorChooser::setValue( GLfloat r, GLfloat g, GLfloat b, GLfloat )
{
    rgb_color.r = r;
    rgb_color.g = g;
    rgb_color.b = b;

    if( color_thing ) {
	color_thing->setColor( SbColor( r, g, b ) );
    }
}


void
ColorChooser::create_widget( char *name, Widget parent )
{
    int	ac;
    Arg	av[10];

    // what we do here is create the typical label and button.
    // the button pops up the color thingee.

    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    rowcol = XmCreateRowColumn( parent,
			       name,
			       av, ac );
    XtManageChild( rowcol );

    label = XmCreateLabel( rowcol,
			  name,
			  NULL, 0 );
    XtManageChild( label );

    // now a little button
    button = XmCreatePushButton( rowcol,
				"Set Color",
				NULL, 0 );
    XtManageChild( button );
    XtAddCallback( button,
		  XmNactivateCallback,
		  &ColorChooser::showCB,
		  (XtPointer) this );

    // to be initialized later
    color_thing = NULL;
}


void
ColorChooser::ce_applyCB( void *userData, const SbColor *color )
{
    ColorChooser *self = (ColorChooser *) userData;
    SgColorChooserCallbackStruct	cs;

    cs.r = (short)((*color)[0] * 255);
    cs.g = (short)((*color)[1] * 255);
    cs.b = (short)((*color)[2] * 255);

    // before calling the callback, we have to save the state of
    // the current glX connection, and set it to the current one
    GLXDrawable	drawable = glXGetCurrentDrawable();
    GLXContext	context = glXGetCurrentContext();

    glXMakeCurrent( self->glWindow->getDisplay(),
		   self->glWindow->getNormalWindow(),
		   self->glWindow->getNormalContext() );

    ColorChooser::applyCB( NULL, self, &cs );

    // now restore previous values
    glXMakeCurrent( self->glWindow->getDisplay(), drawable, context );
}


void
ColorChooser::showCB( Widget w, XtPointer client, XtPointer call )
{
    ColorChooser *self = (ColorChooser *) client;

    self->show_cb( w, call );
}

void
ColorChooser::show_cb( Widget, XtPointer )
{
    if( color_thing == NULL ) {
	color_thing = new MyColorEditor;

	Arg av[4];
	int ac;
	char *name;
	XmString Mname;

	ac = 0;
	XtSetArg( av[ac], XmNlabelString, &Mname ); ac++;
	XtGetValues( label, av, ac );
	XmStringGetLtoR( Mname,
			(XmStringCharSet) XmSTRING_DEFAULT_CHARSET,
			&name );
	
	color_thing->setTitle( name );
	color_thing->setWYSIWYG( TRUE );
	color_thing->addColorChangedCallback( &ColorChooser::ce_applyCB,
					     this );
	color_thing->setColor( SbColor( rgb_color.r, rgb_color.g, rgb_color.b ) );
    }
    
    color_thing->show();
}

void
ColorChooser::applyCB( Widget w, XtPointer client, XtPointer call )
{
    ColorChooser *self = (ColorChooser *) client;

    self->apply_cb( w, call );
}

void
ColorChooser::apply_cb( Widget, XtPointer call )
{
    SgColorChooserCallbackStruct *cb =
	(SgColorChooserCallbackStruct *) call;

    rgb_color.r = cb->r / 255.;
    rgb_color.g = cb->g / 255.;
    rgb_color.b = cb->b / 255.;
    
    callCallbacks( newValue, (void *) &rgb_color );
}

void
ColorChooser::cancelCB( Widget w, XtPointer client, XtPointer call )
{
    ColorChooser *self = (ColorChooser *) client;

    self->cancel_cb( w, call );
}

void
ColorChooser::cancel_cb( Widget, XtPointer )
{
    toplev_window.hide();
}

void
ColorChooser::okCB( Widget w, XtPointer client, XtPointer call )
{
    ColorChooser *self = (ColorChooser *) client;

    self->ok_cb( w, call );
}

void
ColorChooser::ok_cb( Widget w, XtPointer call )
{
    apply_cb( w, call );
    cancel_cb( w, call );
}
