/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Mon Jan 10 11:02:54 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:14:02 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "FAOps.h"

#include "IfaceGeneratorP.h"

#include "TextItem.h"
#include <stdlib.h>

int FAOps::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Fog/Antialiasing Ops", 1,

    IF_RADIO, IF_THIS, NULL,
    (int) "Fog: ", 2,
    0, (int) &FAOps::cb_set_fog,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Fog Mode: ", 3,
    1, (int) &FAOps::cb_set_glFog_mode,
    (int) "GL_LINEAR", GL_LINEAR,
    (int) "GL_EXP", GL_EXP,
    (int) "GL_EXP2", GL_EXP2,

    IF_TEXT, IF_THIS, NULL,
    (int) "Fog Density: ",
    10, IF_VALUE+0,
    (int) &TextItem::FRACTIONAL,
    (int) &FAOps::cb_set_glFog_density,

    IF_TEXT, IF_THIS, NULL,
    (int) "Fog Start: ",
    10, IF_VALUE+1,
    (int) &TextItem::FRACTIONAL,
    (int) &FAOps::cb_set_glFog_start,

    IF_TEXT, IF_THIS, NULL,
    (int) "Fog End: ",
    10, IF_VALUE+2,
    (int) &TextItem::FRACTIONAL,
    (int) &FAOps::cb_set_glFog_end,

    IF_TEXT, IF_THIS, NULL,
    (int) "Fog Index: ",
    10, IF_VALUE+3,
    (int) &TextItem::INTEGER,
    (int) &FAOps::cb_set_glFog_index,

    IF_RGBA, IF_THIS, NULL,
    (int) "Fog Color: ",
    IF_VALUE+10, IF_VALUE+11, IF_VALUE+12, IF_VALUE+13,
    (int) &FAOps::cb_set_glFog_color,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Point Smoothing: ", 2,
    0, (int) &FAOps::cb_set_point_smooth,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Line Smoothing: ", 2,
    0, (int) &FAOps::cb_set_line_smooth,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Polygon Smoothing: ", 2,
    0, (int) &FAOps::cb_set_polygon_smooth,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_END
};

FAOps::FAOps( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();
}

FAOps::~FAOps()
{
}

///////////////////////////////////////////////////////////////////

void
FAOps::install_values()
{
    GLfloat	val;
    get_glFog_density( &val );
    install( iface, IF_VALUE+0, (int) ftoa(val) );

    get_glFog_start( &val );
    install( iface, IF_VALUE+1, (int) ftoa(val) );

    get_glFog_end( &val );
    install( iface, IF_VALUE+2, (int) ftoa(val) );

    GLint	ind;
    get_glFog_index( &ind );
    install( iface, IF_VALUE+3, (int) itoa(ind) );

    GLfloat	r, g, b, a;
    get_glFog_color( &r, &g, &b, &a );
    install( iface, IF_VALUE+10, (int) ftoa(r) );
    install( iface, IF_VALUE+11, (int) ftoa(g) );
    install( iface, IF_VALUE+12, (int) ftoa(b) );
    install( iface, IF_VALUE+13, (int) ftoa(a) );
}


void
FAOps::remember()
{
    GLenum	e;
    GLint	i;
    GLfloat	f;
    GLfloat	f1, f2, f3, f4;
    Boolean	b;

    get_glFog_mode( &e );
    get_glFog_density( &f );
    get_glFog_start( &f );
    get_glFog_end( &f );
    get_glFog_index( &i );
    get_glFog_color( &f1, &f2, &f3, &f4 );
    get_fog( &b );
}


//
// fog
//
void
FAOps::set_glFog_mode( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_LINEAR:
	    s = "GL_LINEAR";
	    break;
	  case GL_EXP:
	    s = "GL_EXP";
	    break;
	  case GL_EXP2:
	    s = "GL_EXP2";
	    break;
	}
	printf("glFog GL_FOG_MODE %s\n", s );
    }

    glFogiv( GL_FOG_MODE, (int *) &mode );

    _glFog_mode = mode;

    redraw();
}

void
FAOps::get_glFog_mode( GLenum *mode )
{
    glGetIntegerv( GL_FOG_MODE, (int *) mode );

    _glFog_mode = *mode;
}

void
FAOps::set_glFog_density( GLfloat dens )
{
    if( debuglevel > 0 ) {
	printf("glFog GL_FOG_DENSITY %f\n", dens );
    }

    glFogfv( GL_FOG_DENSITY, &dens );

    _glFog_density = dens;

    redraw();
}

void
FAOps::get_glFog_density( GLfloat *dens )
{
    glGetFloatv( GL_FOG_DENSITY, dens );

    _glFog_density = *dens;
}

void
FAOps::set_glFog_start( GLfloat start )
{
    if( debuglevel > 0 ) {
	printf("glFog GL_FOG_START %f\n", start );
    }

    glFogfv( GL_FOG_START, &start );

    _glFog_start = start;

    redraw();
}

void
FAOps::get_glFog_start( GLfloat *start )
{
    glGetFloatv( GL_FOG_START, start );

    _glFog_start = *start;
}

void
FAOps::set_glFog_end( GLfloat end )
{
    if( debuglevel > 0 ) {
	printf("glFog GL_FOG_END %f\n", end );
    }

    glFogfv( GL_FOG_END, &end );

    _glFog_end = end;

    redraw();
}

void
FAOps::get_glFog_end( GLfloat *end )
{
    glGetFloatv( GL_FOG_END, end );

    _glFog_end = *end;
}

void
FAOps::set_glFog_index( GLint index )
{
    if( debuglevel > 0 ) {
	printf("glFog GL_FOG_INDEX %d\n", index );
    }

    glFogiv( GL_FOG_INDEX, &index );

    _glFog_index = index;

    redraw();
}

void
FAOps::get_glFog_index( GLint *index )
{
    glGetIntegerv( GL_FOG_INDEX, index );

    _glFog_index = *index;
}

void
FAOps::set_glFog_color( GLclampf red, GLclampf green,
			GLclampf blue, GLclampf alpha )
{
    if( debuglevel > 0 ) {
	printf("glFog GL_FOG_COLOR [%f %f %f %f]\n",
	       red, green, blue, alpha );
    }

    GLfloat	v[4];

    v[0] = red;
    v[1] = green;
    v[2] = blue;
    v[3] = alpha;

    glFogfv( GL_FOG_COLOR, v );

    _glFog_color.r = red;
    _glFog_color.g = green;
    _glFog_color.b = blue;
    _glFog_color.a = alpha;

    redraw();
}

void
FAOps::get_glFog_color( GLclampf *red, GLclampf *green,
			GLclampf *blue, GLclampf *alpha )
{
    GLfloat	v[4];

    glGetFloatv( GL_FOG_COLOR, v );

    *red = v[0];
    *green = v[1];
    *blue = v[2];
    *alpha = v[3];

    _glFog_color.r = *red;
    _glFog_color.g = *green;
    _glFog_color.b = *blue;
    _glFog_color.a = *alpha;
}


void
FAOps::set_fog( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_FOG\n");
	} else {
	    printf("glDisable GL_FOG\n");
	}
    }
    switch_flag( GL_FOG, bool );

    _fog = bool;

    redraw();
}

void
FAOps::get_fog( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_FOG );

    _fog = state;

    *bool = state;
}



//
// antialiasing
//
void
FAOps::set_point_smooth( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_POINT_SMOOTH\n");
	} else {
	    printf("glDisable GL_POINT_SMOOTH\n");
	}
    }
    switch_flag( GL_POINT_SMOOTH, bool );

    _point_smooth = bool;

    redraw();
}

void
FAOps::get_point_smooth( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_POINT_SMOOTH );

    _point_smooth = state;

    *bool = state;
}

void
FAOps::set_line_smooth( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_LINE_SMOOTH\n");
	} else {
	    printf("glDisable GL_LINE_SMOOTH\n");
	}
    }
    switch_flag( GL_LINE_SMOOTH, bool );

    _line_smooth = bool;

    redraw();
}

void
FAOps::get_line_smooth( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_LINE_SMOOTH );

    _line_smooth = state;

    *bool = state;
}

void
FAOps::set_polygon_smooth( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_POLYGON_SMOOTH\n");
	} else {
	    printf("glDisable GL_POLYGON_SMOOTH\n");
	}
    }
    switch_flag( GL_POLYGON_SMOOTH, bool );

    _polygon_smooth = bool;

    redraw();
}

void
FAOps::get_polygon_smooth( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_POLYGON_SMOOTH );

    _polygon_smooth = state;

    *bool = state;
}


void
FAOps::reset()
{
    int	debugstack;
    Boolean redrawstack;
    
    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }

    set_glFog_mode( _glFog_mode );
    set_glFog_density( _glFog_density );
    set_glFog_start( _glFog_start );
    set_glFog_end( _glFog_end );
    set_glFog_index( _glFog_index );
    set_glFog_color( _glFog_color.r, _glFog_color.g,
		    _glFog_color.b, _glFog_color.a );
    set_fog( _fog );
    set_point_smooth( _point_smooth );
    set_line_smooth( _line_smooth );
    set_polygon_smooth( _polygon_smooth );
    
    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}


////////////////////////////////////////////////////////////////////
//  callbacks
////////////////////////////////////////////////////////////////////

void
FAOps::cb_set_fog( VkCallbackObject *, void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    int	state = (int) call;

    self->set_fog( state );
}

void
FAOps::cb_set_glFog_mode( VkCallbackObject *,
			  void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    GLenum mode = (GLenum) call;

    self->set_glFog_mode( mode );
}

void
FAOps::cb_set_glFog_density( VkCallbackObject *,
			     void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    float dens = atof( (char *) call );

    self->set_glFog_density( dens );
}

void
FAOps::cb_set_glFog_start( VkCallbackObject *,
			   void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    float start = atof( (char *) call );

    self->set_glFog_start( start );
}

void
FAOps::cb_set_glFog_end( VkCallbackObject *,
			   void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    float end = atof( (char *) call );

    self->set_glFog_end( end );
}

void
FAOps::cb_set_glFog_index( VkCallbackObject *,
			   void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    int index = atoi( (char *) call );

    self->set_glFog_index( index );
}

void
FAOps::cb_set_glFog_color( VkCallbackObject *,
			   void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glFog_color( c->r, c->g, c->b, c->a );
}

void
FAOps::cb_set_point_smooth( VkCallbackObject *,
			    void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    int	state = (int) call;

    self->set_point_smooth( state );
}

void
FAOps::cb_set_line_smooth( VkCallbackObject *,
			    void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    int	state = (int) call;

    self->set_line_smooth( state );
}

void
FAOps::cb_set_polygon_smooth( VkCallbackObject *,
			    void *client, void *call )
{
    FAOps *self = (FAOps *) client;
    int	state = (int) call;

    self->set_polygon_smooth( state );
}
