/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Fri Jan  7 18:43:12 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:15:14 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "FBControl.h"

#include "IfaceGeneratorP.h"

#include "TextItem.h"
#include <stdlib.h>
#include "GlWindow.h"

int FBControl::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Framebuffer Control", 1,

    IF_RGBA, IF_THIS, NULL,
    (int) "Clear Color: ",
    IF_VALUE+10, IF_VALUE+11, IF_VALUE+12, IF_VALUE+13,
    (int) &FBControl::cb_set_glClearColor,

    IF_TEXT, IF_THIS, NULL,
    (int) "Clear Index: ",
    10, (int) "0",
    (int) &TextItem::FRACTIONAL,
    (int) &FBControl::cb_set_glClearIndex,

    IF_TEXT, IF_THIS, NULL,
    (int) "Clear Depth: ",
    10, IF_VALUE+0,
    (int) &TextItem::FRACTIONAL,
    (int) &FBControl::cb_set_glClearDepth,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_TEXT, IF_THIS, NULL,
    (int) "Stencil Clear Value(hex): ",
    10, IF_VALUE+1,
    (int) &TextItem::HEX,
    (int) &FBControl::cb_set_glClearStencil,

    IF_TEXT, IF_THIS, NULL,
    (int) "Stencil Write Mask(hex): ",
    10, IF_VALUE+2,
    (int) &TextItem::HEX,
    (int) &FBControl::cb_set_glStencilMask,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Depth Mask: ", 2,
    1, (int) &FBControl::cb_set_glDepthMask,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Red Write Mask: ", 2,
    1, (int) &FBControl::cb_set_glColorMask_red,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Green Write Mask: ", 2,
    1, (int) &FBControl::cb_set_glColorMask_green,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Blue Write Mask: ", 2,
    1, (int) &FBControl::cb_set_glColorMask_blue,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Alpha Write Mask: ", 2,
    1, (int) &FBControl::cb_set_glColorMask_alpha,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_TEXT, IF_THIS, NULL,
    (int) "Index Write Mask: ",
    5, IF_VALUE+3,
    (int) &TextItem::HEX,
    (int) &FBControl::cb_set_glIndexMask,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Draw Buffer: ", 11,
    6, (int) &FBControl::cb_set_glDrawBuffer,
    (int) "GL_NONE", GL_NONE,
    (int) "GL_FRONT_LEFT", GL_FRONT_LEFT,
    (int) "GL_FRONT_RIGHT", GL_FRONT_RIGHT,
    (int) "GL_BACK_LEFT", GL_BACK_LEFT,
    (int) "GL_BACK_RIGHT", GL_BACK_RIGHT,
    (int) "GL_FRONT", GL_FRONT,
    (int) "GL_BACK", GL_BACK,
    (int) "GL_LEFT", GL_LEFT,
    (int) "GL_RIGHT", GL_RIGHT,
    (int) "GL_FRONT_AND_BACK", GL_FRONT_AND_BACK,
    (int) "GL_AUX0", GL_AUX0,

    IF_END
};

FBControl::FBControl( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();
}

FBControl::~FBControl()
{
}

///////////////////////////////////////////////////////////////////

void
FBControl::install_values()
{
    GLclampd	depth;
    get_glClearDepth( &depth );
    install( iface, IF_VALUE+0, (int) ftoa(depth) );

    GLint	s;
    get_glClearStencil( &s );
    install( iface, IF_VALUE+1, (int) itox(s) );

    GLuint	mask;
    get_glStencilMask( &mask );
    install( iface, IF_VALUE+2, (int) itox(mask) );

    get_glIndexMask( &mask );
    install( iface, IF_VALUE+3, (int) itox(mask) );

    GLfloat	r, g, b, a;
    get_glClearColor( &r, &g, &b, &a );
    install( iface, IF_VALUE+10, (int) ftoa(r) );
    install( iface, IF_VALUE+11, (int) ftoa(g) );
    install( iface, IF_VALUE+12, (int) ftoa(b) );
    install( iface, IF_VALUE+13, (int) ftoa(a) );
}

void
FBControl::remember()
{
    GLboolean	b;
    GLboolean	b1, b2, b3, b4;
    GLint	i;
    GLuint	ui;
    GLfloat	f;
    GLfloat	f1, f2, f3, f4;
    GLclampd	d;
    GLenum	e;
    
    get_glClearColor( &f1, &f2, &f3, &f4 );
    get_glClearIndex( &f );
    get_glClearDepth( &d );
    get_glClearStencil( &i );

    get_glDepthMask( &b );
    get_glStencilMask( &ui );
    get_glColorMask( &b1, &b2, &b3, &b4 );
    _glIndexMask = 0xfff;
    get_glDrawBuffer( &e );
}


void
FBControl::set_glClearColor(GLclampf red,
			    GLclampf green,
			    GLclampf blue,
			    GLclampf alpha )
{
    if( debuglevel > 0 ) {
	printf("glClearColor %f %f %f %f\n",
	       red, green, blue, alpha );
    }
    
    glClearColor( red, green, blue, alpha );

    _glClearColor.r = red;
    _glClearColor.g = green;
    _glClearColor.b = blue;
    _glClearColor.a = alpha;

    redraw();
}

void
FBControl::get_glClearColor(GLclampf *red,
			    GLclampf *green,
			    GLclampf *blue,
			    GLclampf *alpha )
{
    GLclampf	v[4];

    glGetFloatv( GL_COLOR_CLEAR_VALUE, v );

    *red = v[0];
    *green = v[1];
    *blue = v[2];
    *alpha = v[3];

    _glClearColor.r = *red;
    _glClearColor.g = *green;
    _glClearColor.b = *blue;
    _glClearColor.a = *alpha;
}

void
FBControl::set_glClearIndex( GLfloat c )
{
    if( debuglevel > 0 ) {
	printf("glClearIndex %f\n", c );
    }
    
    glClearIndex( c );

    _glClearIndex = c;

    redraw();
}

void
FBControl::get_glClearIndex( GLfloat *c )
{
    glGetFloatv( GL_INDEX_CLEAR_VALUE, c );

    _glClearIndex = *c;
}

void
FBControl::set_glClearDepth( GLclampd depth )
{
    if( debuglevel > 0 ) {
	printf("glClearDepth %lf\n", depth );
    }

    glClearDepth( depth );

    _glClearDepth = depth;

    redraw();
}

void
FBControl::get_glClearDepth( GLclampd *depth )
{
    glGetDoublev( GL_DEPTH_CLEAR_VALUE, depth );

    _glClearDepth = *depth;
}

void
FBControl::set_glClearStencil( GLint s )
{
    if( debuglevel > 0 ) {
	printf("glClearStencil 0x%x\n", s );
    }

    glClearStencil( s );

    _glClearStencil = s;

    redraw();
}

void
FBControl::get_glClearStencil( GLint *s )
{
    glGetIntegerv( GL_STENCIL_CLEAR_VALUE, s );

    _glClearStencil = *s;
}

void
FBControl::set_glDepthMask( GLboolean mask )
{
    if( debuglevel > 0 ) {
	if( mask ) {
	    printf("glDepthMask TRUE\n");
	} else {
	    printf("glDepthMask FALSE\n");
	}
    }

    glDepthMask( mask );

    _glDepthMask = mask;

    redraw();
}

void
FBControl::get_glDepthMask( GLboolean *mask )
{
    glGetBooleanv( GL_DEPTH_WRITEMASK, mask );

    _glDepthMask = *mask;
}

void
FBControl::set_glStencilMask( GLuint mask )
{
    if( debuglevel > 0 ) {
	printf("glStencilMask 0x%x\n", mask );
    }

    glStencilMask( mask );

    _glStencilMask = mask;

    redraw();
}

void
FBControl::get_glStencilMask( GLuint *mask )
{
    glGetIntegerv( GL_STENCIL_WRITEMASK, (int *) mask );

    _glStencilMask = *mask;
}

void
FBControl::set_glColorMask( GLboolean red, GLboolean green,
			   GLboolean blue, GLboolean alpha )
{
    if( debuglevel > 0 ) {
	char *(s[2]);
	s[0] = "FALSE";
	s[1] = "TRUE";

	printf("glColorMask %s %s %s %s\n",
	       s[red],
	       s[green],
	       s[blue],
	       s[alpha] );
    }

    glColorMask( red, green, blue, alpha );

    _glColorMask_red = red;
    _glColorMask_green = green;
    _glColorMask_blue = blue;
    _glColorMask_alpha = alpha;

    redraw();
}

void
FBControl::get_glColorMask( GLboolean *red, GLboolean *green,
			   GLboolean *blue, GLboolean *alpha )
{
    GLboolean v[4];

    glGetBooleanv( GL_COLOR_WRITEMASK, v );

    *red = v[0];
    *green = v[1];
    *blue = v[2];
    *alpha = v[3];

    _glColorMask_red = *red;
    _glColorMask_green = *green;
    _glColorMask_blue = *blue;
    _glColorMask_alpha = *alpha;
}

void
FBControl::set_glIndexMask( GLuint mask )
{
    if( debuglevel > 0 ) {
	printf("glIndexMask 0x%x\n", mask );
    }

    glIndexMask( mask );

    _glIndexMask = mask;

    redraw();
}

void
FBControl::get_glIndexMask( GLuint *mask )
{
    glGetIntegerv( GL_INDEX_WRITEMASK, (int *) mask );

    _glIndexMask = *mask;
}



void
FBControl::set_glDrawBuffer( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_NONE:
	    s = "GL_NONE";
	    break;
	  case GL_FRONT_LEFT:
	    s = "GL_FRONT_LEFT";
	    break;
	  case GL_FRONT_RIGHT:
	    s = "GL_FRONT_RIGHT";
	    break;
	  case GL_BACK_LEFT:
	    s = "GL_BACK_LEFT";
	    break;
	  case GL_BACK_RIGHT:
	    s = "GL_BACK_RIGHT";
	    break;
	  case GL_FRONT:
	    s = "GL_FRONT";
	    break;
	  case GL_BACK:
	    s = "GL_BACK";
	    break;
	  case GL_LEFT:
	    s = "GL_LEFT";
	    break;
	  case GL_RIGHT:
	    s = "GL_RIGHT";
	    break;
	  case GL_FRONT_AND_BACK:
	    s = "GL_FRONT_AND_BACK";
	    break;
	  case GL_AUX0:
	    s = "GL_AUX0";
	    break;
	}

	printf("glDrawBuffer %s\n", s );
    }
    
    glDrawBuffer( mode );

    _glDrawBuffer = mode;

    redraw();
}

void
FBControl::get_glDrawBuffer( GLenum *mode )
{
    glGetIntegerv( GL_DRAW_BUFFER, (int *) mode );

    _glDrawBuffer = *mode;
}

void
FBControl::reset()
{
    int	debugstack;
    Boolean redrawstack;
    
    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }

    set_glClearColor( _glClearColor.r, _glClearColor.g,
		     _glClearColor.b, _glClearColor.a );
    set_glClearIndex( _glClearIndex );
    set_glClearDepth( _glClearDepth );
    set_glClearStencil( _glClearStencil );
    set_glDepthMask( _glDepthMask );
    set_glStencilMask( _glStencilMask );
    if( glWindow->getRgbaMode() ) {
	set_glColorMask( _glColorMask_red, _glColorMask_green,
			_glColorMask_blue, _glColorMask_alpha );
    } else {
	set_glIndexMask( _glIndexMask );
    }
    set_glDrawBuffer( _glDrawBuffer );

    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}


///// callbacks

void
FBControl::cb_set_glClearColor( VkCallbackObject *,
			       void *client, void *call )
{
    FBControl *self = (FBControl *)client;
    RGBAColor	*c = (RGBAColor *) call;

    self->set_glClearColor( c->r, c->g, c->b, c->a );
}

void
FBControl::cb_set_glClearIndex( VkCallbackObject *,
			       void *client, void *call )
{
    FBControl *self = (FBControl *)client;
    GLfloat f = atof( (char *) call );

    self->set_glClearIndex( f );
}

void
FBControl::cb_set_glClearDepth( VkCallbackObject *,
			       void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    GLclampd depth = atof( (char *) call );

    self->set_glClearDepth( depth );
}

void
FBControl::cb_set_glClearStencil( VkCallbackObject *,
				 void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    GLint s = xtoi( (char *) call );

    self->set_glClearStencil( s );
}

void
FBControl::cb_set_glDepthMask( VkCallbackObject *,
			      void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int bool = (int) call;

    self->set_glDepthMask( bool );
}

void
FBControl::cb_set_glStencilMask( VkCallbackObject *,
				void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int mask = xtoi( (char *) call );

    self->set_glStencilMask( mask );
}

void
FBControl::cb_set_glColorMask_red( VkCallbackObject *,
				  void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int new_red = (int) call;

    GLboolean	red, green, blue, alpha;

    self->get_glColorMask( &red, &green, &blue, &alpha );
    self->set_glColorMask( new_red, green, blue, alpha );
}

void
FBControl::cb_set_glColorMask_green( VkCallbackObject *,
				  void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int new_green = (int) call;

    GLboolean	red, green, blue, alpha;

    self->get_glColorMask( &red, &green, &blue, &alpha );
    self->set_glColorMask( red, new_green, blue, alpha );
}

void
FBControl::cb_set_glColorMask_blue( VkCallbackObject *,
				  void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int new_blue = (int) call;

    GLboolean	red, green, blue, alpha;

    self->get_glColorMask( &red, &green, &blue, &alpha );
    self->set_glColorMask( red, green, new_blue, alpha );
}

void
FBControl::cb_set_glColorMask_alpha( VkCallbackObject *,
				  void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int new_alpha = (int) call;

    GLboolean	red, green, blue, alpha;

    self->get_glColorMask( &red, &green, &blue, &alpha );
    self->set_glColorMask( red, green, blue, new_alpha );
}

void
FBControl::cb_set_glIndexMask( VkCallbackObject *,
			      void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    int mask = (int) xtoi( (char *) call );

    self->set_glIndexMask( mask );
}

void
FBControl::cb_set_glDrawBuffer( VkCallbackObject *,
			       void *client, void *call )
{
    FBControl *self = (FBControl *) client;
    GLenum mode = (GLenum) call;

    self->set_glDrawBuffer( mode );
}
