/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _FBCONTROL_H
#define _FBCONTROL_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Fri Jan  7 18:23:16 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:15:12 1994 by George Kyriazis
 *
 * FrameBuffer control
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class FBControl : public IfaceGenerator {
  public:
    FBControl( GlWindow * );
    virtual ~FBControl();

    // clearing the FB
    void	set_glClearColor(GLclampf red,
				 GLclampf green,
				 GLclampf blue,
				 GLclampf alpha );
    void	get_glClearColor(GLclampf *red,
				 GLclampf *green,
				 GLclampf *blue,
				 GLclampf *alpha );
    void	set_glClearIndex( GLfloat c );
    void	get_glClearIndex( GLfloat *c );
    void	set_glClearDepth( GLclampd depth );
    void	get_glClearDepth( GLclampd *depth );
    void	set_glClearStencil( GLint s );
    void	get_glClearStencil( GLint *s );

    // Mask control
    void	set_glDepthMask( GLboolean mask );
    void	get_glDepthMask( GLboolean *mask );
    void	set_glStencilMask( GLuint mask );
    void	get_glStencilMask( GLuint *mask );
    void	set_glColorMask(GLboolean red,
				GLboolean green,
				GLboolean blue,
				GLboolean alpha );
    void	get_glColorMask(GLboolean *red,
				GLboolean *green,
				GLboolean *blue,
				GLboolean *alpha );
    void	set_glIndexMask( GLuint mask );
    void	get_glIndexMask( GLuint *mask );

    // FB control
    void	set_glDrawBuffer( GLenum mode );
    void	get_glDrawBuffer( GLenum *mode );

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // our state
    RGBAColor	_glClearColor;
    GLfloat     _glClearIndex;
    GLfloat	_glClearDepth;
    GLint	_glClearStencil;
    GLboolean	_glDepthMask;
    GLuint	_glStencilMask;
    GLboolean	_glColorMask_red;
    GLboolean	_glColorMask_green;
    GLboolean	_glColorMask_blue;
    GLboolean	_glColorMask_alpha;
    GLuint	_glIndexMask;
    GLenum	_glDrawBuffer;

    // callbacks!
    static	void	cb_set_glClearColor( CB_ARGS );
    static	void	cb_set_glClearIndex( CB_ARGS );
    static	void	cb_set_glClearDepth( CB_ARGS );
    static	void	cb_set_glClearStencil( CB_ARGS );
    static	void	cb_set_glDepthMask( CB_ARGS );
    static	void	cb_set_glStencilMask( CB_ARGS );
    static	void	cb_set_glColorMask_red( CB_ARGS );
    static	void	cb_set_glColorMask_green( CB_ARGS );
    static	void	cb_set_glColorMask_blue( CB_ARGS );
    static	void	cb_set_glColorMask_alpha( CB_ARGS );
    static	void	cb_set_glIndexMask( CB_ARGS );
    static	void	cb_set_glDrawBuffer( CB_ARGS );
};

#endif /* _FBCONTROL_H */
