/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _FRAGMENTOPS1_H
#define _FRAGMENTOPS1_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Fri Jan  7 19:08:31 1994 by George Kyriazis
 * Last Modified: Wed Aug 17 13:20:43 1994 by George Kyriazis
 *
 * Control of operations that are performed on a per-fragment basis
 *  this one has depth/dithering/alpha test/alpha blending
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class FragmentOps1 : public IfaceGenerator {
  public:
    FragmentOps1( GlWindow * );
    virtual ~FragmentOps1();

    // alpha test
    void	set_glAlphaFunc(GLenum func,
				GLclampf ref );
    void	get_glAlphaFunc(GLenum *func,
				GLclampf *ref );
    void	set_alpha_test( Boolean );
    void	get_alpha_test( Boolean * );

    // depth buffer testing
    void	set_glDepthFunc( GLenum func );
    void	get_glDepthFunc( GLenum *func );
    void	set_depth_test( Boolean );
    void	get_depth_test( Boolean * );
    void	set_glDepthRange( GLclampd near, GLclampd far );
    void	get_glDepthRange( GLclampd *near, GLclampd *far );
#ifdef OGL_EXTENSIONS
    void	set_glPolygonOffsetEXT( GLfloat factor, GLfloat bias );
    void	get_glPolygonOffsetEXT( GLfloat *factor, GLfloat *bias );
    void	set_polygon_offset_EXT( Boolean );
    void	get_polygon_offset_EXT( Boolean * );
#endif

    // dithering
    void	set_dither( Boolean );
    void	get_dither( Boolean * );

    // blending
    void	set_glBlendFunc( GLenum sfactor, GLenum dfactor );
    void	get_glBlendFunc( GLenum *sfactor, GLenum *dfactor );
    void	set_blend( Boolean );
    void	get_blend( Boolean * );

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // our state
    GLenum	_glAlphaFunc_func;
    GLclampf	_glAlphaFunc_ref;
    Boolean	_alpha_test;
    GLenum	_glDepthFunc;
    Boolean	_depth_test;
    GLclampd	_glDepthRange_near;
    GLclampd	_glDepthRange_far;
#ifdef OGL_EXTENSIONS
    GLfloat	_glPolygonOffsetEXT_factor;
    GLfloat	_glPolygonOffsetEXT_bias;
    Boolean	_polygon_offset_ext;
#endif
    Boolean	_dither;
    GLenum	_glBlendFunc_sfactor;
    GLenum	_glBlendFunc_dfactor;
    Boolean	_blend;

    
    // callbacks
    static	void	cb_set_depth_test( CB_ARGS );
    static	void	cb_set_glDepthFunc( CB_ARGS );
    static	void	cb_set_glDepthRange_near( CB_ARGS );
    static	void	cb_set_glDepthRange_far( CB_ARGS );
#ifdef OGL_EXTENSIONS
    static	void	cb_set_glPolygonOffsetEXT_factor( CB_ARGS );
    static	void	cb_set_glPolygonOffsetEXT_bias( CB_ARGS );
    static	void	cb_set_polygon_offset_ext( CB_ARGS );
#endif
    static	void	cb_set_dither( CB_ARGS );
    static	void	cb_set_alpha_test( CB_ARGS );
    static	void	cb_set_glAlphaFunc_func( CB_ARGS );
    static	void	cb_set_glAlphaFunc_ref( CB_ARGS );
    static	void	cb_set_glBlendFunc_sfactor( CB_ARGS );
    static	void	cb_set_glBlendFunc_dfactor( CB_ARGS );
    static	void	cb_set_blend( CB_ARGS );
};

#endif /* _FRAGMENTOPS1_H */
