/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _FRAGMENTOPS2_H
#define _FRAGMENTOPS2_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Fri Jan  7 19:08:31 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:14:51 1994 by George Kyriazis
 *
 * Control of operations that are performed on a per-fragment basis
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class FragmentOps2 : public IfaceGenerator {
  public:
    FragmentOps2( GlWindow * );
    virtual ~FragmentOps2();

    // scissoring
    void	set_glScissor(GLint x,
			      GLint y,
			      GLsizei width,
			      GLsizei height );
    void	get_glScissor(GLint *x,
			      GLint *y,
			      GLsizei *width,
			      GLsizei *height );
    void	set_scissor( Boolean );
    void	get_scissor( Boolean * );

    // stencils
    void	set_glStencilOp(GLenum fail,
				GLenum zfail,
				GLenum zpass );
    void	get_glStencilOp(GLenum *fail,
				GLenum *zfail,
				GLenum *zpass );
    void	set_glStencilFunc(GLenum func,
				  GLint ref,
				  GLuint mask );
    void	get_glStencilFunc(GLenum *func,
				  GLint *ref,
				  GLuint *mask );
    void	set_stencil_test( Boolean );
    void	get_stencil_test( Boolean * );

    // logic ops
    void	set_glLogicOp( GLenum opcode );
    void	get_glLogicOp( GLenum *opcode );
    void	set_logic_op( Boolean );
    void	get_logic_op( Boolean * );

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // our state
    GLint	_glScissor_x;
    GLint	_glScissor_y;
    GLint	_glScissor_width;
    GLint	_glScissor_height;
    Boolean	_scissor;
    GLenum	_glStencilOp_fail;
    GLenum	_glStencilOp_zfail;
    GLenum	_glStencilOp_zpass;
    GLenum	_glStencilFunc_func;
    GLint	_glStencilFunc_ref;
    GLuint	_glStencilFunc_mask;
    Boolean	_stencil_test;
    GLenum	_glLogicOp;
    Boolean	_logic_op;

    
    // callbacks
    static	void	cb_set_scissor( CB_ARGS );
    static	void	cb_set_scissor_x( CB_ARGS );
    static	void	cb_set_scissor_y( CB_ARGS );
    static	void	cb_set_scissor_width( CB_ARGS );
    static	void	cb_set_scissor_height( CB_ARGS );
    static	void	cb_set_glStencilOp_fail( CB_ARGS );
    static	void	cb_set_glStencilOp_zfail( CB_ARGS );
    static	void	cb_set_glStencilOp_zpass( CB_ARGS );
    static	void	cb_set_glStencilFunc_func( CB_ARGS );
    static	void	cb_set_glStencilFunc_ref( CB_ARGS );
    static	void	cb_set_glStencilFunc_mask( CB_ARGS );
    static	void	cb_set_stencil_test( CB_ARGS );
    static	void	cb_set_glLogicOp( CB_ARGS );
    static	void	cb_set_logic_op( CB_ARGS );
};

#endif /* _FRAGMENTOPS2_H */
