/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Tue Jul 12 12:25:06 1994 by George Kyriazis
 * Last Modified: Tue Jul 12 13:40:40 1994 by George Kyriazis
 *
 * how to render a cube
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#define private protected
#include "GlCube.h"
#undef private

#include <Inventor/actions/SoGLRenderAction.h>


SO_NODE_SOURCE(GlCube);

GlCube::GlCube()
{
    SO_NODE_CONSTRUCTOR(GlCube);
    isBuiltIn = TRUE;
}

GlCube::~GlCube()
{
}


void
GlCube::initClass()
{
    classTypeId = SoType::overrideType(SoCube::getClassTypeId(),
				       &GlCube::createInstance);
    parentFieldData = SoCube::getFieldDataPtr();
}


void
GlCube::GLRender(SoGLRenderAction *action)
{
    SbBool	unitSize;
    float	w, h, d;

    // First see if the object is visible and should be rendered now
    if (! shouldGLRender(action))
	return;

    // Scale world if necessary
    getSize(w, h, d);
    unitSize = (w == 1.0 && h == 1.0 && d == 1.0);
    if (! unitSize) {
	glPushMatrix();
	glScalef(w, h, d);
    }

    // Render the cube. 
    GLRenderGeneric(action);

    if (! unitSize)
	glPopMatrix();
}

void
GlCube::GLRenderGeneric(SoGLRenderAction *action)
{
    int			numDivisions, face, vert;
    float		s;
    SbVec3f		pt, norm;

    numDivisions    = computeNumDivisions(action);

    if (numDivisions == 1)
	glBegin(GL_QUADS);

    for (face = 0; face < 6; face++) {

	glNormal3fv(normals[face].getValue());

	// Simple case of one polygon per face 
	if (numDivisions == 1) {
	    for (vert = 0; vert < 4; vert++) {
		glTexCoord2fv(texCoords[vert].getValue());
		glVertex3fv(verts[face][vert]->getValue());
	    }
	}

	// More than one polygon per face
	else {
	    float	di = 1.0 / numDivisions;
	    SbVec3f	topPoint,    botPoint,    nextBotPoint;
	    SbVec3f	horizSpace, vertSpace;
	    int		strip, rect;

	    botPoint = *verts[face][0];

	    // Compute spacing between adjacent points in both directions
	    horizSpace = di * (*verts[face][1] - botPoint);
	    vertSpace  = di * (*verts[face][3] - botPoint);

	    // For each horizontal strip
	    for (strip = 0; strip < numDivisions; strip++) {

		// Compute current top point. Save it to use as bottom
		// of next strip
		nextBotPoint = topPoint = botPoint + vertSpace;

		glBegin(GL_TRIANGLE_STRIP);

		// Send points at left end of strip
		s = 0.0;
		glTexCoord2f(s, (strip + 1) * di);
		glVertex3fv(topPoint.getValue());
		glTexCoord2f(s, strip * di);
		glVertex3fv(botPoint.getValue());

		// For each rectangular piece of strip
		for (rect = 0; rect < numDivisions; rect++) {

		    // Go to next rect
		    topPoint += horizSpace;
		    botPoint += horizSpace;
		    s += di;

		    // Send points at right side of rect
		    glTexCoord2f(s, (strip + 1) * di);
		    glVertex3fv(topPoint.getValue());
		    glTexCoord2f(s, strip * di);
		    glVertex3fv(botPoint.getValue());
		}

		glEnd();

		// Get ready for next strip
		botPoint = nextBotPoint;
	    }
	}
    }

    if (numDivisions == 1)
	glEnd();
}
