/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Wed Jan 26 14:48:32 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:12:51 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

// need to have access!
#define private protected
#include "GlSceneMgr.h"

#include <GL/gl.h>
#include <Inventor/sensors/SoNodeSensor.h>
#include "GlRenderAction.h"

GlSceneManager::GlSceneManager( SbBool useCurrentGlValues )
: SoSceneManager( useCurrentGlValues )
{
    graphicsInitNeeded = TRUE;

    SbViewportRegion theRegion = getGLRenderAction()->getViewportRegion();
    setGLRenderAction( new GlRenderAction( theRegion ) );
}

GlSceneManager::~GlSceneManager()
{
}


////////////////////////////////////////////////////////////////////////

void
GlSceneManager::render( SbBool clearWindow, SbBool clearZbuffer )
{
    SbBool	needZbuffer;

    // reinitialize if necessary
    if (graphicsInitNeeded) {
	int numBits[1];
	glGetIntegerv(GL_DEPTH_BITS, numBits);
	needZbuffer = (numBits[0] != 0);
	
	graphicsInitNeeded = FALSE;
    }

    // get the width and height
    const SbViewportRegion &theRegion =
	getGLRenderAction()->getViewportRegion();
    SbVec2s size   = theRegion.getViewportSizePixels();
    SbVec2s origin = theRegion.getViewportOriginPixels();
    
    glViewport(origin[0], origin[1], size[0], size[1]);
    
    // clear buffers
    if (clearWindow) {
	// no need to set the color.
	glClear(GL_COLOR_BUFFER_BIT);
    }
    if (needZbuffer) {
	if (clearZbuffer) glClear(GL_DEPTH_BUFFER_BIT);
    }
    
    // render the scene graph!
    if (getSceneGraph() != NULL)
    	getGLRenderAction()->apply( getSceneGraph() );
    
    // sensor doesn't need to fire again if it's still scheduled
    sceneSensor->unschedule();
}
