/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _GLWINDOW_H
#define _GLWINDOW_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Wed Jan 26 11:09:49 1994 by George Kyriazis
 * Last Modified: Wed Sep 21 19:00:14 1994 by George Kyriazis
 *
 * the toplevel class containing all the connection and toplevel objects
 *
 ************************************************************************/

#ident "$Revision: 1.1 $ SGI"

#include "Object.h"

// X defines Object
#define Object XObject
#include <X11/Intrinsic.h>
#undef Object

class GlSceneManager;
class FBControl;
class FragmentOps1;
class FragmentOps2;
class TextureOps;
class TexGenOps;
class FAOps;
class RasterOps;
class LightAndShade;
class MaterialProps;
class Hints;
class ModelClipPlanes;
class Transforms;

#include <Vk/VkWindow.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include "SoXtBitmapButton.h"
#include <Inventor/Xt/SoXtIcons.h>
class SoTransformManip;
class SoSelection;
class SoMaterial;
#include <Inventor/actions/SoCallbackAction.h>
class SoPerspectiveCamera;
class SoOrthographicCamera;
class SoCallback;

class GlWindow : public Object, public VkWindow, public SoXtExaminerViewer {
  public:
    GlWindow( VkApp *, char *name );
    virtual ~GlWindow();

    // set the scene graph
    virtual	void	setSceneGraph( SoNode * );

    // set a camera
    virtual	void	setCamera( SoCamera * );

    // grab a handle to a plane where we should put additional geometry
    SoGroup	*get_pre_extender_node() { return preExtenderNode; }
    SoGroup	*get_post_extender_node() { return postExtenderNode; }

    // set different modes
    virtual	void	setRgbaMode( Boolean );
    virtual	void	setDbufMode( Boolean );
    virtual	void	setCiDepth( int );
    virtual	void	setVisualType( Boolean rgbaMode,
				      Boolean dbufMode,
				      int ciDepth = 8,
				      int rDepth = 1,
				      int gDepth = 1,
				      int bDepth = 1,
				      int aDepth = 0,
				      int zDepth = 1,
				      int sDepth = 1,
				      int accumRDepth = 1,
				      int accumGDepth = 1,
				      int accumBDepth = 1,
				      int accumADepth = 1,
				      Boolean stereo = FALSE
				      );
    Boolean	getRgbaMode() { return rgbaMode; }
    virtual	void	setUpdateMode( Boolean ); // how often to update

    // set the various transforms
    virtual	void	set_model_view( GLfloat * );

    // deal with files
    virtual	void	deleteScene();
    virtual	void	readFile( char * );

    // if we show frames per second
    virtual	void	showFPS( Boolean );

    // WM stuff
    virtual	void	show();
    virtual	void	hide();

  protected:
    // GUI things
    Widget	fileDialog;
    
    void	createMenuBar();
    void	createVisualMenu();
    virtual	Widget	setUpInterface( Widget parent );
    virtual	void	afterRealizeHook();
    Boolean	not_realized;

    static	void	openCB( Widget, XtPointer, XtPointer );
    static	void	quitCB( Widget, XtPointer, XtPointer );

    static	void	setManipCB( Widget, XtPointer, XtPointer );

    void	getFileName();
    static	void	fileDialogCB( Widget, XtPointer, XtPointer );

    // scene node stuff
    SoSelection	*toplevelNode;
    SoNode	*localSceneGraph;
    SoGroup	*preExtenderNode;
    SoGroup	*postExtenderNode;
    SoCallback	*cameraCbNode;

    GlSceneManager *sceneManager;

    // rgba or ci modes, etc.
    Boolean	rgbaMode;
    Boolean	dbufMode;
    int		ciDepth;
    int		rDepth, gDepth, bDepth, aDepth;
    int		zDepth;
    int		sDepth;
    int		accumRDepth, accumGDepth, accumBDepth, accumADepth;
    Boolean	stereoMode;
    Boolean	updateMode;
    XVisualInfo *vislist;
    XVisualInfo	*vis;
    virtual	void	setNormalVisual( XVisualInfo * );
    virtual	void	buildColormap();

    virtual	void	actualRedraw();

    // stuff for displaying frames per second
    Boolean	showFPSmode;
    virtual	void	displayFPS( float fps );

    void	alterGlAttribList();
    void	buildVisual();
    int		renderToPixmap();

    void	alterGraph( SoNode *, Boolean verbose = FALSE );

    virtual void	createViewerButtons( Widget );
    static void		pushButtonCB(Widget, int id, void *);

    // manipulator things
    static	void	selectionCallback( void *, SoPath * );
    static	void	deselectionCallback( void *, SoPath * );
    static	SoPath*	pickFilterCallback( void *, const SoPickedPoint * );
    SoPath *	findTransformForAttach( const SoPath * );
    SbBool	isAffectedByTransform( SoNode * );
    SoMaterial*	findMaterialForAttach( const SoPath * );
    static SoCallbackAction::Response
	findMtlPreTailCB( void *, SoCallbackAction *, const SoNode * );
    SbBool	isAffectedByMaterial( SoNode * );
    void	attachManip( SoPath * );
    void	detachManip( SoPath * );
    SoTransformManip	*allocateManipOfType( int );
    
    SoTransformManip	*manip;	// the manip
    SoPath		*xfPath;// its path in the graph
    int			currentManipType;
    SoXtBitmapButton	**buttonList;	// for the buttons on the right

    // popups
    FBControl	*fbControl;
    FragmentOps1*fragmentOps1;
    FragmentOps2*fragmentOps2;
    TextureOps	*textureOps;
    TexGenOps	*texGenOps;
    FAOps	*faOps;
    RasterOps	*rasterOps;
    LightAndShade *lightAndShade;
    MaterialProps *materialProps;
    Hints	*hints;
    ModelClipPlanes *modelClipPlanes;
    Transforms	*transforms;
    
    void	createPopups();

    static	void	popupCB( Widget, XtPointer, XtPointer );

    // the different types of view matrices
    SoPerspectiveCamera	*perspectiveCamera;
    SoOrthographicCamera*orthographicCamera;
    static	void	cameraCB( Widget, XtPointer, XtPointer );
    static	void	grabCameraCB( void *, SoAction * );

    // visuals menu
    static	void	visualCB( Widget, XtPointer, XtPointer );

    // callbacks for quit events
    virtual	void	handleWmDeleteMessage();
    virtual	void	handleWmQuitMessage();
};

#endif /* _GLWINDOW_H */
