/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Feb 17 18:04:08 1994 by George Kyriazis
 * Last Modified: Fri Sep  9 15:21:30 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "Hints.h"

#include "IfaceGeneratorP.h"

int Hints::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Hints", 1,

    IF_RADIO, IF_THIS, NULL,
    (int) "Fog Hint: ", 3,
    IF_VALUE+0, (int) &Hints::cb_set_glHint_fog,
    (int) "GL_FASTEST", GL_FASTEST,
    (int) "GL_NICEST", GL_NICEST,
    (int) "GL_DONT_CARE", GL_DONT_CARE,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Point Smooth Hint: ", 3,
    IF_VALUE+1, (int) &Hints::cb_set_glHint_point_smooth,
    (int) "GL_FASTEST", GL_FASTEST,
    (int) "GL_NICEST", GL_NICEST,
    (int) "GL_DONT_CARE", GL_DONT_CARE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Line Smooth Hint: ", 3,
    IF_VALUE+2, (int) &Hints::cb_set_glHint_line_smooth,
    (int) "GL_FASTEST", GL_FASTEST,
    (int) "GL_NICEST", GL_NICEST,
    (int) "GL_DONT_CARE", GL_DONT_CARE,

    IF_RADIO, IF_THIS, NULL,
    (int) "Polygon Smooth Hint: ", 3,
    IF_VALUE+3, (int) &Hints::cb_set_glHint_polygon_smooth,
    (int) "GL_FASTEST", GL_FASTEST,
    (int) "GL_NICEST", GL_NICEST,
    (int) "GL_DONT_CARE", GL_DONT_CARE,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Perspective Correction Hint: ", 3,
    IF_VALUE+4, (int) &Hints::cb_set_glHint_perspective_correction,
    (int) "GL_FASTEST", GL_FASTEST,
    (int) "GL_NICEST", GL_NICEST,
    (int) "GL_DONT_CARE", GL_DONT_CARE,

    IF_END
};


Hints::Hints( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();
}

Hints::~Hints()
{
}


///////////////////////////////////////////////////////////////////

static int
fix_my_stupid_enum( GLenum mode )
{
    switch( mode ) {
      case GL_FASTEST:
	return 0;
      case GL_NICEST:
	return 1;
    }

    // anything else
    return 2;
}

void
Hints::install_values()
{
    GLenum	mode;

    get_glHint_fog( &mode );
    install( iface, IF_VALUE+0, fix_my_stupid_enum( mode ) );

    get_glHint_point_smooth( &mode );
    install( iface, IF_VALUE+1, fix_my_stupid_enum( mode ) );

    get_glHint_line_smooth( &mode );
    install( iface, IF_VALUE+2, fix_my_stupid_enum( mode ) );

    get_glHint_polygon_smooth( &mode );
    install( iface, IF_VALUE+3, fix_my_stupid_enum( mode ) );

    get_glHint_perspective_correction( &mode );
    install( iface, IF_VALUE+4, fix_my_stupid_enum( mode ) );
}

void
Hints::remember()
{
    GLenum	e;

    get_glHint_fog( &e );
    get_glHint_point_smooth( &e );
    get_glHint_line_smooth( &e );
    get_glHint_polygon_smooth( &e );
    get_glHint_perspective_correction( &e );
}



void
Hints::print_hint_type( GLenum mode )
{
    switch( mode ) {
      case GL_FASTEST:
	printf("GL_FASTEST\n");
	break;
      case GL_NICEST:
	printf("GL_NICEST\n");
	break;
      case GL_DONT_CARE:
	printf("GL_DONT_CARE\n");
	break;
    }
}

void
Hints::set_glHint_fog( GLenum mode )
{
    if( debuglevel > 0 ) {
	printf("glHint GL_FOG_HINT " );
	print_hint_type( mode );
    }

    glHint( GL_FOG_HINT, mode );

    _glHint_fog = mode;

    redraw();
}

void
Hints::get_glHint_fog( GLenum *mode )
{
    glGetIntegerv( GL_FOG_HINT, (int *) mode );

    _glHint_fog = *mode;
}


void
Hints::set_glHint_point_smooth( GLenum mode )
{
    if( debuglevel > 0 ) {
	printf("glHint GL_POINT_SMOOTH_HINT " );
	print_hint_type( mode );
    }

    glHint( GL_POINT_SMOOTH_HINT, mode );

    _glHint_point_smooth = mode;

    redraw();
}

void
Hints::get_glHint_point_smooth( GLenum *mode )
{
    glGetIntegerv( GL_POINT_SMOOTH_HINT, (int *) mode );

    _glHint_point_smooth = *mode;
}


void
Hints::set_glHint_line_smooth( GLenum mode )
{
    if( debuglevel > 0 ) {
	printf("glHint GL_LINE_SMOOTH_HINT " );
	print_hint_type( mode );
    }

    glHint( GL_LINE_SMOOTH_HINT, mode );

    _glHint_line_smooth = mode;

    redraw();
}

void
Hints::get_glHint_line_smooth( GLenum *mode )
{
    glGetIntegerv( GL_LINE_SMOOTH_HINT, (int *) mode );

    _glHint_line_smooth = *mode;
}


void
Hints::set_glHint_polygon_smooth( GLenum mode )
{
    if( debuglevel > 0 ) {
	printf("glHint GL_POLYGON_SMOOTH_HINT " );
	print_hint_type( mode );
    }

    glHint( GL_POLYGON_SMOOTH_HINT, mode );

    _glHint_polygon_smooth = mode;

    redraw();
}

void
Hints::get_glHint_polygon_smooth( GLenum *mode )
{
    glGetIntegerv( GL_POLYGON_SMOOTH_HINT, (int *) mode );

    _glHint_polygon_smooth = *mode;
}


void
Hints::set_glHint_perspective_correction( GLenum mode )
{
    if( debuglevel > 0 ) {
	printf("glHint GL_PERSPECTIVE_CORRECTION_HINT " );
	print_hint_type( mode );
    }

    glHint( GL_PERSPECTIVE_CORRECTION_HINT, mode );

    _glHint_perspective_correction = mode;

    redraw();
}

void
Hints::get_glHint_perspective_correction( GLenum *mode )
{
    glGetIntegerv( GL_PERSPECTIVE_CORRECTION_HINT, (int *) mode );

    _glHint_perspective_correction = *mode;
}


void
Hints::reset()
{
    int	debugstack;
    Boolean redrawstack;

    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }

    set_glHint_fog( _glHint_fog );
    set_glHint_point_smooth( _glHint_point_smooth );
    set_glHint_line_smooth( _glHint_line_smooth );
    set_glHint_polygon_smooth( _glHint_polygon_smooth );
    set_glHint_perspective_correction( _glHint_perspective_correction );

    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}


/////  callbacks

void
Hints::cb_set_glHint_fog( VkCallbackObject *, void *client, void *call )
{
    Hints *self = (Hints *) client;
    int	mode = (int) call;

    self->set_glHint_fog( mode );
}

void
Hints::cb_set_glHint_point_smooth( VkCallbackObject *,
				  void *client, void *call )
{
    Hints *self = (Hints *) client;
    int	mode = (int) call;

    self->set_glHint_point_smooth( mode );
}

void
Hints::cb_set_glHint_line_smooth( VkCallbackObject *,
				  void *client, void *call )
{
    Hints *self = (Hints *) client;
    int	mode = (int) call;

    self->set_glHint_line_smooth( mode );
}

void
Hints::cb_set_glHint_polygon_smooth( VkCallbackObject *,
				  void *client, void *call )
{
    Hints *self = (Hints *) client;
    int	mode = (int) call;

    self->set_glHint_polygon_smooth( mode );
}

void
Hints::cb_set_glHint_perspective_correction( VkCallbackObject *,
				  void *client, void *call )
{
    Hints *self = (Hints *) client;
    int	mode = (int) call;

    self->set_glHint_perspective_correction( mode );
}
