/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Wed Jan 12 16:31:26 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:15:18 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

// X defines Object
#define Object XObject
#include <X11/Intrinsic.h>
#undef Object

#include <stdlib.h>

#include "IfaceGenerator.h"

#include "IfaceGeneratorP.h"

#include "TopLevelWindow.h"
#include "RadioBoxItem.h"
#include "TextItem.h"
#include "ColorChooser.h"
#include "ColorChooserWAlpha.h"
#include "FileName.h"
#include "PlaneItem.h"
#include "MatrixItem.h"
#include "SeparatorItem.h"

#include "GlWindow.h"

IfaceGenerator::IfaceGenerator( GlWindow *dwa )
{
    glWindow = dwa;
}

IfaceGenerator::~IfaceGenerator()
{
}

void
IfaceGenerator::install_this( int *script )
{
    install( script, IF_THIS, get_this() );
}

void
IfaceGenerator::install( int *script, int token, int repl )
{
    while( *script != IF_END ) {
	if( *script == token ) {
	    *script = repl;
	}
	script++;
    }
}


void
IfaceGenerator::redraw()
{
    if( redrawStatus ) {
	glWindow->scheduleRedraw();
    }
}

void
IfaceGenerator::setRedraw( Boolean state )
{
    redrawStatus = state;
}


void
IfaceGenerator::parse( int *script )
{
    int	*s = script;
    Boolean	go_ahead = TRUE;

    if( *s != IF_BEGIN ) {
	error("Could not find IF_BEGIN");
	return;
    }
    s++;

    while( go_ahead ) {
	switch( *s ) {
	  case IF_WINDOW:
	    {
		s++;
		s++;	// ignore "this".
		TopLevelWindow **handle = (TopLevelWindow **) *s++;
		toplevel = new TopLevelWindow( (char *) *s++ );
		int	ncols = (int) *s++;

		toplevel->set_columns( ncols );
		
		if( handle ) {
		    *handle = toplevel;
		}
	    }
	    break;
	    
	  case IF_RADIO:
	    {
		s++;
		int	i;
		RadioBoxItem *rb;

		void	*self = (void *) *s++;
		RadioBoxItem	**handle = (RadioBoxItem **) *s++;
		char *name = (char *) *s++;
		int	n = *s++;
		int	orig = *s++;
		VkCallbackFunction func = (VkCallbackFunction) *s++;
		
		rb = new RadioBoxItem( name, *toplevel, n );
		rb->set_initial_value( orig );

		rb->addCallback( RadioBoxItem::newValue, func, self );
		
		for( i = 0; i < n; i++ ) {
		    char *name = (char *) *s++;
		    int	ret = *s++;
		    
		    rb->set( i, name, ret );
		}
		rb->end();

		if( handle ) {
		    *handle = rb;
		}
	    }
	    break;

	  case IF_TEXT:
	    {
		s++;

		TextItem *ti;

		void	*self = (void *) *s++;
		TextItem **handle = (TextItem **) *s++;
		char *name = (char *) *s++;
		int	size = *s++;
		char *initial = (char *) *s++;
		char **accept = (char **) *s++;
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		ti = new TextItem( name, *toplevel, size );
		// have to first set the accepted string, and then
		// set the text, otherwise it won't get accepted.
		ti->set_accepted( *accept );
		ti->set_text( initial );

		ti->addCallback( TextItem::newValue, func, self );

		if( handle ) {
		    *handle = ti;
		}
	    }
	    break;

	  case IF_RGB:
	    {
		s++;

		ColorChooser	*cc;

		void *self = (void *) *s++;
		ColorChooser	**handle = (ColorChooser **) *s++;
		char *name = (char *) *s++;
		GLfloat r = atof( (char *) *s++ );
		GLfloat g = atof( (char *) *s++ );
		GLfloat b = atof( (char *) *s++ );
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		cc = new ColorChooser( name, *toplevel, glWindow );

		cc->setValue( r, g, b );
		cc->addCallback( ColorChooser::newValue, func, self );

		if( handle ) {
		    *handle = cc;
		}
	    }
	    break;

	  case IF_RGBA:
	    {
		s++;

		ColorChooserWAlpha	*cc;

		void *self = (void *) *s++;
		ColorChooserWAlpha **handle = (ColorChooserWAlpha **) *s++;
		char *name = (char *) *s++;
		GLfloat r = atof( (char *) *s++ );
		GLfloat g = atof( (char *) *s++ );
		GLfloat b = atof( (char *) *s++ );
		GLfloat a = atof( (char *) *s++ );
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		cc = new ColorChooserWAlpha( name, *toplevel, glWindow );

		cc->setValue( r, g, b, a );
		cc->addCallback( ColorChooserWAlpha::newValue, func, self );

		if( handle ) {
		    *handle = cc;
		}
	    }
	    break;

	  case IF_FNAME:
	    {
		s++;

		FileName	*fn;

		void	*self = (void *) *s++;
		FileName	**handle = (FileName **) *s++;
		char *name = (char *) *s++;
		char *acc = (char *) *s++;
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		fn = new FileName( name, *toplevel );

		fn->set_pattern_mask( acc );
		fn->addCallback( FileName::newValue, func, self );

		if( handle ) {
		    *handle = fn;
		}
	    }
	    break;

	  case IF_PLANE:
	    {
		s++;

		PlaneItem	*pi;

		void	*self = (void *) *s++;
		PlaneItem **handle = (PlaneItem **) *s++;
		char *name = (char *) *s++;
		float	a = atof( (char *) *s++ );
		float	b = atof( (char *) *s++ );
		float	c = atof( (char *) *s++ );
		float	d = atof( (char *) *s++ );
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		pi = new PlaneItem( name, *toplevel, glWindow );

		pi->set_plane( a, b, c, d );
		pi->addCallback( PlaneItem::newValue, func, self );

		if( handle ) {
		    *handle = pi;
		}
	    }
	    break;

	  case IF_MATRIX:
	    {
		s++;

		MatrixItem	*pi;

		void	*self = (void *) *s++;
		MatrixItem **handle = (MatrixItem **) *s++;
		char *name = (char *) *s++;
		float	a[16];
		a[0] = atof( (char *) *s++ );
		a[1] = atof( (char *) *s++ );
		a[2] = atof( (char *) *s++ );
		a[3] = atof( (char *) *s++ );
		a[4] = atof( (char *) *s++ );
		a[5] = atof( (char *) *s++ );
		a[6] = atof( (char *) *s++ );
		a[7] = atof( (char *) *s++ );
		a[8] = atof( (char *) *s++ );
		a[9] = atof( (char *) *s++ );
		a[10] = atof( (char *) *s++ );
		a[11] = atof( (char *) *s++ );
		a[12] = atof( (char *) *s++ );
		a[13] = atof( (char *) *s++ );
		a[14] = atof( (char *) *s++ );
		a[15] = atof( (char *) *s++ );
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		pi = new MatrixItem( name, *toplevel );

		pi->set_matrix( a );
		pi->addCallback( MatrixItem::newValue, func, self );

		if( handle ) {
		    *handle = pi;
		}
	    }
	    break;

	  case IF_SEPARATOR:
	    {
		s++;

		SeparatorItem	*pi;

		void *self = (void *) *s++;
		SeparatorItem **handle = (SeparatorItem **) *s++;
		char *name = (char *) *s++;

		pi = new SeparatorItem( name, *toplevel );

		if( handle ) {
		    *handle = pi;
		}
	    }
	    break;
	    
	  case IF_MULTI_TEXT:
	    {
		s++;

		TextItem *ti;

		void	*self = (void *) *s++;
		TextItem **handle = (TextItem **) *s++;
		char *name = (char *) *s++;
		int	cols = *s++;
		int	rows = *s++;
		char *initial = (char *) *s++;
		char **accept = (char **) *s++;
		VkCallbackFunction func = (VkCallbackFunction) *s++;

		ti = new TextItem( name, *toplevel, cols, rows );
		// have to first set the accepted string, and then
		// set the text, otherwise it won't get accepted.
		ti->set_accepted( *accept );
		ti->set_text( initial );

		ti->addCallback( TextItem::newValue, func, self );

		if( handle ) {
		    *handle = ti;
		}
	    }
	    break;

	  case IF_END:
// 	    toplevel->show();
	    go_ahead = FALSE;
	    break;
	}
    }
}


