/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _LIGHTANDSHADE_H
#define _LIGHTANDSHADE_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Fri Jan 21 11:46:06 1994 by George Kyriazis
 * Last Modified: Mon Aug  1 19:01:10 1994 by George Kyriazis
 *
 * Lighting and shading ops
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class RadioBoxItem;
class ColorChooserWAlpha;
class TextItem;

class LightAndShade : public IfaceGenerator {
  public:
    LightAndShade( GlWindow * );
    virtual ~LightAndShade();

    // shading
    void	set_glShadeModel( GLenum mode );
    void	get_glShadeModel( GLenum *mode );

    // lighting model
    void	set_glLightModel_ambient(GLfloat r,
					 GLfloat g,
					 GLfloat b,
					 GLfloat a );
    void	get_glLightModel_ambient(GLfloat *r,
					 GLfloat *g,
					 GLfloat *b,
					 GLfloat *a );
    void	set_glLightModel_local_viewer( GLboolean state );
    void	get_glLightModel_local_viewer( GLboolean *state );
    void	set_glLightModel_two_side( GLboolean state );
    void	get_glLightModel_two_side( GLboolean *state );

    // lighting
    void	set_glFrontFace( GLenum mode );
    void	get_glFrontFace( GLenum *mode );
    void	set_lighting( Boolean );
    void	get_lighting( Boolean * );

    // light parameters
    void	set_light_status( int lno, GLboolean state );
    void	get_light_status( int lno, GLboolean *state );
    void	set_glLight_ambient(int lno,
				    GLfloat r,
				    GLfloat g,
				    GLfloat b,
				    GLfloat a );
    void	get_glLight_ambient(int lno,
				    GLfloat *r,
				    GLfloat *g,
				    GLfloat *b,
				    GLfloat *a );
    void	set_glLight_diffuse(int lno,
				    GLfloat r,
				    GLfloat g,
				    GLfloat b,
				    GLfloat a );
    void	get_glLight_diffuse(int lno,
				    GLfloat *r,
				    GLfloat *g,
				    GLfloat *b,
				    GLfloat *a );
    void	set_glLight_specular(int lno,
				     GLfloat r,
				     GLfloat g,
				     GLfloat b,
				     GLfloat a );
    void	get_glLight_specular(int lno,
				     GLfloat *r,
				     GLfloat *g,
				     GLfloat *b,
				     GLfloat *a );
    void	set_glLight_position(int lno,
				     GLfloat x,
				     GLfloat y,
				     GLfloat z,
				     GLfloat w );
    void	get_glLight_position(int lno,
				     GLfloat *x,
				     GLfloat *y,
				     GLfloat *z,
				     GLfloat *w );
    void	set_glLight_spot_direction(int lno,
					   GLfloat x,
					   GLfloat y,
					   GLfloat z );
    void	get_glLight_spot_direction(int lno,
					   GLfloat *x,
					   GLfloat *y,
					   GLfloat *z );
    void	set_glLight_spot_exp( int lno, GLfloat exp );
    void	get_glLight_spot_exp( int lno, GLfloat *exp );
    void	set_glLight_spot_cutoff( int lno, GLfloat ang );
    void	get_glLight_spot_cutoff( int lno, GLfloat *ang );
    void	set_glLight_const_atten( int lno, GLfloat fac );
    void	get_glLight_const_atten( int lno, GLfloat *fac );
    void	set_glLight_linear_atten( int lno, GLfloat fac );
    void	get_glLight_linear_atten( int lno, GLfloat *fac );
    void	set_glLight_quad_atten( int lno, GLfloat fac );
    void	get_glLight_quad_atten( int lno, GLfloat *fac );
    
    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    int	current_light;

    // local state
    GLenum	_glShadeModel;
    RGBAColor	_glLightModel_ambient;
    GLboolean	_glLightModel_local_viewer;
    GLboolean	_glLightModel_two_side;
    GLenum	_glFrontFace;
    Boolean	_lighting;

#define	MAXNUM_L	8
    GLboolean	_light_status[MAXNUM_L];
    RGBAColor	_glLight_ambient[MAXNUM_L];
    RGBAColor	_glLight_diffuse[MAXNUM_L];
    RGBAColor	_glLight_specular[MAXNUM_L];
    GLfloat	_glLight_position_x[MAXNUM_L];
    GLfloat	_glLight_position_y[MAXNUM_L];
    GLfloat	_glLight_position_z[MAXNUM_L];
    GLfloat	_glLight_position_w[MAXNUM_L];
    GLfloat	_glLight_spot_direction_x[MAXNUM_L];
    GLfloat	_glLight_spot_direction_y[MAXNUM_L];
    GLfloat	_glLight_spot_direction_z[MAXNUM_L];
    GLfloat	_glLight_spot_exp[MAXNUM_L];
    GLfloat	_glLight_spot_cutoff[MAXNUM_L];
    GLfloat	_glLight_const_atten[MAXNUM_L];
    GLfloat	_glLight_linear_atten[MAXNUM_L];
    GLfloat	_glLight_quad_atten[MAXNUM_L];

    // callbacks
    static	void	cb_set_glShadeModel( CB_ARGS );
    static	void	cb_set_glLightModel_ambient( CB_ARGS );
    static	void	cb_set_glLightModel_local_viewer( CB_ARGS );
    static	void	cb_set_glLightModel_two_side( CB_ARGS );
    static	void	cb_set_glFrontFace( CB_ARGS );
    static	void	cb_set_lighting( CB_ARGS );
    static	void	cb_set_light_status( CB_ARGS );
    static	void	cb_set_glLight_ambient( CB_ARGS );
    static	void	cb_set_glLight_diffuse( CB_ARGS );
    static	void	cb_set_glLight_specular( CB_ARGS );
    static	void	cb_set_glLight_position( CB_ARGS );
    static	void	cb_set_glLight_spot_direction( CB_ARGS );
    static	void	cb_set_glLight_spot_exp( CB_ARGS );
    static	void	cb_set_glLight_spot_cutoff( CB_ARGS );
    static	void	cb_set_glLight_const_atten( CB_ARGS );
    static	void	cb_set_glLight_linear_atten( CB_ARGS );
    static	void	cb_set_glLight_quad_atten( CB_ARGS );

    static	void	cb_set_current_light( CB_ARGS );

    // widget handles
    RadioBoxItem	*enable_thing;
    ColorChooserWAlpha	*ambient_color_thing;
    ColorChooserWAlpha	*diffuse_color_thing;
    ColorChooserWAlpha	*specular_color_thing;
    TextItem		*light_position_thing;
    TextItem		*spot_direction_thing;
    TextItem		*spot_exp_thing;
    TextItem		*spot_cutoff_thing;
    TextItem		*const_atten_thing;
    TextItem		*linear_atten_thing;
    TextItem		*quad_atten_thing;
};

#endif /* _LIGHTANDSHADE_H */
