/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Mon Jan 24 16:10:26 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:50 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "MaterialProps.h"

#include "IfaceGeneratorP.h"

#include "TextItem.h"
#include <stdlib.h>

#define	ALTER_INVENTOR_MATERIALS

int MaterialProps::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Material Properties", 1,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Front Ambient: ",
    IF_VALUE+10, IF_VALUE+11, IF_VALUE+12, IF_VALUE+13,
    (int) &MaterialProps::cb_set_glMaterial_front_ambient,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Front Diffuse: ",
    IF_VALUE+20, IF_VALUE+21, IF_VALUE+22, IF_VALUE+23,
    (int) &MaterialProps::cb_set_glMaterial_front_diffuse,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Front Specular: ",
    IF_VALUE+30, IF_VALUE+31, IF_VALUE+32, IF_VALUE+33,
    (int) &MaterialProps::cb_set_glMaterial_front_specular,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Front Emission: ",
    IF_VALUE+40, IF_VALUE+41, IF_VALUE+42, IF_VALUE+43,
    (int) &MaterialProps::cb_set_glMaterial_front_emission,

    IF_TEXT, IF_THIS, NULL,
    (int) "Material Front Shininess: ",
    10, IF_VALUE+0,
    (int) &TextItem::FRACTIONAL,
    (int) &MaterialProps::cb_set_glMaterial_front_shininess,

    IF_TEXT, IF_THIS, NULL,
    (int) "Material Front Indices: ",
    15, IF_VALUE+100,
    (int) &TextItem::INTVECTOR,
    (int) &MaterialProps::cb_set_glMaterial_front_ci,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Back Ambient: ",
    IF_VALUE+50, IF_VALUE+51, IF_VALUE+52, IF_VALUE+53,
    (int) &MaterialProps::cb_set_glMaterial_back_ambient,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Back Diffuse: ",
    IF_VALUE+60, IF_VALUE+61, IF_VALUE+62, IF_VALUE+63,
    (int) &MaterialProps::cb_set_glMaterial_back_diffuse,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Back Specular: ",
    IF_VALUE+70, IF_VALUE+71, IF_VALUE+72, IF_VALUE+73,
    (int) &MaterialProps::cb_set_glMaterial_back_specular,

    IF_RGBA, IF_THIS, NULL,
    (int) "Material Back Emission: ",
    IF_VALUE+80, IF_VALUE+81, IF_VALUE+82, IF_VALUE+83,
    (int) &MaterialProps::cb_set_glMaterial_back_emission,

    IF_TEXT, IF_THIS, NULL,
    (int) "Material Back Shininess: ",
    10, IF_VALUE+1,
    (int) &TextItem::FRACTIONAL,
    (int) & MaterialProps::cb_set_glMaterial_back_shininess,

    IF_TEXT, IF_THIS, NULL,
    (int) "Material Back Indices: ",
    15, IF_VALUE+101,
    (int) &TextItem::INTVECTOR,
    (int) &MaterialProps::cb_set_glMaterial_back_ci,

    IF_END
};

MaterialProps::MaterialProps( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();
}

MaterialProps::~MaterialProps()
{
}

///////////////////////////////////////////////////////////////////

void
MaterialProps::install_values()
{
    GLfloat	sh;
    GLfloat	r, g, b, a;

    get_glMaterial_ambient( GL_FRONT, &r, &g, &b, &a );
    install( iface, IF_VALUE+10, (int) ftoa(r) );
    install( iface, IF_VALUE+11, (int) ftoa(g) );
    install( iface, IF_VALUE+12, (int) ftoa(b) );
    install( iface, IF_VALUE+13, (int) ftoa(a) );

    get_glMaterial_diffuse( GL_FRONT, &r, &g, &b, &a );
    install( iface, IF_VALUE+20, (int) ftoa(r) );
    install( iface, IF_VALUE+21, (int) ftoa(g) );
    install( iface, IF_VALUE+22, (int) ftoa(b) );
    install( iface, IF_VALUE+23, (int) ftoa(a) );

    get_glMaterial_specular( GL_FRONT, &r, &g, &b, &a );
    install( iface, IF_VALUE+30, (int) ftoa(r) );
    install( iface, IF_VALUE+31, (int) ftoa(g) );
    install( iface, IF_VALUE+32, (int) ftoa(b) );
    install( iface, IF_VALUE+33, (int) ftoa(a) );

    get_glMaterial_emission( GL_FRONT, &r, &g, &b, &a );
    install( iface, IF_VALUE+40, (int) ftoa(r) );
    install( iface, IF_VALUE+41, (int) ftoa(g) );
    install( iface, IF_VALUE+42, (int) ftoa(b) );
    install( iface, IF_VALUE+43, (int) ftoa(a) );

    get_glMaterial_shininess( GL_FRONT, &sh );
    install( iface, IF_VALUE+0, (int) ftoa(sh) );

    get_glMaterial_ambient( GL_BACK, &r, &g, &b, &a );
    install( iface, IF_VALUE+50, (int) ftoa(r) );
    install( iface, IF_VALUE+51, (int) ftoa(g) );
    install( iface, IF_VALUE+52, (int) ftoa(b) );
    install( iface, IF_VALUE+53, (int) ftoa(a) );

    get_glMaterial_diffuse( GL_BACK, &r, &g, &b, &a );
    install( iface, IF_VALUE+60, (int) ftoa(r) );
    install( iface, IF_VALUE+61, (int) ftoa(g) );
    install( iface, IF_VALUE+62, (int) ftoa(b) );
    install( iface, IF_VALUE+63, (int) ftoa(a) );

    get_glMaterial_specular( GL_BACK, &r, &g, &b, &a );
    install( iface, IF_VALUE+70, (int) ftoa(r) );
    install( iface, IF_VALUE+71, (int) ftoa(g) );
    install( iface, IF_VALUE+72, (int) ftoa(b) );
    install( iface, IF_VALUE+73, (int) ftoa(a) );

    get_glMaterial_emission( GL_BACK, &r, &g, &b, &a );
    install( iface, IF_VALUE+80, (int) ftoa(r) );
    install( iface, IF_VALUE+81, (int) ftoa(g) );
    install( iface, IF_VALUE+82, (int) ftoa(b) );
    install( iface, IF_VALUE+83, (int) ftoa(a) );

    get_glMaterial_shininess( GL_BACK, &sh );
    install( iface, IF_VALUE+1, (int) ftoa(sh) );

    int	amb, diff, spec;
    char	txt[80];
    
    get_glMaterial_ci( GL_FRONT, &amb, &diff, &spec );
    sprintf( txt, "%d  %d  %d", amb, diff, spec );
    install( iface, IF_VALUE+100, (int) new_strcpy( txt ) );
    
    get_glMaterial_ci( GL_BACK, &amb, &diff, &spec );
    sprintf( txt, "%d  %d  %d", amb, diff, spec );
    install( iface, IF_VALUE+101, (int) new_strcpy( txt ) );
}

void
MaterialProps::remember()
{
    rememberCurrentValues();
}

///////////////////////////////////////////////////////////////////

void
MaterialProps::set_glMaterial_ambient(GLenum face,
				      GLfloat r, GLfloat g,
				      GLfloat b, GLfloat a )
{
    if( debuglevel > 0 ) {
	char *f;

	switch( face ) {
	  case GL_FRONT:
	    f = "GL_FRONT";
	    break;
	  case GL_BACK:
	    f = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    f = "GL_FRONT_AND_BACK";
	    break;
	}
	
	printf("glMaterial %s GL_AMBIENT [%f %f %f %f]\n",
	       f, r, g, b, a );
    }

    GLfloat	v[4];

    v[0] = r;
    v[1] = g;
    v[2] = b;
    v[3] = a;

    glMaterialfv( face, GL_AMBIENT, v );

    switch( face ) {
      case GL_FRONT:
	_front_ambient.r = r;
	_front_ambient.g = g;
	_front_ambient.b = b;
	_front_ambient.a = a;
	break;

      case GL_BACK:
	_back_ambient.r = r;
	_back_ambient.g = g;
	_back_ambient.b = b;
	_back_ambient.a = a;
	break;

      case GL_FRONT_AND_BACK:
	_front_ambient.r = r;
	_front_ambient.g = g;
	_front_ambient.b = b;
	_front_ambient.a = a;
	_back_ambient.r = r;
	_back_ambient.g = g;
	_back_ambient.b = b;
	_back_ambient.a = a;
	break;
    }

    redraw();
}

void
MaterialProps::get_glMaterial_ambient(GLenum face,
				      GLfloat *r, GLfloat *g,
				      GLfloat *b, GLfloat *a )
{
    GLfloat	v[4];

    glGetMaterialfv( face, GL_AMBIENT, v );

    *r = v[0];
    *g = v[1];
    *b = v[2];
    *a = v[3];

    switch( face ) {
      case GL_FRONT:
	_front_ambient.r = *r;
	_front_ambient.g = *g;
	_front_ambient.b = *b;
	_front_ambient.a = *a;
	break;

      case GL_BACK:
	_back_ambient.r = *r;
	_back_ambient.g = *g;
	_back_ambient.b = *b;
	_back_ambient.a = *a;
	break;

      case GL_FRONT_AND_BACK:
	_front_ambient.r = *r;
	_front_ambient.g = *g;
	_front_ambient.b = *b;
	_front_ambient.a = *a;
	_back_ambient.r = *r;
	_back_ambient.g = *g;
	_back_ambient.b = *b;
	_back_ambient.a = *a;
	break;
    }
}

void
MaterialProps::set_glMaterial_diffuse(GLenum face,
				      GLfloat r, GLfloat g,
				      GLfloat b, GLfloat a )
{
    if( debuglevel > 0 ) {
	char *f;

	switch( face ) {
	  case GL_FRONT:
	    f = "GL_FRONT";
	    break;
	  case GL_BACK:
	    f = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    f = "GL_FRONT_AND_BACK";
	    break;
	}
	
	printf("glMaterial %s GL_DIFFUSE [%f %f %f %f]\n",
	       f, r, g, b, a );
    }

    GLfloat	v[4];

    v[0] = r;
    v[1] = g;
    v[2] = b;
    v[3] = a;

    glMaterialfv( face, GL_DIFFUSE, v );
    
    switch( face ) {
      case GL_FRONT:
	_front_diffuse.r = r;
	_front_diffuse.g = g;
	_front_diffuse.b = b;
	_front_diffuse.a = a;
	break;

      case GL_BACK:
	_back_diffuse.r = r;
	_back_diffuse.g = g;
	_back_diffuse.b = b;
	_back_diffuse.a = a;
	break;

      case GL_FRONT_AND_BACK:
	_front_diffuse.r = r;
	_front_diffuse.g = g;
	_front_diffuse.b = b;
	_front_diffuse.a = a;
	_back_diffuse.r = r;
	_back_diffuse.g = g;
	_back_diffuse.b = b;
	_back_diffuse.a = a;
	break;
    }

    redraw();
}

void
MaterialProps::get_glMaterial_diffuse(GLenum face,
				      GLfloat *r, GLfloat *g,
				      GLfloat *b, GLfloat *a )
{
    GLfloat	v[4];

    glGetMaterialfv( face, GL_DIFFUSE, v );

    *r = v[0];
    *g = v[1];
    *b = v[2];
    *a = v[3];
    
    switch( face ) {
      case GL_FRONT:
	_front_diffuse.r = *r;
	_front_diffuse.g = *g;
	_front_diffuse.b = *b;
	_front_diffuse.a = *a;
	break;

      case GL_BACK:
	_back_diffuse.r = *r;
	_back_diffuse.g = *g;
	_back_diffuse.b = *b;
	_back_diffuse.a = *a;
	break;

      case GL_FRONT_AND_BACK:
	_front_diffuse.r = *r;
	_front_diffuse.g = *g;
	_front_diffuse.b = *b;
	_front_diffuse.a = *a;
	_back_diffuse.r = *r;
	_back_diffuse.g = *g;
	_back_diffuse.b = *b;
	_back_diffuse.a = *a;
	break;
    }
}

void
MaterialProps::set_glMaterial_specular(GLenum face,
				      GLfloat r, GLfloat g,
				      GLfloat b, GLfloat a )
{
    if( debuglevel > 0 ) {
	char *f;

	switch( face ) {
	  case GL_FRONT:
	    f = "GL_FRONT";
	    break;
	  case GL_BACK:
	    f = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    f = "GL_FRONT_AND_BACK";
	    break;
	}
	
	printf("glMaterial %s GL_SPECULAR [%f %f %f %f]\n",
	       f, r, g, b, a );
    }

    GLfloat	v[4];

    v[0] = r;
    v[1] = g;
    v[2] = b;
    v[3] = a;

    glMaterialfv( face, GL_SPECULAR, v );
    
    switch( face ) {
      case GL_FRONT:
	_front_specular.r = r;
	_front_specular.g = g;
	_front_specular.b = b;
	_front_specular.a = a;
	break;

      case GL_BACK:
	_back_specular.r = r;
	_back_specular.g = g;
	_back_specular.b = b;
	_back_specular.a = a;
	break;

      case GL_FRONT_AND_BACK:
	_front_specular.r = r;
	_front_specular.g = g;
	_front_specular.b = b;
	_front_specular.a = a;
	_back_specular.r = r;
	_back_specular.g = g;
	_back_specular.b = b;
	_back_specular.a = a;
	break;
    }

    redraw();
}

void
MaterialProps::get_glMaterial_specular(GLenum face,
				      GLfloat *r, GLfloat *g,
				      GLfloat *b, GLfloat *a )
{
    GLfloat	v[4];

    glGetMaterialfv( face, GL_SPECULAR, v );

    *r = v[0];
    *g = v[1];
    *b = v[2];
    *a = v[3];
    
    switch( face ) {
      case GL_FRONT:
	_front_specular.r = *r;
	_front_specular.g = *g;
	_front_specular.b = *b;
	_front_specular.a = *a;
	break;

      case GL_BACK:
	_back_specular.r = *r;
	_back_specular.g = *g;
	_back_specular.b = *b;
	_back_specular.a = *a;
	break;

      case GL_FRONT_AND_BACK:
	_front_specular.r = *r;
	_front_specular.g = *g;
	_front_specular.b = *b;
	_front_specular.a = *a;
	_back_specular.r = *r;
	_back_specular.g = *g;
	_back_specular.b = *b;
	_back_specular.a = *a;
	break;
    }
}

void
MaterialProps::set_glMaterial_emission(GLenum face,
				      GLfloat r, GLfloat g,
				      GLfloat b, GLfloat a )
{
    if( debuglevel > 0 ) {
	char *f;

	switch( face ) {
	  case GL_FRONT:
	    f = "GL_FRONT";
	    break;
	  case GL_BACK:
	    f = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    f = "GL_FRONT_AND_BACK";
	    break;
	}
	
	printf("glMaterial %s GL_EMISSION [%f %f %f %f]\n",
	       f, r, g, b, a );
    }

    GLfloat	v[4];

    v[0] = r;
    v[1] = g;
    v[2] = b;
    v[3] = a;

    glMaterialfv( face, GL_EMISSION, v );
    
    switch( face ) {
      case GL_FRONT:
	_front_emission.r = r;
	_front_emission.g = g;
	_front_emission.b = b;
	_front_emission.a = a;
	break;

      case GL_BACK:
	_back_emission.r = r;
	_back_emission.g = g;
	_back_emission.b = b;
	_back_emission.a = a;
	break;

      case GL_FRONT_AND_BACK:
	_front_emission.r = r;
	_front_emission.g = g;
	_front_emission.b = b;
	_front_emission.a = a;
	_back_emission.r = r;
	_back_emission.g = g;
	_back_emission.b = b;
	_back_emission.a = a;
	break;
    }

    redraw();
}

void
MaterialProps::get_glMaterial_emission(GLenum face,
				      GLfloat *r, GLfloat *g,
				      GLfloat *b, GLfloat *a )
{
    GLfloat	v[4];

    glGetMaterialfv( face, GL_EMISSION, v );

    *r = v[0];
    *g = v[1];
    *b = v[2];
    *a = v[3];
    
    switch( face ) {
      case GL_FRONT:
	_front_emission.r = *r;
	_front_emission.g = *g;
	_front_emission.b = *b;
	_front_emission.a = *a;
	break;

      case GL_BACK:
	_back_emission.r = *r;
	_back_emission.g = *g;
	_back_emission.b = *b;
	_back_emission.a = *a;
	break;

      case GL_FRONT_AND_BACK:
	_front_emission.r = *r;
	_front_emission.g = *g;
	_front_emission.b = *b;
	_front_emission.a = *a;
	_back_emission.r = *r;
	_back_emission.g = *g;
	_back_emission.b = *b;
	_back_emission.a = *a;
	break;
    }
}

void
MaterialProps::set_glMaterial_shininess( GLenum face, GLfloat sh )
{
    if( debuglevel > 0 ) {
	char *f;

	switch( face ) {
	  case GL_FRONT:
	    f = "GL_FRONT";
	    break;
	  case GL_BACK:
	    f = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    f = "GL_FRONT_AND_BACK";
	    break;
	}
	
	printf("glMaterial %s GL_SHININESS %f\n", f, sh );
    }

    glMaterialf( face, GL_SHININESS, sh );

    switch( face ) {
      case GL_FRONT:
	_front_shininess = sh;
	break;

      case GL_BACK:
	_back_shininess = sh;
	break;

      case GL_FRONT_AND_BACK:
	_front_shininess = sh;
	_back_shininess = sh;
	break;
    }

    redraw();
}

void
MaterialProps::get_glMaterial_shininess( GLenum face, GLfloat *sh )
{
    glGetMaterialfv( face, GL_SHININESS, sh );

    switch( face ) {
      case GL_FRONT:
	_front_shininess = *sh;
	break;

      case GL_BACK:
	_back_shininess = *sh;
	break;

      case GL_FRONT_AND_BACK:
	_front_shininess = *sh;
	_back_shininess = *sh;
	break;
    }
}

void
MaterialProps::set_glMaterial_ci(GLenum face,
				 int ambient_index,
				 int diffuse_index,
				 int specular_index )
{
    if( debuglevel > 0 ) {
	char *f;

	switch( face ) {
	  case GL_FRONT:
	    f = "GL_FRONT";
	    break;
	  case GL_BACK:
	    f = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    f = "GL_FRONT_AND_BACK";
	    break;
	}

	printf("glMaterial %s GL_COLOR_INDEXES [%d %d %d]\n",
	       f, ambient_index, diffuse_index, specular_index );
    }

    GLint	v[3];

    v[0] = ambient_index;
    v[1] = diffuse_index;
    v[2] = specular_index;

    glMaterialiv( face, GL_COLOR_INDEXES, v );

    switch( face ) {
      case GL_FRONT:
	_front_ci_ambient = ambient_index;
	_front_ci_diffuse = diffuse_index;
	_front_ci_specular = specular_index;
	break;

      case GL_BACK:
	_back_ci_ambient = ambient_index;
	_back_ci_diffuse = diffuse_index;
	_back_ci_specular = specular_index;
	break;

      case GL_FRONT_AND_BACK:
	_front_ci_ambient = ambient_index;
	_front_ci_diffuse = diffuse_index;
	_front_ci_specular = specular_index;
	_back_ci_ambient = ambient_index;
	_back_ci_diffuse = diffuse_index;
	_back_ci_specular = specular_index;
	break;
    }

    redraw();
}

void
MaterialProps::get_glMaterial_ci(GLenum face,
				 int *ambient_index,
				 int *diffuse_index,
				 int *specular_index )
{
    GLint	v[3];

    glGetMaterialiv( face, GL_COLOR_INDEXES, v );

    *ambient_index = v[0];
    *diffuse_index = v[1];
    *specular_index = v[2];

    switch( face ) {
      case GL_FRONT:
	_front_ci_ambient = *ambient_index;
	_front_ci_diffuse = *diffuse_index;
	_front_ci_specular = *specular_index;
	break;

      case GL_BACK:
	_back_ci_ambient = *ambient_index;
	_back_ci_diffuse = *diffuse_index;
	_back_ci_specular = *specular_index;
	break;

      case GL_FRONT_AND_BACK:
	_front_ci_ambient = *ambient_index;
	_front_ci_diffuse = *diffuse_index;
	_front_ci_specular = *specular_index;
	_back_ci_ambient = *ambient_index;
	_back_ci_diffuse = *diffuse_index;
	_back_ci_specular = *specular_index;
	break;
    }
}

void
MaterialProps::reset()
{
    int debugstack;
    Boolean redrawstack;

    
    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }
    
    setCurrentMaterialProps();

    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}


///////////////////////////////////////////////////////////////////

void
MaterialProps::setCurrentMaterialProps()
{
    set_glMaterial_ambient( GL_FRONT, _front_ambient.r, _front_ambient.g,
			   _front_ambient.b, _front_ambient.a );
    set_glMaterial_diffuse( GL_FRONT, _front_diffuse.r, _front_diffuse.g,
			   _front_diffuse.b, _front_diffuse.a );
    set_glMaterial_specular( GL_FRONT, _front_specular.r, _front_specular.g,
			    _front_specular.b, _front_specular.a );
    set_glMaterial_emission( GL_FRONT, _front_emission.r, _front_emission.g,
			    _front_emission.b, _front_emission.a );
    set_glMaterial_shininess( GL_FRONT, _front_shininess );
    set_glMaterial_ci( GL_FRONT, _front_ci_ambient, _front_ci_diffuse,
		      _front_ci_specular );
    
    set_glMaterial_ambient( GL_BACK, _back_ambient.r, _back_ambient.g,
			   _back_ambient.b, _back_ambient.a );
    set_glMaterial_diffuse( GL_BACK, _back_diffuse.r, _back_diffuse.g,
			   _back_diffuse.b, _back_diffuse.a );
    set_glMaterial_specular( GL_BACK, _back_specular.r, _back_specular.g,
			    _back_specular.b, _back_specular.a );
    set_glMaterial_emission( GL_BACK, _back_emission.r, _back_emission.g,
			    _back_emission.b, _back_emission.a );
    set_glMaterial_shininess( GL_BACK, _back_shininess );
    set_glMaterial_ci( GL_BACK, _back_ci_ambient, _back_ci_diffuse,
		      _back_ci_specular );
}

void
MaterialProps::rememberCurrentValues()
{
    get_glMaterial_ambient( GL_FRONT, &_front_ambient.r, &_front_ambient.g,
			   &_front_ambient.b, &_front_ambient.a );
    get_glMaterial_diffuse( GL_FRONT, &_front_diffuse.r, &_front_diffuse.g,
			   &_front_diffuse.b, &_front_diffuse.a );
    get_glMaterial_specular( GL_FRONT, &_front_specular.r, &_front_specular.g,
			    &_front_specular.b, &_front_specular.a );
    get_glMaterial_emission( GL_FRONT, &_front_emission.r, &_front_emission.g,
			    &_front_emission.b, &_front_emission.a );
    get_glMaterial_shininess( GL_FRONT, &_front_shininess );
    get_glMaterial_ci( GL_FRONT, &_front_ci_ambient, &_front_ci_diffuse,
		      &_front_ci_specular );
    
    get_glMaterial_ambient( GL_BACK, &_back_ambient.r, &_back_ambient.g,
			   &_back_ambient.b, &_back_ambient.a );
    get_glMaterial_diffuse( GL_BACK, &_back_diffuse.r, &_back_diffuse.g,
			   &_back_diffuse.b, &_back_diffuse.a );
    get_glMaterial_specular( GL_BACK, &_back_specular.r, &_back_specular.g,
			    &_back_specular.b, &_back_specular.a );
    get_glMaterial_emission( GL_BACK, &_back_emission.r, &_back_emission.g,
			    &_back_emission.b, &_back_emission.a );
    get_glMaterial_shininess( GL_BACK, &_back_shininess );
    get_glMaterial_ci( GL_BACK, &_back_ci_ambient, &_back_ci_diffuse,
		      &_back_ci_specular );
}

///////////////////////////////////////////////////////////////////

void
MaterialProps::cb_set_glMaterial_front_ambient( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_ambient( GL_FRONT, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_front_diffuse( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_diffuse( GL_FRONT, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_front_specular( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_specular( GL_FRONT, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_front_emission( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_emission( GL_FRONT, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_front_shininess( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    GLfloat f = atof( (char *) call );

    self->set_glMaterial_shininess( GL_FRONT, f );
}

void
MaterialProps::cb_set_glMaterial_front_ci( VkCallbackObject *,
					  void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    char *txt = (char *) call;

    int	amb, diff, spec;
    sscanf( txt, "%d %d %d", &amb, &diff, &spec );

    self->set_glMaterial_ci( GL_FRONT, amb, diff, spec );
}


void
MaterialProps::cb_set_glMaterial_back_ambient( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_ambient( GL_BACK, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_back_diffuse( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_diffuse( GL_BACK, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_back_specular( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_specular( GL_BACK, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_back_emission( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glMaterial_emission( GL_BACK, c->r, c->g, c->b, c->a );
}

void
MaterialProps::cb_set_glMaterial_back_shininess( VkCallbackObject *,
					       void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    GLfloat f = atof( (char *) call );

    self->set_glMaterial_shininess( GL_BACK, f );
}

void
MaterialProps::cb_set_glMaterial_back_ci( VkCallbackObject *,
					 void *client, void *call )
{
    MaterialProps *self = (MaterialProps *) client;
    char *txt = (char *) call;

    int	amb, diff, spec;
    sscanf( txt, "%d %d %d", &amb, &diff, &spec );

    self->set_glMaterial_ci( GL_BACK, amb, diff, spec );
}


