/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _MATERIALPROPS_H
#define _MATERIALPROPS_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Mon Jan 24 15:56:05 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:48 1994 by George Kyriazis
 *
 * Material properties
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"
class SoMaterial;

class MaterialProps : public IfaceGenerator {
  public:
    MaterialProps( GlWindow * );
    virtual ~MaterialProps();

    // material props
    void	set_glMaterial_ambient(GLenum face,
				       GLfloat r,
				       GLfloat g,
				       GLfloat b,
				       GLfloat a );
    void	get_glMaterial_ambient(GLenum face,
				       GLfloat *r,
				       GLfloat *g,
				       GLfloat *b,
				       GLfloat *a );
    void	set_glMaterial_diffuse(GLenum face,
				       GLfloat r,
				       GLfloat g,
				       GLfloat b,
				       GLfloat a );
    void	get_glMaterial_diffuse(GLenum face,
				       GLfloat *r,
				       GLfloat *g,
				       GLfloat *b,
				       GLfloat *a );
    void	set_glMaterial_specular(GLenum face,
				       GLfloat r,
				       GLfloat g,
				       GLfloat b,
				       GLfloat a );
    void	get_glMaterial_specular(GLenum face,
				       GLfloat *r,
				       GLfloat *g,
				       GLfloat *b,
				       GLfloat *a );
    void	set_glMaterial_emission(GLenum face,
				       GLfloat r,
				       GLfloat g,
				       GLfloat b,
				       GLfloat a );
    void	get_glMaterial_emission(GLenum face,
				       GLfloat *r,
				       GLfloat *g,
				       GLfloat *b,
				       GLfloat *a );
    void	set_glMaterial_shininess( GLenum, GLfloat shininess );
    void	get_glMaterial_shininess( GLenum, GLfloat *shininess );
    void	set_glMaterial_ci(GLenum face,
				  int ambient_index,
				  int diffuse_index,
				  int specular_index );
    void	get_glMaterial_ci(GLenum face,
				  int *ambient_index,
				  int *diffuse_index,
				  int *specular_index );

    void	setCurrentMaterialProps();

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // we need to keep some state
    RGBAColor	_front_ambient;
    RGBAColor	_front_diffuse;
    RGBAColor	_front_specular;
    RGBAColor	_front_emission;
    GLfloat	_front_shininess;
    GLint	_front_ci_ambient, _front_ci_diffuse, _front_ci_specular;
    RGBAColor	_back_ambient;
    RGBAColor	_back_diffuse;
    RGBAColor	_back_specular;
    RGBAColor	_back_emission;
    GLfloat	_back_shininess;
    GLint	_back_ci_ambient, _back_ci_diffuse, _back_ci_specular;

    void	rememberCurrentValues();

    // callbacks
    static	void	cb_set_glMaterial_front_ambient( CB_ARGS );
    static	void	cb_set_glMaterial_front_diffuse( CB_ARGS );
    static	void	cb_set_glMaterial_front_specular( CB_ARGS );
    static	void	cb_set_glMaterial_front_emission( CB_ARGS );
    static	void	cb_set_glMaterial_front_shininess( CB_ARGS );
    static	void	cb_set_glMaterial_front_ci( CB_ARGS );
    static	void	cb_set_glMaterial_back_ambient( CB_ARGS );
    static	void	cb_set_glMaterial_back_diffuse( CB_ARGS );
    static	void	cb_set_glMaterial_back_specular( CB_ARGS );
    static	void	cb_set_glMaterial_back_emission( CB_ARGS );
    static	void	cb_set_glMaterial_back_shininess( CB_ARGS );
    static	void	cb_set_glMaterial_back_ci( CB_ARGS );
};

#endif /* _MATERIALPROPS_H */
