/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Tue Mar 15 14:38:00 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:12:14 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "MatrixItem.h"

#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/Text.h>

const char * const MatrixItem::newValue = "NewValue";
const char * const MatrixItem::acceptable_string = "01234567890-. \n";

XtCallbackRec	MatrixItem::act_call[2] = {
    { &MatrixItem::activateCB, NULL },
    NULL
};

XtCallbackRec	MatrixItem::ver_call[2] = {
    { &MatrixItem::verifyCB, NULL },
    NULL
};


MatrixItem::MatrixItem( char *Name, Widget Parent )
: VkComponent( Name )
{
    create_widget( Name, Parent );

    ourMat = new GLfloat[16];
}

MatrixItem::~MatrixItem()
{
    delete [] ourMat;
}



void
MatrixItem::create_widget( char *name, Widget parent )
{
    int	ac;
    Arg	av[10];

    // create the container first
    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    rowcol = XmCreateRowColumn( parent,
			       name,
			       av, ac );
    XtManageChild( rowcol );

    label = XmCreateLabel( rowcol,
			  name,
			  NULL, 0 );
    XtManageChild( label );

    // fix the callback lists
    act_call[0].closure = this;
    ver_call[0].closure = this;

    // now the text item
    ac = 0;
    XtSetArg( av[ac], XmNrows, 4 ); ac++;
    XtSetArg( av[ac], XmNeditMode, XmMULTI_LINE_EDIT ); ac++;
    XtSetArg( av[ac], XmNcolumns, 36 ); ac++;
    XtSetArg( av[ac], XmNuserData, this ); ac++;
    XtSetArg( av[ac], XmNactivateCallback, act_call ); ac++;
    XtSetArg( av[ac], XmNmodifyVerifyCallback, ver_call ); ac++;
    XtSetArg( av[ac], XmNeditable, FALSE ); ac++;
    ti = XmCreateText( rowcol,
		      name,
		      av, ac );
    XtManageChild( ti );
}


void
MatrixItem::set_matrix( GLfloat *mat )
{
    int	ac;
    Arg	av[10];

    char text[5000];

    for( int i = 0; i < 16; i++ ) {
	ourMat[i] = mat[i];
    }

    sprintf( text,
	    "%f %f %f %f\n%f %f %f %f\n%f %f %f %f\n%f %f %f %f",
	    mat[0], mat[1], mat[2], mat[3],
	    mat[4], mat[5], mat[6], mat[7],
	    mat[8], mat[9], mat[10], mat[11],
	    mat[12], mat[13], mat[14], mat[15] );

    ac = 0;
    XtSetArg( av[ac], XmNvalue, text); ac++;
    XtSetValues( ti, av, ac );
}

GLfloat *
MatrixItem::get_matrix()
{
    return ourMat;
}


void
MatrixItem::activateCB( Widget w, XtPointer client, XtPointer call )
{
    MatrixItem *self = (MatrixItem *) client;

    self->activate_cb( w, call );
}

void
MatrixItem::activate_cb( Widget, XtPointer )
{
    // grab the matrix first
    char *text;
    int	ac;
    Arg	av[10];

    ac = 0;
    XtSetArg( av[ac], XmNvalue, &text ); ac++;
    XtGetValues( ti, av, ac );

    sscanf( text, "%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f",
	   &ourMat[0],
	   &ourMat[1],
	   &ourMat[2],
	   &ourMat[3],
	   &ourMat[4],
	   &ourMat[5],
	   &ourMat[6],
	   &ourMat[7],
	   &ourMat[8],
	   &ourMat[9],
	   &ourMat[10],
	   &ourMat[11],
	   &ourMat[12],
	   &ourMat[13],
	   &ourMat[14],
	   &ourMat[15] );
    
    // pass our contents as call data
    callCallbacks( newValue, ourMat );
}

void
MatrixItem::verifyCB( Widget w, XtPointer client, XtPointer call )
{
    MatrixItem *self = (MatrixItem *) client;

    self->verify_cb( w, call );
}

void
MatrixItem::verify_cb( Widget, XtPointer call )
{
    XmTextVerifyCallbackStruct *cb =
	(XmTextVerifyCallbackStruct *) call;

    int	i;
    char *text = cb->text->ptr;
    int max = cb->text->length;
    for( i = 0; i < max; i++ ) {
	if( NULL == strchr( acceptable_string, text[i] ) ) {
	    // we are hosed..  Not in string.
	    cb->doit = FALSE;
	    return;
	}
    }
}

