/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Fri Feb 18 11:00:18 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:40 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "ModelClipPlanes.h"

#include "IfaceGeneratorP.h"

#include "PlaneItem.h"
#include "GlWindow.h"
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/actions/SoGLRenderAction.h>

int ModelClipPlanes::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Model Clip Planes", 1,

    IF_RADIO, IF_THIS, NULL,
    (int) "Clip Plane 0 Enable", 2,
    0, (int) &ModelClipPlanes::cb_set_clip_enable0,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_PLANE, IF_THIS, NULL,
    (int) "Clip Plane 0",
    IF_VALUE+000, IF_VALUE+001, IF_VALUE+002, IF_VALUE+003,
    (int) &ModelClipPlanes::cb_set_glClipPlane0,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Clip Plane 1 Enable", 2,
    0, (int) &ModelClipPlanes::cb_set_clip_enable1,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_PLANE, IF_THIS, NULL,
    (int) "Clip Plane 1",
    IF_VALUE+100, IF_VALUE+101, IF_VALUE+102, IF_VALUE+103,
    (int) &ModelClipPlanes::cb_set_glClipPlane1,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Clip Plane 2 Enable", 2,
    0, (int) &ModelClipPlanes::cb_set_clip_enable2,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_PLANE, IF_THIS, NULL,
    (int) "Clip Plane 2",
    IF_VALUE+200, IF_VALUE+201, IF_VALUE+202, IF_VALUE+203,
    (int) &ModelClipPlanes::cb_set_glClipPlane2,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Clip Plane 3 Enable", 2,
    0, (int) &ModelClipPlanes::cb_set_clip_enable3,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_PLANE, IF_THIS, NULL,
    (int) "Clip Plane 3",
    IF_VALUE+300, IF_VALUE+301, IF_VALUE+302, IF_VALUE+303,
    (int) &ModelClipPlanes::cb_set_glClipPlane3,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Clip Plane 4 Enable", 2,
    0, (int) &ModelClipPlanes::cb_set_clip_enable4,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_PLANE, IF_THIS, NULL,
    (int) "Clip Plane 4",
    IF_VALUE+400, IF_VALUE+401, IF_VALUE+402, IF_VALUE+403,
    (int) &ModelClipPlanes::cb_set_glClipPlane4,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Clip Plane 5 Enable", 2,
    0, (int) &ModelClipPlanes::cb_set_clip_enable5,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_PLANE, IF_THIS, NULL,
    (int) "Clip Plane 5",
    IF_VALUE+500, IF_VALUE+501, IF_VALUE+502, IF_VALUE+503,
    (int) &ModelClipPlanes::cb_set_glClipPlane5,

    IF_END
};

ModelClipPlanes::ModelClipPlanes( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();

    setup_iv();
}

ModelClipPlanes::~ModelClipPlanes()
{
    finish_up_iv();
}


///////////////////////////////////////////////////////////////////

void
ModelClipPlanes::install_values()
{
    GLdouble	x, y, z, w;

    int	i;

    for( i = 0; i < MAXNUM_CLIP_PLANES; i++ ) {
	get_glClipPlane( i, &x, &y, &z, &w );

	install( iface, IF_VALUE+100*i+0, (int) ftoa(x) );
	install( iface, IF_VALUE+100*i+1, (int) ftoa(y) );
	install( iface, IF_VALUE+100*i+2, (int) ftoa(z) );
	install( iface, IF_VALUE+100*i+3, (int) ftoa(w) );
    }
}

void
ModelClipPlanes::remember()
{
    GLdouble	d1, d2, d3, d4;
    Boolean	b;

    int	i;

    for( i = 0; i < MAXNUM_CLIP_PLANES; i++ ) {
	get_glClipPlane( i, &d1, &d2, &d3, &d4 );
	get_clip_enable( i, &b );
    }
}

void
ModelClipPlanes::setup_iv()
{
    // create the nodes
    cbSetNode = new SoCallback;
    cbSetNode->ref();
    cbUnsetNode = new SoCallback;
    cbUnsetNode->ref();

    // initialize the callbacks
    cbSetNode->setCallback( setClipPlanes, this );
    cbUnsetNode->setCallback( unsetClipPlanes, this );

    // add to the parent
    glWindow->get_pre_extender_node()->addChild( cbSetNode );
    glWindow->get_post_extender_node()->addChild( cbUnsetNode );
}

void
ModelClipPlanes::finish_up_iv()
{
    // remove from the parent
    glWindow->get_pre_extender_node()->removeChild( cbSetNode );
    glWindow->get_post_extender_node()->removeChild( cbUnsetNode );

    // add delete finally
    cbSetNode->unref();
    cbUnsetNode->unref();
}

void
ModelClipPlanes::setClipPlanes( void *user, SoAction *action )
{
    // only do it when we are rendering
    if( action->isOfType( SoGLRenderAction::getClassTypeId() ) == FALSE ) {
	return;
    }

    ModelClipPlanes *self = (ModelClipPlanes *)user;

    int	i;
    for( i = 0; i < MAXNUM_CLIP_PLANES; i++ ) {
	if( self->_clip_enable[i] ) {
	    glEnable( GL_CLIP_PLANE0+i );
	    self->set_glClipPlane( i,
				  self->_glClipPlane_x[i],
				  self->_glClipPlane_y[i],
				  self->_glClipPlane_z[i],
				  self->_glClipPlane_w[i] );
	}
    }
}

void
ModelClipPlanes::unsetClipPlanes( void *user, SoAction *action )
{
    // only do it when we are rendering
    if( action->isOfType( SoGLRenderAction::getClassTypeId() ) == FALSE ) {
	return;
    }

    ModelClipPlanes *self = (ModelClipPlanes *)user;

    int	i;
    for( i = 0; i < MAXNUM_CLIP_PLANES; i++ ) {
	if( self->_clip_enable[i] ) {
	    glDisable( GL_CLIP_PLANE0+i );
	}
    }
}

///////////////////////////////////////////////////////////////////


//
// clip operations
//

void
ModelClipPlanes::set_clip_enable( int plane, Boolean state )
{
    if( debuglevel > 0 ) {
	if( state ) {
	    printf("glEnable GL_CLIP_PLANE%d\n", plane );
	} else {
	    printf("glDisable GL_CLIP_PLANE%d\n", plane );
	}
    }

    switch_flag( GL_CLIP_PLANE0+plane, state );

    _clip_enable[plane] = state;

    redraw();
}

void
ModelClipPlanes::get_clip_enable( int plane, Boolean *state )
{
    Boolean bool = glIsEnabled( GL_CLIP_PLANE0+plane );

    *state = bool;

    _clip_enable[plane] = bool;
}

void
ModelClipPlanes::set_glClipPlane( int plane,
				 GLdouble a, GLdouble b,
				 GLdouble c, GLdouble d )
{
    if( debuglevel > 0 ) {
	printf("glClipPlane GL_CLIP_PLANE%d [%lf %lf %lf %lf]\n",
	       plane, a, b, c, d );
    }

    GLdouble	v[4];

    v[0] = a;
    v[1] = b;
    v[2] = c;
    v[3] = d;

    glClipPlane( GL_CLIP_PLANE0 + plane, v );

    _glClipPlane_x[plane] = a;
    _glClipPlane_y[plane] = b;
    _glClipPlane_z[plane] = c;
    _glClipPlane_w[plane] = d;

    redraw();
}

void
ModelClipPlanes::get_glClipPlane( int plane,
				 GLdouble *a, GLdouble *b,
				 GLdouble *c, GLdouble *d )
{
    GLdouble	v[4];

    glGetClipPlane( plane, v );

    *a = v[0];
    *b = v[1];
    *c = v[2];
    *d = v[3];

    _glClipPlane_x[plane] = *a;
    _glClipPlane_y[plane] = *b;
    _glClipPlane_z[plane] = *c;
    _glClipPlane_w[plane] = *d;
}



void
ModelClipPlanes::reset()
{
    int	debugstack;
    Boolean redrawstack;

    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }

    int	i;

    for( i = 0; i < MAXNUM_CLIP_PLANES; i++ ) {
	set_clip_enable( i, _clip_enable[i] );
	set_glClipPlane( i,
			_glClipPlane_x[i],
			_glClipPlane_y[i],
			_glClipPlane_z[i],
			_glClipPlane_w[i] );
    }


    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}



////////////////////////////////////////////////////////////////////
//  callbacks
////////////////////////////////////////////////////////////////////


void
ModelClipPlanes::cb_set_clip_enable0( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    int	state = (int) call;

    self->set_clip_enable( 0, state );
}

void
ModelClipPlanes::cb_set_clip_enable1( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    int	state = (int) call;

    self->set_clip_enable( 1, state );
}

void
ModelClipPlanes::cb_set_clip_enable2( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    int	state = (int) call;

    self->set_clip_enable( 2, state );
}

void
ModelClipPlanes::cb_set_clip_enable3( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    int	state = (int) call;

    self->set_clip_enable( 3, state );
}

void
ModelClipPlanes::cb_set_clip_enable4( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    int	state = (int) call;

    self->set_clip_enable( 4, state );
}

void
ModelClipPlanes::cb_set_clip_enable5( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    int	state = (int) call;

    self->set_clip_enable( 5, state );
}


void
ModelClipPlanes::cb_set_glClipPlane0( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    PlaneItem	*pi = (PlaneItem *) call;

    float	x, y, z, w;

    pi->get_plane( &x, &y, &z, &w );

    self->set_glClipPlane( 0, x, y, z, w );
}

void
ModelClipPlanes::cb_set_glClipPlane1( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    PlaneItem	*pi = (PlaneItem *) call;

    float	x, y, z, w;

    pi->get_plane( &x, &y, &z, &w );

    self->set_glClipPlane( 1, x, y, z, w );
}

void
ModelClipPlanes::cb_set_glClipPlane2( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    PlaneItem	*pi = (PlaneItem *) call;

    float	x, y, z, w;

    pi->get_plane( &x, &y, &z, &w );

    self->set_glClipPlane( 2, x, y, z, w );
}

void
ModelClipPlanes::cb_set_glClipPlane3( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    PlaneItem	*pi = (PlaneItem *) call;

    float	x, y, z, w;

    pi->get_plane( &x, &y, &z, &w );

    self->set_glClipPlane( 3, x, y, z, w );
}

void
ModelClipPlanes::cb_set_glClipPlane4( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    PlaneItem	*pi = (PlaneItem *) call;

    float	x, y, z, w;

    pi->get_plane( &x, &y, &z, &w );

    self->set_glClipPlane( 4, x, y, z, w );
}

void
ModelClipPlanes::cb_set_glClipPlane5( VkCallbackObject *,
				     void *client, void *call )
{
    ModelClipPlanes *self = (ModelClipPlanes *) client;
    PlaneItem	*pi = (PlaneItem *) call;

    float	x, y, z, w;

    pi->get_plane( &x, &y, &z, &w );

    self->set_glClipPlane( 5, x, y, z, w );
}

