/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _MODELCLIPPLANES_H
#define _MODELCLIPPLANES_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Fri Feb 18 10:53:26 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:38 1994 by George Kyriazis
 *
 * the 6 model clip planes
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class SoCallback;
class SoAction;

class ModelClipPlanes : public IfaceGenerator {
  public:
    ModelClipPlanes( GlWindow * );
    virtual ~ModelClipPlanes();

    // the six planes are the same
    void	set_clip_enable( int plane, Boolean state );
    void	get_clip_enable( int plane, Boolean *state );
    void	set_glClipPlane( int plane,
				GLdouble a,
				GLdouble b,
				GLdouble c,
				GLdouble d );
    void	get_glClipPlane( int plane,
				GLdouble *a,
				GLdouble *b,
				GLdouble *c,
				GLdouble *d );

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // callback node needed to enable/disable the clip planes
    SoCallback	*cbSetNode;
    SoCallback	*cbUnsetNode;
    void	setup_iv();
    void	finish_up_iv();
    static	void	setClipPlanes( void *user, SoAction *action );
    static	void	unsetClipPlanes( void *user, SoAction *action );

    // local state
#define	MAXNUM_CLIP_PLANES	6
    Boolean	_clip_enable[MAXNUM_CLIP_PLANES];
    GLdouble	_glClipPlane_x[MAXNUM_CLIP_PLANES];
    GLdouble	_glClipPlane_y[MAXNUM_CLIP_PLANES];
    GLdouble	_glClipPlane_z[MAXNUM_CLIP_PLANES];
    GLdouble	_glClipPlane_w[MAXNUM_CLIP_PLANES];

    // callbacks
    static	void	cb_set_clip_enable0( CB_ARGS );
    static	void	cb_set_clip_enable1( CB_ARGS );
    static	void	cb_set_clip_enable2( CB_ARGS );
    static	void	cb_set_clip_enable3( CB_ARGS );
    static	void	cb_set_clip_enable4( CB_ARGS );
    static	void	cb_set_clip_enable5( CB_ARGS );
    static	void	cb_set_glClipPlane0( CB_ARGS );
    static	void	cb_set_glClipPlane1( CB_ARGS );
    static	void	cb_set_glClipPlane2( CB_ARGS );
    static	void	cb_set_glClipPlane3( CB_ARGS );
    static	void	cb_set_glClipPlane4( CB_ARGS );
    static	void	cb_set_glClipPlane5( CB_ARGS );
};

#endif /* _MODELCLIPPLANES_H */
