/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Feb 10 12:01:55 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:12:21 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "PlaneItem.h"

#include "GlWindow.h"

#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/PushB.h>

#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/manips/SoTransformBoxManip.h>
#include <Inventor/draggers/SoDragger.h>
#include <Inventor/nodes/SoResetTransform.h>

const char * const PlaneItem::newValue = "NewValue";

XtCallbackRec	PlaneItem::act_call[2] = {
    { &PlaneItem::activateCB, NULL },
    NULL
};
XtCallbackRec	PlaneItem::ver_call[2] = {
    { &PlaneItem::verifyCB, NULL },
    NULL
};


PlaneItem::PlaneItem( char *name, Widget parent, GlWindow *w )
: VkComponent( name )
{
    glw = w;

    dont_set_text = FALSE;
    dont_set_manip = FALSE;

    show_hide_state = FALSE;
    
    create_widget( name, parent );
    create_3d_plane();
}

PlaneItem::~PlaneItem()
{
    destroy_3d_plane();
}

void
PlaneItem::create_widget( char *name, Widget parent )
{
    int	ac;
    Arg	av[10];

    // create the container first
    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    rowcol = XmCreateRowColumn( parent,
			       name,
			       av, ac );
    XtManageChild( rowcol );


    label = XmCreateLabel( rowcol,
			  name,
			  NULL, 0 );
    XtManageChild( label );

    // fix the callback structs
    act_call[0].closure = this;
    ver_call[0].closure = this;

    // the text item
    ac = 0;
    XtSetArg( av[ac], XmNrows, 1 ); ac++;
    XtSetArg( av[ac], XmNcolumns, 30 ); ac++;
    XtSetArg( av[ac], XmNuserData, this ); ac++;
    XtSetArg( av[ac], XmNactivateCallback, act_call ); ac++;
    XtSetArg( av[ac], XmNmodifyVerifyCallback, ver_call ); ac++;
    ti = XmCreateText( rowcol,
		      name,
		      av, ac );
    XtManageChild( ti );

    // the pushbuttons
    pb_show_hide = XmCreatePushButton( rowcol,
				      "show plane",
				      NULL, 0 );
    XtAddCallback( pb_show_hide, XmNactivateCallback,
		  &PlaneItem::showhideCB, this );
    XtManageChild( pb_show_hide );
}

void
PlaneItem::set_ti_text( char *text )
{
    int	ac;
    Arg	av[10];

    // set the text
    ac = 0;
    XtSetArg( av[ac], XmNvalue, text ); ac++;
    XtSetValues( ti, av, ac );

    // and call the callbacks
    XtCallCallbacks( ti, XmNactivateCallback, this );
}

char *
PlaneItem::get_ti_text()
{
    char *text;
    int	ac;
    Arg	av[10];

    ac = 0;
    XtSetArg( av[ac], XmNvalue, &text ); ac++;
    XtGetValues( ti, av, ac );

    return text;
}

void
PlaneItem::set_plane( float A, float B, float C, float D )
{
    a = A;
    b = B;
    c = C;
    d = D;

    // let the text widget know
    if( dont_set_text == FALSE ) {
	char text[100];
	sprintf( text, "%1.4f %1.4f %1.4f %1.4f", a, b, c, d );
	if( dont_set_manip == FALSE ) {
	    dont_set_manip = TRUE;
	    set_ti_text( text );
	    dont_set_manip = FALSE;
	} else {
	    set_ti_text( text );
	}
    }

    // let the plane know
    update_3d_plane();
}

void
PlaneItem::get_plane( float *A, float *B, float *C, float *D )
{
    *A = a;
    *B = b;
    *C = c;
    *D = d;
}

void
PlaneItem::create_3d_plane()
{
    planeNode = new SoGroup( 2 );
    planeNode->ref();
    
    planeCube = new SoCube;
    planeCube->width.setValue( 3 );
    planeCube->depth.setValue( 3 );
    planeCube->height.setValue( .01 );
    planeCube->ref();

    planeManip = new SoTransformBoxManip;
    planeManip->getDragger()->
	addMotionCallback( (SoDraggerCB *) &PlaneItem::update_with_3d_info,
			  this );
    planeManip->ref();

    resetXform = new SoResetTransform;
    resetXform->ref();

    build_3d_group();
}

void
PlaneItem::build_3d_group()
{
    // finally add everything to the node
    planeNode->removeAllChildren();
    planeNode->addChild( resetXform );
    planeNode->addChild( planeManip );
    planeNode->addChild( planeCube );
}

void
PlaneItem::destroy_3d_plane()
{
    planeNode->unref();		// nukes children
}

void
PlaneItem::show_3d_plane()
{
    build_3d_group();
    
    glw->get_post_extender_node()->addChild( planeNode );
}

void
PlaneItem::update_3d_plane()
{
    if( dont_set_manip ) return;
    
    // let's normalize the plane normal
    float	L = sqrt( a*a + b*b + c*c );
    
    if( L == 0 ) return;	// invalid plane eqn
	
    float	overL = 1 / L;
    float	A = a * overL;
    float	B = b * overL;
    float	C = c * overL;

    // now create the matrix

    // the rotation axis is how we rotate [0,1,0] to the
    // normal of the plane
    SbRotation	rm( SbVec3f( 0, 1, 0 ), SbVec3f( A, B, C ) );
    SbVec3f	axis;
    float	radians;
    rm.getValue( axis, radians );

    // now set the manipulator
    planeManip->rotation.setValue( axis, radians );
    // translation is a a distance off the normal of the plane
    planeManip->translation.setValue( -A*overL*d, -B*overL*d, -C*overL*d );
}

void
PlaneItem::hide_3d_plane()
{
    glw->get_post_extender_node()->removeChild( planeNode );
}

void
PlaneItem::update_with_3d_info( PlaneItem *self )
{
    float	a, b, c, d;
    
    // get the axis and rotate [0,1,0] around it
    SbVec3f	axis;
    float	radians;
    SbVec3f	y_axis( 0, 1, 0 );
    SbVec3f	normal;

    self->planeManip->rotation.getValue( axis, radians );
    SbRotation	rm( axis, radians );
    rm.multVec( y_axis, normal );

    a = normal[0];
    b = normal[1];
    c = normal[2];

    // now find d.
    // project the translation into the normal
    SbVec3f	dist( self->planeManip->translation.getValue() );
    d = - ( dist[0]*normal[0] + dist[1]*normal[1] + dist[2]*normal[2] );

    // now notify and update
    self->dont_set_manip = TRUE;
    self->set_plane( a, b, c, d );
    self->dont_set_manip = FALSE;
}
    
    

/////////////////////////////////////
// callbacks
/////////////////////////////////////

void
PlaneItem::activateCB( Widget, XtPointer client, XtPointer )
{
    PlaneItem *self = (PlaneItem *) client;

    float	A, B, C, D;
    char *text = self->get_ti_text();
    
    sscanf( text, "%f %f %f %f", &A, &B, &C, &D );
    self->dont_set_text = TRUE;
    self->set_plane( A, B, C, D );
    self->dont_set_text = FALSE;

    // we pass ourselves as data..  client will grab data.
    self->callCallbacks( self->newValue, self );
}

void
PlaneItem::verifyCB( Widget, XtPointer client, XtPointer call )
{
    PlaneItem *self = (PlaneItem *) client;

    XmTextVerifyCallbackStruct *cb =
	(XmTextVerifyCallbackStruct *) call;

    int	i;
    char *text = cb->text->ptr;
    int	max = cb->text->length;
    char *acceptable_string = "-0123456789. ";
    for( i = 0; i < max; i++ ) {
	if( NULL == strchr( acceptable_string, text[i] ) ) {
	    // NOT!
	    cb->doit = FALSE;
	    return;
	}
    }
}

void
PlaneItem::showhideCB( Widget, XtPointer client, XtPointer )
{
    PlaneItem *self = (PlaneItem *) client;

    XmString	str;

    switch( self->show_hide_state ) {
      case FALSE:
	self->show_3d_plane();
	self->show_hide_state = TRUE;
	str = XmStringCreateLocalized( "hide plane" );
	break;

      case TRUE:
	self->hide_3d_plane();
	self->show_hide_state = FALSE;
	str = XmStringCreateLocalized( "show plane" );
	break;
    }

    int	ac = 0;
    Arg	av[10];

    XtSetArg( av[ac], XmNlabelString, str ); ac++;
    XtSetValues( self->pb_show_hide, av, ac );
}

