/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _PLANEITEM_H
#define _PLANEITEM_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Feb 10 11:58:46 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:06 1994 by George Kyriazis
 *
 * a plane that has a visual representation as a text item
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#define Object XObject
#include <X11/Intrinsic.h>
#undef Object

#include "Object.h"
#include <Vk/VkComponent.h>
class GlWindow;

class SoGroup;
class SoTransformBoxManip;
class SoCube;
class SoResetTransform;

class PlaneItem : public Object, public VkComponent {
  public:
    PlaneItem( char *name, Widget parent, GlWindow * );
    virtual ~PlaneItem();

    // the plane equation
    virtual	void	set_plane( float a, float b, float c, float d );
    virtual	void	get_plane( float *a, float *b,
				  float *c, float *d );

    // callback
    static const char *const newValue;

  protected:
    virtual	void	create_widget( char *name, Widget parent );

    void	set_ti_text( char * );
    char *	get_ti_text();

    Boolean	dont_set_text;
    Boolean	dont_set_manip;

    void	create_3d_plane();
    void	destroy_3d_plane();
    void	show_3d_plane();
    void	update_3d_plane();	// inform that manipulator
    void	hide_3d_plane();
    void	build_3d_group();

    static	void	update_with_3d_info( PlaneItem * );// the manip moved

    Widget	rowcol;
    Widget	label;
    Widget	ti;
    Widget	pb_show_hide;
    Boolean	show_hide_state;

    GlWindow	*glw;

    // this is all the inventor stuff for the 3d plane
    SoGroup	*planeNode;
    SoCube	*planeCube;
    SoTransformBoxManip *planeManip;
    SoResetTransform	*resetXform;

    // the plane coeffients
    float	a, b, c, d;

    // callbacks
    static	void	activateCB( Widget w, XtPointer, XtPointer );
    static	void	verifyCB( Widget w, XtPointer, XtPointer );
    static	void	showhideCB( Widget w, XtPointer, XtPointer );

    // don't touch this
    static	XtCallbackRec	act_call[2];
    static	XtCallbackRec	ver_call[2];
};

#endif /* _PLANEITEM_H */
