/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Tue Jan 11 13:04:37 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:12:32 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "RadioBoxItem.h"

#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>

#include <Vk/VkApp.h>
#include <Vk/VkRadioBox.h>

#include <stdio.h>

const char * const RadioBoxItem::newValue = "NewValue";

RadioBoxItem::RadioBoxItem( char *Name, Widget Parent, int n_choices )
: VkComponent( Name ),

  name( Name ),
  parent( Parent )
{
    number = n_choices;
    
    // allocate strings
    str = new mystring[number];
    xmstr = new XmString[number];
    // and name it some stupid names
    char	n[80];
    int	i;
    for( i = 0; i < number; i++ ) {
	sprintf( n, "button%d", i );
	xmstr[i] = XmStringCreateLocalized( n );
    }

    // allocate return values
    ret = new int[number];

    // allocate the option buttons
    buttons = new Widget[number];
}

RadioBoxItem::~RadioBoxItem()
{
    delete [] xmstr;
    delete [] str;
    delete [] ret;
    delete [] buttons;
}

void
RadioBoxItem::end()
{
    create_widget( name, parent );
}

void
RadioBoxItem::create_widget( char *name, Widget parent )
{
    if( number > 3 ) {
	create_menu( name, parent );
    } else {
	create_radio( name, parent );
    }
}

void
RadioBoxItem::create_radio( char *name, Widget parent )
{
    int	ac;
    Arg	av[10];

    // create the container first
    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    rowcol = XmCreateRowColumn( parent,
			       name, av, ac );
    XtManageChild( rowcol );

    label = XmCreateLabel( rowcol,
			  name,
			  NULL, 0 );
    XtManageChild( label );

    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    XtSetArg( av[ac], XmNuserData, this ); ac++;
    XtSetArg( av[ac], XmNpacking, XmPACK_TIGHT ); ac++;
    rb = XmCreateRadioBox( rowcol,
			  name,
			  av, ac );

    int	i;
    for( i = 0; i < number; i++ ) {
	ac = 0;
	XtSetArg( av[ac], XmNlabelString, xmstr[i] ); ac++;
	if( i == initial_value ) {
	    XtSetArg( av[ac], XmNset, TRUE ); ac++;
	}
	buttons[i] = XtCreateManagedWidget( str[i],
					   xmToggleButtonWidgetClass,
					   rb,
					   av, ac );
	XtAddCallback( buttons[i],
		      XmNarmCallback, &RadioBoxItem::buttonCB,
		      (XtPointer) i );
    }

    XtManageChild( rb );
}

void
RadioBoxItem::create_menu( char *name, Widget parent )
{
    int	ac;
    Arg	av[10];
    int	i;

    // create the container first
    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    rowcol = XmCreateRowColumn( parent,
			       name,
			       av, ac );
    XtManageChild( rowcol );

    ac = 0;
    label = XmCreateLabel( rowcol,
			  name,
			  av, ac );
    XtManageChild( label );

    // now create the pulldown
    ac = 0;
    XtSetArg( av[ac], XmNuserData, this ); ac++;
    pulldown = XmCreatePulldownMenu( rowcol,
				   name,
				   av, ac );

    // ..with it's button widgets..
    Widget	w;
    Widget	init;
    for( i = 0; i < number; i++ ) {
	ac = 0;
	if( i ) {
	    XtSetArg( av[ac], XmNrecomputeSize, False ); ac++;
	}
	w = XtCreateManagedWidget( str[i],
				  xmPushButtonWidgetClass,
				  pulldown,
				  av, ac );
	XtAddCallback( w, "activateCallback",
		      &RadioBoxItem::buttonCB, (XtPointer) i );
	if( i == initial_value ) {
	    init = w;
	}
    }

    // ..and the option widget too!
    ac = 0;
    XtSetArg( av[ac], XmNsubMenuId, pulldown ); ac++;
    XtSetArg( av[ac], XmNmenuHistory, init ); ac++;
    rb = XmCreateOptionMenu( rowcol,
			    name,
			    av, ac );
    XtManageChild( rb );
    
}


void
RadioBoxItem::set( int index, char *name, int return_value )
{
    strcpy( str[index], name );
    xmstr[index] = XmStringCreateLocalized( name );
    ret[index] = return_value;
}

void
RadioBoxItem::set_initial_value( int index )
{
    if( index >= 0 ) {
	initial_value = index;
    }
}

void
RadioBoxItem::set_value( int index )
{
    int	ac;
    Arg	av[10];

    if( number > 3 ) {
	// we have a menu
	ac = 0;
	XtSetArg( av[ac], XmNmenuHistory, index ); ac++;
	XtSetValues( rb, av, ac );
    } else {
	// we have a radio
	int	i;
	for( i = 0; i < number; i++ ) {
	    ac = 0;
	    XtSetArg( av[ac], XmNset, FALSE ); ac++;
	    XtSetValues( buttons[i], av, ac );
	}
	ac = 0;
	XtSetArg( av[ac], XmNset, TRUE ); ac++;
	XtSetValues( buttons[index], av, ac );
    }
}


void
RadioBoxItem::buttonCB( Widget w, XtPointer client_data, XtPointer )
{
    int	ac;
    Arg	av[10];
    
    RadioBoxItem *self;

    // get our instance from the rowcol widget
    ac = 0;
    XtSetArg( av[ac], XmNuserData, &self ); ac++;
    XtGetValues( XtParent(w), av, ac );

    // now call the callback
    self->button_cb( w, client_data );
}

void
RadioBoxItem::button_cb( Widget, XtPointer call_data )
{
    int	index = (int) call_data;

    // now call the callbacks
    callCallbacks( newValue, (void *) (ret[index]) );
}
