/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Jan 20 15:11:14 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:14:00 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "RasterOps.h"

#include "IfaceGeneratorP.h"

#include "TextItem.h"
#include <stdlib.h>

int RasterOps::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Rasterization Operations", 1,

    IF_RADIO, IF_THIS, NULL,
    (int) "Cull Mode: ", 3,
    1, (int) &RasterOps::cb_set_glCullFace,
    (int) "GL_FRONT", GL_FRONT,
    (int) "GL_BACK", GL_BACK,
    (int) "GL_FRONT_AND_BACK", GL_FRONT_AND_BACK,

    IF_RADIO, IF_THIS, NULL,
    (int) "Culling: ", 2,
    0, (int) &RasterOps::cb_set_cull_face,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Front Polygon Mode: ", 3,
    2, (int) &RasterOps::cb_set_glPolygonMode_front,
    (int) "GL_POINT", GL_POINT,
    (int) "GL_LINE", GL_LINE,
    (int) "GL_FILL", GL_FILL,

    IF_RADIO, IF_THIS, NULL,
    (int) "Back Polygon Mode: ", 3,
    2, (int) &RasterOps::cb_set_glPolygonMode_back,
    (int) "GL_POINT", GL_POINT,
    (int) "GL_LINE", GL_LINE,
    (int) "GL_FILL", GL_FILL,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Polygon Stipple: ", 2,
    0, (int) &RasterOps::cb_set_polygon_stipple,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_MULTI_TEXT, IF_THIS, NULL,
    (int) "Polygon stipple pattern: ",
    32, 8, IF_VALUE+1234,
    (int) &TextItem::HEX,
    (int) &RasterOps::cb_set_glPolygonStipple,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_TEXT, IF_THIS, NULL,
    (int) "Line Width: ",
    5, IF_VALUE+0,
    (int) &TextItem::FRACTIONAL,
    (int) &RasterOps::cb_set_glLineWidth,

    IF_RADIO, IF_THIS, NULL,
    (int) "Line Stipple: ", 2,
    0, (int) &RasterOps::cb_set_line_stipple,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_TEXT, IF_THIS, NULL,
    (int) "Line Stipple Factor: ",
    5, IF_VALUE+1,
    (int) &TextItem::INTEGER,
    (int) &RasterOps::cb_set_glLineStipple_factor,

    IF_TEXT, IF_THIS, NULL,
    (int) "Line Stipple Pattern: ",
    5, IF_VALUE+2,
    (int) &TextItem::HEX,
    (int) &RasterOps::cb_set_glLineStipple_pattern,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_TEXT, IF_THIS, NULL,
    (int) "Point Size: ",
    5, IF_VALUE+3,
    (int) &TextItem::FRACTIONAL,
    (int) &RasterOps::cb_set_glPointSize,

    IF_END
};


RasterOps::RasterOps( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();
}

RasterOps::~RasterOps()
{
}

///////////////////////////////////////////////////////////////////

void
RasterOps::install_values()
{
    static char *text = "ffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff\nffffffffffffffffffffffffffffffff";
    install( iface, IF_VALUE+1234, (int) text );
    
    GLfloat	width;
    get_glLineWidth( &width );
    install( iface, IF_VALUE+0, (int) ftoa(width) );

    GLint	factor;
    GLushort	pattern;
    get_glLineStipple( &factor, &pattern );
    install( iface, IF_VALUE+1, (int) itoa(factor) );
    install( iface, IF_VALUE+2, (int) itox(pattern) );

    GLfloat	size;
    get_glPointSize( &size );
    install( iface, IF_VALUE+3, (int) ftoa(size) );
}

void
RasterOps::remember()
{
    GLenum	e;
    GLfloat	f;
    GLint	i;
    GLushort	us;
    Boolean	b;

    get_glCullFace( &e );
    get_cull_face( &b );
    get_glPolygonMode_front( &e );
    get_glPolygonMode_back( &e );
    get_polygon_stipple( &b );
    get_glLineWidth( &f );
    get_glLineStipple( &i, &us );
    get_line_stipple( &b );
    get_glPointSize( &f );
}


///////////////////////////////////////////////////////////////////


void
RasterOps::set_glCullFace( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_FRONT:
	    s = "GL_FRONT";
	    break;
	  case GL_BACK:
	    s = "GL_BACK";
	    break;
	  case GL_FRONT_AND_BACK:
	    s = "GL_FRONT_AND_BACK";
	    break;
	}

	printf("glCullFace %s\n", s );
    }

    glCullFace( mode );

    _glCullFace = mode;

    redraw();
}

void
RasterOps::get_glCullFace( GLenum *mode )
{
    glGetIntegerv( GL_CULL_FACE_MODE, (int *) mode );

    _glCullFace = *mode;
}

void
RasterOps::set_cull_face( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_CULL_FACE\n");
	} else {
	    printf("glDisable GL_CULL_FACE\n");
	}
    }

    switch_flag( GL_CULL_FACE, bool );

    _cull_face = bool;

    redraw();
}

void
RasterOps::get_cull_face( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_CULL_FACE );

    _cull_face = state;

    *bool = state;
}

void
RasterOps::set_glPolygonMode_front( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_POINT:
	    s = "GL_POINT";
	    break;
	  case GL_LINE:
	    s = "GL_LINE";
	    break;
	  case GL_FILL:
	    s = "GL_FILL";
	    break;
	}

	printf("glPolygonMode GL_FRONT %s\n", s );
    }

    glPolygonMode( GL_FRONT, mode );

    _glPolygonMode_front = mode;

    redraw();
}

void
RasterOps::get_glPolygonMode_front( GLenum *mode )
{
    int	v[2];

    glGetIntegerv( GL_POLYGON_MODE, v );

    *mode = v[0];

    _glPolygonMode_front = *mode;
}

void
RasterOps::set_glPolygonMode_back( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_POINT:
	    s = "GL_POINT";
	    break;
	  case GL_LINE:
	    s = "GL_LINE";
	    break;
	  case GL_FILL:
	    s = "GL_FILL";
	    break;
	}

	printf("glPolygonMode GL_BACK %s\n", s );
    }

    glPolygonMode( GL_BACK, mode );

    _glPolygonMode_back = mode;

    redraw();
}

void
RasterOps::get_glPolygonMode_back( GLenum *mode )
{
    int	v[2];

    glGetIntegerv( GL_POLYGON_MODE, v );

    *mode = v[1];

    _glPolygonMode_back = *mode;
}

void
RasterOps::set_glPolygonMode_front_and_back( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_POINT:
	    s = "GL_POINT";
	    break;
	  case GL_LINE:
	    s = "GL_LINE";
	    break;
	  case GL_FILL:
	    s = "GL_FILL";
	    break;
	}

	printf("glPolygonMode GL_FRONT_AND_BACK %s\n", s );
    }

    glPolygonMode( GL_FRONT_AND_BACK, mode );

    _glPolygonMode_front = mode;
    _glPolygonMode_back = mode;

    redraw();
}

void
RasterOps::set_polygon_stipple( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_POLYGON_STIPPLE\n");
	} else {
	    printf("glDisable GL_POLYGON_STIPPLE\n");
	}
    }

    switch_flag( GL_POLYGON_STIPPLE, bool );

    _polygon_stipple = bool;

    redraw();
}

void
RasterOps::get_polygon_stipple( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_POLYGON_STIPPLE );

    _polygon_stipple = state;

    *bool = state;
}

void
RasterOps::set_glPolygonStipple( GLubyte *mask )
{
    if( debuglevel > 0 ) {
	int	i;
	char	text[512];
	int	j = 0;
	static	char *hexconv = "0123456789abcdef";

	for( i = 0; i < 128; i++ ) {
	    text[j++] = hexconv[ (mask[i] & 0xf0) >> 4 ];
	    text[j++] = hexconv[ mask[i] & 0xf ];
	}
	text[j] = '\0';

	printf("glPolygonStipple %s\n", text );
    }

    glPolygonStipple( mask );

    _glPolygonStipple = mask;

    redraw();
}

void
RasterOps::set_glLineWidth( GLfloat width )
{
    if( debuglevel > 0 ) {
	printf("glLineWidth %f\n", width );
    }

    glLineWidth( width );

    _glLineWidth = width;

    redraw();
}

void
RasterOps::get_glLineWidth( GLfloat *width )
{
    glGetFloatv( GL_LINE_WIDTH, width );

    _glLineWidth = *width;
}

void
RasterOps::set_glLineStipple( GLint factor, GLushort pattern )
{
    if( debuglevel > 0 ) {
	printf("glLineStipple %d 0x%x\n", factor, pattern );
    }

    glLineStipple( factor, pattern );

    _glLineStipple_factor = factor;
    _glLineStipple_pattern = pattern;

    redraw();
}

void
RasterOps::get_glLineStipple( GLint *factor, GLushort *pattern )
{
    glGetIntegerv( GL_LINE_STIPPLE_REPEAT, factor );

    int	patt;
    glGetIntegerv( GL_LINE_STIPPLE_PATTERN, &patt );
    *pattern = patt;

    _glLineStipple_factor = *factor;
    _glLineStipple_pattern = *pattern;
}

void
RasterOps::set_line_stipple( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_LINE_STIPPLE\n");
	} else {
	    printf("glDisable GL_LINE_STIPPLE\n");
	}
    }

    switch_flag( GL_LINE_STIPPLE, bool );

    _line_stipple = bool;

    redraw();
}

void
RasterOps::get_line_stipple( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_LINE_STIPPLE );

    _line_stipple = state;

    *bool = state;
}

void
RasterOps::set_glPointSize( GLfloat size )
{
    if( debuglevel > 0 ) {
	printf("glPointSize %f\n", size );
    }

    glPointSize( size );

    _glPointSize = size;

    redraw();
}

void
RasterOps::get_glPointSize( GLfloat *size )
{
    glGetFloatv( GL_POINT_SIZE, size );

    _glPointSize = *size;
}


void
RasterOps::reset()
{
    int	debugstack;
    Boolean redrawstack;
    
    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }

    set_glCullFace( _glCullFace );
    set_cull_face( _cull_face );
    set_glPolygonMode_front( _glPolygonMode_front );
    set_glPolygonMode_back( _glPolygonMode_back );
    set_polygon_stipple( _polygon_stipple );
    set_glLineWidth( _glLineWidth );
    set_glLineStipple( _glLineStipple_factor, _glLineStipple_pattern );
    set_line_stipple( _line_stipple );
    set_glPointSize( _glPointSize );

    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}


///////////////////////////////////////////////////////////////////

void
RasterOps::cb_set_glCullFace( VkCallbackObject *, void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLenum mode = (GLenum) call;

    self->set_glCullFace( mode );
}

void
RasterOps::cb_set_cull_face( VkCallbackObject *, void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    int	state = (int) call;

    self->set_cull_face( state );
}

void
RasterOps::cb_set_glPolygonMode_front( VkCallbackObject *,
				      void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLenum mode = (GLenum) call;

    self->set_glPolygonMode_front( mode );
}

void
RasterOps::cb_set_glPolygonMode_back( VkCallbackObject *,
				      void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLenum mode = (GLenum) call;

    self->set_glPolygonMode_back( mode );
}

void
RasterOps::cb_set_polygon_stipple( VkCallbackObject *,
				  void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    int	bool = (int) call;

    self->set_polygon_stipple( bool );
}

void
RasterOps::cb_set_glPolygonStipple( VkCallbackObject *,
				   void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    char *text = (char *) call;

    // convert the text into hex
    static	char	*hexconv = "0123456789abcdef";
    char foo;
    GLubyte mask[128];
    int	i = 0;
    while( foo = *text ) {
	if( foo == '\n' ) { text++; continue; } // filter out returns
	mask[i] = strchr( hexconv, tolower(foo) ) - hexconv;
	mask[i] <<= 4;
	text++; foo = *text;
	mask[i] += strchr( hexconv, tolower(foo) ) - hexconv;
	text++;
	i++; if( i >= 128 ) break;
    }

    self->set_glPolygonStipple( mask );
}

void
RasterOps::cb_set_glLineWidth( VkCallbackObject *,
			      void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLfloat width = atof( (char *)call );

    self->set_glLineWidth( width );
}

void
RasterOps::cb_set_glLineStipple_factor( VkCallbackObject *,
				       void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLint new_factor = atoi( (char *)call );

    GLint	factor;
    GLushort	pattern;

    self->get_glLineStipple( &factor, &pattern );
    self->set_glLineStipple( new_factor, pattern );
}

void
RasterOps::cb_set_glLineStipple_pattern( VkCallbackObject *,
				       void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLint new_pattern = xtoi( (char *)call );

    GLint	factor;
    GLushort	pattern;

    self->get_glLineStipple( &factor, &pattern );
    self->set_glLineStipple( factor, new_pattern );
}

void
RasterOps::cb_set_line_stipple( VkCallbackObject *, void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    int bool = (int) call;

    self->set_line_stipple( bool );
}

void
RasterOps::cb_set_glPointSize( VkCallbackObject *, void *client, void *call )
{
    RasterOps *self = (RasterOps *) client;
    GLfloat	size = atof( (char *)call );

    self->set_glPointSize( size );
}

