/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _RASTEROPS_H
#define _RASTEROPS_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Jan 20 15:10:09 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:58 1994 by George Kyriazis
 *
 * Rasterization operations
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class RasterOps : public IfaceGenerator {
  public:
    RasterOps( GlWindow * );
    virtual ~RasterOps();

    // face culling
    void	set_glCullFace( GLenum mode );
    void	get_glCullFace( GLenum *mode );
    void	set_cull_face( Boolean );
    void	get_cull_face( Boolean * );

    // polygon mode
    void	set_glPolygonMode_front( GLenum mode );
    void	get_glPolygonMode_front( GLenum *mode );
    void	set_glPolygonMode_back( GLenum mode );
    void	get_glPolygonMode_back( GLenum *mode );
    void	set_glPolygonMode_front_and_back( GLenum mode );

    // stipple
    void	set_polygon_stipple( Boolean bool );
    void	get_polygon_stipple( Boolean * );
    void	set_glPolygonStipple( GLubyte *mask );
    void	get_glPolygonStipple( GLubyte **mask );
    void	set_glLineWidth( GLfloat width );
    void	get_glLineWidth( GLfloat *width );
    void	set_glLineStipple( GLint factor, GLushort pattern );
    void	get_glLineStipple( GLint *factor, GLushort *pattern );
    void	set_line_stipple( Boolean bool );
    void	get_line_stipple( Boolean * );
    void	set_glPointSize( GLfloat size );
    void	get_glPointSize( GLfloat *size );

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // local vars
    GLenum	_glCullFace;
    Boolean	_cull_face;
    GLenum	_glPolygonMode_front;
    GLenum	_glPolygonMode_back;
    Boolean	_polygon_stipple;
    GLubyte	*_glPolygonStipple;
    GLfloat	_glLineWidth;
    GLint	_glLineStipple_factor;
    GLushort	_glLineStipple_pattern;
    Boolean	_line_stipple;
    GLfloat	_glPointSize;

    // callbacks
    static	void	cb_set_glCullFace( CB_ARGS );
    static	void	cb_set_cull_face( CB_ARGS );
    static	void	cb_set_glPolygonMode_front( CB_ARGS );
    static	void	cb_set_glPolygonMode_back( CB_ARGS );
    static	void	cb_set_polygon_stipple( CB_ARGS );
    static	void	cb_set_glPolygonStipple( CB_ARGS );
    static	void	cb_set_glLineWidth( CB_ARGS );
    static	void	cb_set_glLineStipple_factor( CB_ARGS );
    static	void	cb_set_glLineStipple_pattern( CB_ARGS );
    static	void	cb_set_line_stipple( CB_ARGS );
    static	void	cb_set_glPointSize( CB_ARGS );
};

#endif /* _RASTEROPS_H */
