/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _TEXGENOPS_H
#define _TEXGENOPS_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Tue Feb  8 11:10:35 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:14:06 1994 by George Kyriazis
 *
 * Texture generation operations
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class TexGenOps : public IfaceGenerator {
  public:
    TexGenOps( GlWindow * );
    virtual ~TexGenOps();

    // texture generation
    void	set_texture( GLenum, Boolean );
    void	get_texture( GLenum, Boolean * );
    void	set_glTexGen_gen_mode( GLenum, GLenum );
    void	get_glTexGen_gen_mode( GLenum, GLenum * );
    void	set_glTexGen_obj_plane( GLenum,
				       GLfloat x,
				       GLfloat y,
				       GLfloat z,
				       GLfloat w );
    void	get_glTexGen_obj_plane( GLenum,
				       GLfloat *x,
				       GLfloat *y,
				       GLfloat *z,
				       GLfloat *w );
    void	set_glTexGen_eye_plane( GLenum,
				       GLfloat x,
				       GLfloat y,
				       GLfloat z,
				       GLfloat w );
    void	get_glTexGen_eye_plane( GLenum,
				       GLfloat *x,
				       GLfloat *y,
				       GLfloat *z,
				       GLfloat *w );

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // local state
    Boolean	_texture_s;
    GLenum	_glTexGen_gen_mode_s;
    GLfloat	_glTexGen_obj_plane_x_s;
    GLfloat	_glTexGen_obj_plane_y_s;
    GLfloat	_glTexGen_obj_plane_z_s;
    GLfloat	_glTexGen_obj_plane_w_s;
    GLfloat	_glTexGen_eye_plane_x_s;
    GLfloat	_glTexGen_eye_plane_y_s;
    GLfloat	_glTexGen_eye_plane_z_s;
    GLfloat	_glTexGen_eye_plane_w_s;
    Boolean	_texture_t;
    GLenum	_glTexGen_gen_mode_t;
    GLfloat	_glTexGen_obj_plane_x_t;
    GLfloat	_glTexGen_obj_plane_y_t;
    GLfloat	_glTexGen_obj_plane_z_t;
    GLfloat	_glTexGen_obj_plane_w_t;
    GLfloat	_glTexGen_eye_plane_x_t;
    GLfloat	_glTexGen_eye_plane_y_t;
    GLfloat	_glTexGen_eye_plane_z_t;
    GLfloat	_glTexGen_eye_plane_w_t;
    Boolean	_texture_r;
    GLenum	_glTexGen_gen_mode_r;
    GLfloat	_glTexGen_obj_plane_x_r;
    GLfloat	_glTexGen_obj_plane_y_r;
    GLfloat	_glTexGen_obj_plane_z_r;
    GLfloat	_glTexGen_obj_plane_w_r;
    GLfloat	_glTexGen_eye_plane_x_r;
    GLfloat	_glTexGen_eye_plane_y_r;
    GLfloat	_glTexGen_eye_plane_z_r;
    GLfloat	_glTexGen_eye_plane_w_r;
    Boolean	_texture_q;
    GLenum	_glTexGen_gen_mode_q;
    GLfloat	_glTexGen_obj_plane_x_q;
    GLfloat	_glTexGen_obj_plane_y_q;
    GLfloat	_glTexGen_obj_plane_z_q;
    GLfloat	_glTexGen_obj_plane_w_q;
    GLfloat	_glTexGen_eye_plane_x_q;
    GLfloat	_glTexGen_eye_plane_y_q;
    GLfloat	_glTexGen_eye_plane_z_q;
    GLfloat	_glTexGen_eye_plane_w_q;

    // callbacks
    static	void	cb_set_texture_s( CB_ARGS );
    static	void	cb_set_texture_t( CB_ARGS );
    static	void	cb_set_texture_r( CB_ARGS );
    static	void	cb_set_texture_q( CB_ARGS );
    static	void	cb_set_glTexGen_s_gen_mode( CB_ARGS );
    static	void	cb_set_glTexGen_t_gen_mode( CB_ARGS );
    static	void	cb_set_glTexGen_r_gen_mode( CB_ARGS );
    static	void	cb_set_glTexGen_q_gen_mode( CB_ARGS );
    static	void	cb_set_glTexGen_s_obj_plane( CB_ARGS );
    static	void	cb_set_glTexGen_t_obj_plane( CB_ARGS );
    static	void	cb_set_glTexGen_r_obj_plane( CB_ARGS );
    static	void	cb_set_glTexGen_q_obj_plane( CB_ARGS );
    static	void	cb_set_glTexGen_s_eye_plane( CB_ARGS );
    static	void	cb_set_glTexGen_t_eye_plane( CB_ARGS );
    static	void	cb_set_glTexGen_r_eye_plane( CB_ARGS );
    static	void	cb_set_glTexGen_q_eye_plane( CB_ARGS );

};

#endif /* _TEXGENOPS_H */
