/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Jan 13 10:36:34 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:12:36 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "TextItem.h"

#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/Text.h>

const char * const TextItem::newValue = "NewValue";
const char * const TextItem::BINARY = "01\n";
const char * const TextItem::INTEGER = "0123456789\n";
const char * const TextItem::NUMERIC = "0123456789-\n";
const char * const TextItem::FRACTIONAL = "0123456789-.\n";
const char * const TextItem::VECTOR = "01234567890-. \n";
const char * const TextItem::INTVECTOR = "01234567890- \n";
const char * const TextItem::HEX = "0123456789aAbBcCdDeEfF\n";
const char * const TextItem::ALPHA =
	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\n";
const char * const TextItem::ALPHANUMERIC =
	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789\n";

XtCallbackRec	TextItem::act_call[2] = {
    { &TextItem::activateCB, NULL },
    NULL
};
XtCallbackRec	TextItem::ver_call[2] = {
    { &TextItem::verifyCB, NULL },
    NULL
};

TextItem::TextItem( char *Name, Widget Parent, int size, int rows )
: VkComponent( Name )
{
    create_widget( Name, Parent );

    set_size( size );
    set_rows( rows );
    set_accepted( ALPHANUMERIC );
}

TextItem::~TextItem()
{
}


void
TextItem::create_widget( char *name, Widget parent )
{
    int	ac;
    Arg	av[10];

    // create the container first
    ac = 0;
    XtSetArg( av[ac], XmNorientation, XmHORIZONTAL ); ac++;
    rowcol = XmCreateRowColumn( parent,
			       name,
			       av, ac );
    XtManageChild( rowcol );

    label = XmCreateLabel( rowcol,
			  name,
			  NULL, 0 );
    XtManageChild( label );

    // fix the callback lists
    act_call[0].closure = this;
    ver_call[0].closure = this;

    // now the text item
    ac = 0;
    XtSetArg( av[ac], XmNrows, 1 ); ac++;
    XtSetArg( av[ac], XmNcolumns, 10 ); ac++;
    XtSetArg( av[ac], XmNuserData, this ); ac++;
    XtSetArg( av[ac], XmNactivateCallback, act_call ); ac++;
    XtSetArg( av[ac], XmNmodifyVerifyCallback, ver_call ); ac++;
    ti = XmCreateText( rowcol,
		      name,
		      av, ac );
    XtManageChild( ti );
}


void
TextItem::set_text( char *text )
{
    int	ac;
    Arg	av[10];

    ac = 0;
    XtSetArg( av[ac], XmNvalue, text ); ac++;
    XtSetValues( ti, av, ac );
}

char *
TextItem::get_text()
{
    char	*text;
    int	ac;
    Arg	av[10];

    ac = 0;
    XtSetArg( av[ac], XmNvalue, &text ); ac++;
    XtGetValues( ti, av, ac );

    return text;
}

void
TextItem::set_size( int s )
{
    int	ac;
    Arg	av[10];

    ac = 0;
    XtSetArg( av[ac], XmNcolumns, s ); ac++;
    XtSetValues( ti, av, ac );
}

int
TextItem::get_size()
{
    int	ac;
    Arg	av[10];
    int	s;

    ac = 0;
    XtSetArg( av[ac], XmNcolumns, &s ); ac++;
    XtGetValues( ti, av, ac );

    return s;
}

void
TextItem::set_rows( int r )
{
    int	ac;
    Arg	av[10];

    ac = 0;
    XtSetArg( av[ac], XmNrows, r ); ac++;
    if( r == 1 ) {
	XtSetArg( av[ac], XmNeditMode, XmSINGLE_LINE_EDIT ); ac++;
    } else {
	XtSetArg( av[ac], XmNeditMode, XmMULTI_LINE_EDIT ); ac++;
    }
    XtSetValues( ti, av, ac );
}

int
TextItem::get_rows()
{
    int	ac;
    Arg	av[10];
    int	r;

    ac = 0;
    XtSetArg( av[ac], XmNrows, &r ); ac++;
    XtGetValues( ti, av, ac );

    return r;
}

void
TextItem::activateCB( Widget w, XtPointer client_data, XtPointer call_data )
{
    TextItem *self = (TextItem *) client_data;

    self->activate_cb( w, call_data );
}

void
TextItem::activate_cb( Widget, XtPointer )
{
    // get the text and pass it as call data
    callCallbacks( newValue, get_text() );
}

void
TextItem::verifyCB( Widget w, XtPointer client_data, XtPointer call_data )
{
    TextItem *self = (TextItem *) client_data;

    self->verify_cb( w, call_data );
}

void
TextItem::verify_cb( Widget, XtPointer call_data )
{
    XmTextVerifyCallbackStruct *cb =
	(XmTextVerifyCallbackStruct *) call_data;

    int	i;
    char *text = cb->text->ptr;
    int max = cb->text->length;
    for( i = 0; i < max; i++ ) {
	if( NULL == strchr( acceptable_string, text[i] ) ) {
	    // we are hosed.  Not in string.
	    cb->doit = FALSE;
	    return;
	}
    }
}

void
TextItem::set_accepted( const char *acc )
{
    acceptable_string = acc;
}
