/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _TEXTITEM_H
#define _TEXTITEM_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Thu Jan 13 10:33:37 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:23 1994 by George Kyriazis
 *
 * a C++ encapsulation of a text item
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

// X defines Object
#define Object XObject
#include <X11/Intrinsic.h>
#undef Object

#include "Object.h"
#include <Vk/VkComponent.h>

class TextItem : public Object, public VkComponent {
  public:
    TextItem( char *name, Widget parent, int size = 10, int rows = 1 );
    virtual ~TextItem();

    // get/set the text of the item
    virtual	void	set_text( char * );
    virtual	char *	get_text();

    // get/set the size of the item
    virtual	void	set_size( int s );
    virtual	int	get_size();

    // get/set the number or rows
    virtual	void	set_rows( int r );
    virtual	int	get_rows();

    // what type of things are acceptable?
    virtual	void	set_accepted( const char * );

    // callback
    static const char *const newValue;

    // constants for what type of string is accepted.
    static const char *const BINARY;
    static const char *const INTEGER;
    static const char *const NUMERIC;
    static const char *const HEX;
    static const char *const FRACTIONAL;
    static const char *const ALPHA;
    static const char *const ALPHANUMERIC;
    static const char *const VECTOR;
    static const char *const INTVECTOR;

  protected:
    virtual	void	create_widget( char *name, Widget parent );

    Widget	rowcol;
    Widget	label;
    Widget	ti;

    // callbacks
    static	void	activateCB( Widget w, XtPointer, XtPointer );
    static	void	verifyCB( Widget w, XtPointer, XtPointer );
    void	activate_cb( Widget w, XtPointer );
    void	verify_cb( Widget w, XtPointer );

    const char	*acceptable_string;

    // don't touch this
    static	XtCallbackRec	act_call[2];
    static	XtCallbackRec	ver_call[2];
};

#endif /* _TEXTITEM_H */
