/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Mon Jan 10 11:02:54 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:14:17 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "TextureOps.h"

#include "IfaceGeneratorP.h"

#include "TextItem.h"
#include <stdlib.h>

#include <GL/glu.h>

// the following is used to read in the texture image
// taken from Inventor 2.0, .../nodes/SoTexture2.c++
extern "C" {

typedef struct {
    unsigned short      imagic;         /* stuff saved on disk . . */
    unsigned short      type;
    unsigned short      dim;
    unsigned short      xsize;
    unsigned short      ysize;
    unsigned short      zsize;
    unsigned long       min;
    unsigned long       max;
    unsigned long       wastebytes;     
    char                name[80];
    unsigned long       colormap;
} IMAGE;
extern IMAGE *fiopen(int fd, const char *mode);
extern int getrow(IMAGE *, short *, int, int);
extern void iclose(IMAGE *);
};  /* end of extern "C" */


int TextureOps::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Texture Operations", 1,

    IF_RADIO, IF_THIS, NULL,
    (int) "1D Textures: ", 2,
    0, (int) &TextureOps::cb_set_texture_1d,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "1D Min Filter: ", 6,
    4, (int) &TextureOps::cb_set_glTexParameter_1d_min_filter,
    (int) "GL_NEAREST", GL_NEAREST,
    (int) "GL_LINEAR", GL_LINEAR,
    (int) "GL_NEAREST_MIPMAP_NEAREST", GL_NEAREST_MIPMAP_NEAREST,
    (int) "GL_LINEAR_MIPMAP_NEAREST", GL_LINEAR_MIPMAP_NEAREST,
    (int) "GL_NEAREST_MIPMAP_LINEAR", GL_NEAREST_MIPMAP_LINEAR,
    (int) "GL_LINEAR_MIPMAP_LINEAR", GL_LINEAR_MIPMAP_LINEAR,

    IF_RADIO, IF_THIS, NULL,
    (int) "1D Mag Filter: ", 2,
    1, (int) &TextureOps::cb_set_glTexParameter_1d_mag_filter,
    (int) "GL_NEAREST", GL_NEAREST,
    (int) "GL_LINEAR", GL_LINEAR,

    IF_RADIO, IF_THIS, NULL,
    (int) "1D Wrap S: ", 2,
    1, (int) &TextureOps::cb_set_glTexParameter_1d_wrap_s,
    (int) "GL_CLAMP", GL_CLAMP,
    (int) "GL_REPEAT", GL_REPEAT,

    IF_RADIO, IF_THIS, NULL,
    (int) "1D Wrap T: ", 2,
    1, (int) &TextureOps::cb_set_glTexParameter_1d_wrap_t,
    (int) "GL_CLAMP", GL_CLAMP,
    (int) "GL_REPEAT", GL_REPEAT,

    IF_RGBA, IF_THIS, NULL,
    (int) "1D Tex Border Color: ",
    (int)"0", (int)"0", (int)"0", (int)"0",
    (int) &TextureOps::cb_set_glTexParameter_1d_border_color,

    IF_FNAME, IF_THIS, NULL,
    (int) "1D Texture Filename: ",
    (int) "*.rgb",
    (int) &TextureOps::cb_read_1d_image_file,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "2D Textures: ", 2,
    0, (int) &TextureOps::cb_set_texture_2d,
    (int) "Off", FALSE,
    (int) "On", TRUE,

    IF_RADIO, IF_THIS, NULL,
    (int) "2D Min Filter: ", 6,
    4, (int) &TextureOps::cb_set_glTexParameter_2d_min_filter,
    (int) "GL_NEAREST", GL_NEAREST,
    (int) "GL_LINEAR", GL_LINEAR,
    (int) "GL_NEAREST_MIPMAP_NEAREST", GL_NEAREST_MIPMAP_NEAREST,
    (int) "GL_LINEAR_MIPMAP_NEAREST", GL_LINEAR_MIPMAP_NEAREST,
    (int) "GL_NEAREST_MIPMAP_LINEAR", GL_NEAREST_MIPMAP_LINEAR,
    (int) "GL_LINEAR_MIPMAP_LINEAR", GL_LINEAR_MIPMAP_LINEAR,

    IF_RADIO, IF_THIS, NULL,
    (int) "2D Mag Filter: ", 2,
    1, (int) &TextureOps::cb_set_glTexParameter_2d_mag_filter,
    (int) "GL_NEAREST", GL_NEAREST,
    (int) "GL_LINEAR", GL_LINEAR,

    IF_RADIO, IF_THIS, NULL,
    (int) "2D Wrap S: ", 2,
    1, (int) &TextureOps::cb_set_glTexParameter_2d_wrap_s,
    (int) "GL_CLAMP", GL_CLAMP,
    (int) "GL_REPEAT", GL_REPEAT,

    IF_RADIO, IF_THIS, NULL,
    (int) "2D Wrap T: ", 2,
    1, (int) &TextureOps::cb_set_glTexParameter_2d_wrap_t,
    (int) "GL_CLAMP", GL_CLAMP,
    (int) "GL_REPEAT", GL_REPEAT,

    IF_RGBA, IF_THIS, NULL,
    (int) "2D Tex Border Color: ",
    (int)"0", (int)"0", (int)"0", (int)"0",
    (int) &TextureOps::cb_set_glTexParameter_2d_border_color,

    IF_FNAME, IF_THIS, NULL,
    (int) "2D Texture Filename: ",
    (int) "*.rgb",
    (int) &TextureOps::cb_read_2d_image_file,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_RADIO, IF_THIS, NULL,
    (int) "Texture Env Application: ", 3,
    0, (int) &TextureOps::cb_set_glTexEnv_mode,
    (int) "GL_MODULATE", GL_MODULATE,
    (int) "GL_DECAL", GL_DECAL,
    (int) "GL_BLEND", GL_BLEND,

    IF_END
};

TextureOps::TextureOps( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();

    data_1d = NULL;
    data_2d = NULL;
}

TextureOps::~TextureOps()
{
}

///////////////////////////////////////////////////////////////////

void
TextureOps::install_values()
{
}

void
TextureOps::remember()
{
    GLfloat	f1, f2, f3, f4;
    GLenum	e;
    Boolean	b;

    get_texture_1d( &b );
    get_texture_2d( &b );
    get_glTexParameter_1d_min_filter( &e );
    get_glTexParameter_1d_mag_filter( &e );
    get_glTexParameter_1d_wrap_s( &e );
    get_glTexParameter_1d_wrap_t( &e );
    get_glTexParameter_1d_border_color( &f1, &f2, &f3, &f4 );
    get_glTexParameter_2d_min_filter( &e );
    get_glTexParameter_2d_mag_filter( &e );
    get_glTexParameter_2d_wrap_s( &e );
    get_glTexParameter_2d_wrap_t( &e );
    get_glTexParameter_2d_border_color( &f1, &f2, &f3, &f4 );
    get_glTexEnv_mode( &e );
}
    


//
// texture operations
//

void
TextureOps::set_texture_1d( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_TEXTURE_1D\n");
	} else {
	    printf("glDisable GL_TEXTURE_1D\n");
	}
    }
    switch_flag( GL_TEXTURE_1D, bool );

    _texture_1d = bool;

    redraw();
}

void
TextureOps::get_texture_1d( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_TEXTURE_1D );

    _texture_1d = state;

    *bool = state;
}

void
TextureOps::set_texture_2d( Boolean bool )
{
    if( debuglevel > 0 ) {
	if( bool ) {
	    printf("glEnable GL_TEXTURE_2D\n");
	} else {
	    printf("glDisable GL_TEXTURE_2D\n");
	}
    }
    switch_flag( GL_TEXTURE_2D, bool );

    _texture_2d = bool;

    redraw();
}

void
TextureOps::get_texture_2d( Boolean *bool )
{
    Boolean state = glIsEnabled( GL_TEXTURE_2D );
    
    _texture_2d = state;

    *bool = state;
}

void
TextureOps::set_glTexParameter_1d_min_filter( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_NEAREST:
	    s = "GL_NEAREST";
	    break;

	  case GL_LINEAR:
	    s = "GL_LINEAR";
	    break;

	  case GL_NEAREST_MIPMAP_NEAREST:
	    s = "GL_NEAREST_MIPMAP_NEAREST";
	    break;

	  case GL_LINEAR_MIPMAP_NEAREST:
	    s = "GL_LINEAR_MIPMAP_NEAREST";
	    break;

	  case GL_NEAREST_MIPMAP_LINEAR:
	    s = "GL_NEAREST_MIPMAP_LINEAR";
	    break;

	  case GL_LINEAR_MIPMAP_LINEAR:
	    s = "GL_LINEAR_MIPMAP_LINEAR";
	    break;
	}
	
	printf("glTexParameter GL_TEXTURE_1D GL_TEXTURE_MIN_FILTER %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, (int *) &val );

    _glTexParameter_1d_min_filter = val;

    redraw();
}

void
TextureOps::get_glTexParameter_1d_min_filter( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, (int *) val );

    _glTexParameter_1d_min_filter = *val;
}

void
TextureOps::set_glTexParameter_1d_mag_filter( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_NEAREST:
	    s = "GL_NEAREST";
	    break;

	  case GL_LINEAR:
	    s = "GL_LINEAR";
	    break;
	}

	printf("glTexParameter GL_TEXTURE_1D GL_TEXTURE_MAG_FILTER %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, (int *) &val );

    _glTexParameter_1d_mag_filter = val;

    redraw();
}

void
TextureOps::get_glTexParameter_1d_mag_filter( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, (int *) val );

    _glTexParameter_1d_mag_filter = *val;
}

void
TextureOps::set_glTexParameter_1d_wrap_s( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_CLAMP:
	    s = "GL_CLAMP";
	    break;

	  case GL_REPEAT:
	    s = "GL_REPEAT";
	    break;
	}

	printf("glTexParameter GL_TEXTURE_1D GL_TEXTURE_WRAP_S %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, (int *) &val );

    _glTexParameter_1d_wrap_s = val;

    redraw();
}

void
TextureOps::get_glTexParameter_1d_wrap_s( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, (int *) val );

    _glTexParameter_1d_wrap_s = *val;
}

void
TextureOps::set_glTexParameter_1d_wrap_t( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_CLAMP:
	    s = "GL_CLAMP";
	    break;

	  case GL_REPEAT:
	    s = "GL_REPEAT";
	    break;
	}

	printf("glTexParameter GL_TEXTURE_1D GL_TEXTURE_WRAP_T %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_WRAP_T, (int *) &val );

    _glTexParameter_1d_wrap_t = val;

    redraw();
}

void
TextureOps::get_glTexParameter_1d_wrap_t( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_1D, GL_TEXTURE_WRAP_T, (int *) val );

    _glTexParameter_1d_wrap_t = *val;
}

void
TextureOps::set_glTexParameter_1d_border_color( GLfloat r, GLfloat g,
					   GLfloat b, GLfloat a )
{
    if( debuglevel > 0 ) {
	printf("glTexParameter GL_TEXTURE_1D GL_TEXTURE_BORDER_COLOR [%f %f %f %f]\n",
	       r, g, b, a );
    }

    GLfloat	v[4];

    v[0] = r;
    v[1] = g;
    v[2] = b;
    v[3] = a;

    glTexParameterfv( GL_TEXTURE_1D, GL_TEXTURE_BORDER_COLOR, v );

    _glTexParameter_1d_border_color.r = r;
    _glTexParameter_1d_border_color.g = g;
    _glTexParameter_1d_border_color.b = b;
    _glTexParameter_1d_border_color.a = a;

    redraw();
}

void
TextureOps::get_glTexParameter_1d_border_color( GLfloat *r, GLfloat *g,
					   GLfloat *b, GLfloat *a )
{
    GLfloat	v[4];

    glGetTexParameterfv( GL_TEXTURE_1D, GL_TEXTURE_BORDER_COLOR, v );

    *r = v[0];
    *g = v[1];
    *b = v[2];
    *a = v[3];

    _glTexParameter_1d_border_color.r = *r;
    _glTexParameter_1d_border_color.g = *g;
    _glTexParameter_1d_border_color.b = *b;
    _glTexParameter_1d_border_color.a = *a;
}

void
TextureOps::set_glTexParameter_2d_min_filter( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_NEAREST:
	    s = "GL_NEAREST";
	    break;

	  case GL_LINEAR:
	    s = "GL_LINEAR";
	    break;

	  case GL_NEAREST_MIPMAP_NEAREST:
	    s = "GL_NEAREST_MIPMAP_NEAREST";
	    break;

	  case GL_LINEAR_MIPMAP_NEAREST:
	    s = "GL_LINEAR_MIPMAP_NEAREST";
	    break;

	  case GL_NEAREST_MIPMAP_LINEAR:
	    s = "GL_NEAREST_MIPMAP_LINEAR";
	    break;

	  case GL_LINEAR_MIPMAP_LINEAR:
	    s = "GL_LINEAR_MIPMAP_LINEAR";
	    break;
	}
	
	printf("glTexParameter GL_TEXTURE_2D GL_TEXTURE_MIN_FILTER %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (int *) &val );

    _glTexParameter_2d_min_filter = val;

    redraw();
}

void
TextureOps::get_glTexParameter_2d_min_filter( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, (int *) val );

    _glTexParameter_2d_min_filter = *val;
}

void
TextureOps::set_glTexParameter_2d_mag_filter( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_NEAREST:
	    s = "GL_NEAREST";
	    break;

	  case GL_LINEAR:
	    s = "GL_LINEAR";
	    break;
	}

	printf("glTexParameter GL_TEXTURE_2D GL_TEXTURE_MAG_FILTER %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (int *) &val );

    _glTexParameter_2d_mag_filter = val;

    redraw();
}

void
TextureOps::get_glTexParameter_2d_mag_filter( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, (int *) val );

    _glTexParameter_2d_mag_filter = *val;
}

void
TextureOps::set_glTexParameter_2d_wrap_s( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_CLAMP:
	    s = "GL_CLAMP";
	    break;

	  case GL_REPEAT:
	    s = "GL_REPEAT";
	    break;
	}

	printf("glTexParameter GL_TEXTURE_2D GL_TEXTURE_WRAP_S %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, (int *) &val );

    _glTexParameter_2d_wrap_s = val;

    redraw();
}

void
TextureOps::get_glTexParameter_2d_wrap_s( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, (int *) val );

    _glTexParameter_2d_wrap_s = *val;
}

void
TextureOps::set_glTexParameter_2d_wrap_t( GLenum val )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( val ) {
	  case GL_CLAMP:
	    s = "GL_CLAMP";
	    break;

	  case GL_REPEAT:
	    s = "GL_REPEAT";
	    break;
	}

	printf("glTexParameter GL_TEXTURE_2D GL_TEXTURE_WRAP_T %s\n", s );
    }

    glTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, (int *) &val );

    _glTexParameter_2d_wrap_t = val;

    redraw();
}

void
TextureOps::get_glTexParameter_2d_wrap_t( GLenum *val )
{
    glGetTexParameteriv( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, (int *) val );

    _glTexParameter_2d_wrap_t = *val;
}

void
TextureOps::set_glTexParameter_2d_border_color( GLfloat r, GLfloat g,
					   GLfloat b, GLfloat a )
{
    if( debuglevel > 0 ) {
	printf("glTexParameter GL_TEXTURE_2D GL_TEXTURE_BORDER_COLOR [%f %f %f %f]\n",
	       r, g, b, a );
    }

    GLfloat	v[4];

    v[0] = r;
    v[1] = g;
    v[2] = b;
    v[3] = a;

    glTexParameterfv( GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, v );

    _glTexParameter_2d_border_color.r = r;
    _glTexParameter_2d_border_color.g = g;
    _glTexParameter_2d_border_color.b = b;
    _glTexParameter_2d_border_color.a = a;

    redraw();
}

void
TextureOps::get_glTexParameter_2d_border_color( GLfloat *r, GLfloat *g,
					   GLfloat *b, GLfloat *a )
{
    GLfloat	v[4];

    glGetTexParameterfv( GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, v );

    *r = v[0];
    *g = v[1];
    *b = v[2];
    *a = v[3];

    _glTexParameter_2d_border_color.r = *r;
    _glTexParameter_2d_border_color.g = *g;
    _glTexParameter_2d_border_color.b = *b;
    _glTexParameter_2d_border_color.a = *a;
}

void
TextureOps::set_glTexEnv_mode( GLenum mode )
{
    if( debuglevel > 0 ) {
	char *s;

	switch( mode ) {
	  case GL_MODULATE:
	    s = "GL_MODULATE";
	    break;

	  case GL_DECAL:
	    s = "GL_DECAL";
	    break;

	  case GL_BLEND:
	    s = "GL_BLEND";
	    break;
	}

	printf("glTexEnv GL_TEXTURE_ENV GL_TEXTURE_ENV_MODE %s\n", s );
    }

    glTexEnviv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, (int *) &mode );

    _glTexEnv_mode = mode;

    redraw();
}

void
TextureOps::get_glTexEnv_mode( GLenum *mode )
{
    glGetTexEnviv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, (int *) mode );

    _glTexEnv_mode = *mode;
}


void
TextureOps::build_1d_mipmaps()
{
    int	format;

    switch( d_1d ) {
      case 1:
	format = GL_LUMINANCE;
	break;

      case 2:
	format = GL_LUMINANCE_ALPHA;
	break;

      case 3:
	format = GL_RGB;
	break;

      case 4:
	format = GL_RGBA;
	break;
    }
    
    gluBuild1DMipmaps( GL_TEXTURE_1D,
		      d_1d,
		      w_1d,
		      format,
		      GL_UNSIGNED_BYTE,
		      (void *) data_1d
		      );
}

void
TextureOps::build_2d_mipmaps()
{
    int	format;

    switch( d_2d ) {
      case 1:
	format = GL_LUMINANCE;
	break;

      case 2:
	format = GL_LUMINANCE_ALPHA;
	break;

      case 3:
	format = GL_RGB;
	break;

      case 4:
	format = GL_RGBA;
	break;
    }
    
    gluBuild2DMipmaps( GL_TEXTURE_2D,
		      d_2d,
		      w_2d,
		      h_2d,
		      format,
		      GL_UNSIGNED_BYTE,
		      (void *) data_2d
		      );
}
		      

void
TextureOps::read_1d_image_file( char *fname )
{
    readImageFile( fname, &data_1d, &w_1d, &h_1d, &d_1d );
}

void
TextureOps::read_2d_image_file( char *fname )
{
    readImageFile( fname, &data_2d, &w_2d, &h_2d, &d_2d );
}

void
TextureOps::readImageFile( char *fname, unsigned char **data,
		      int *w, int *h, int *d )
{
    FILE *fp;
    IMAGE *image_in;

    printf(" Opening file %s\n", fname );
    
    fp = fopen( fname, "r" );
    if( fp == NULL ) {
	perror("Cannot open texture file");
	return;
    }
    image_in = fiopen( fileno(fp), "r" );
    if( image_in == NULL ) {
	perror("Cannot open texture file");
	return;
    }

    *w = image_in->xsize;
    *h = image_in->ysize;
    *d = image_in->zsize;

    // allocate new memory
    if( *data ) {
	delete [] (*data);
    }
    *data = new unsigned char [ *w * *h * *d ];

    // allocate temp buffer
    short *rbuf = new short [ *w ];

    // now read the data in
    int	row, i, j;
    for( row = 0; row < *h; row++ ) {
	for( i = 0; i < *d; i++ ) {
	    if( getrow( image_in, rbuf, row, i ) < 0 ) {
		// something's wrong
		row = *h;
		break;
	    }
	    for( j = 0; j < *w; j++ ) {
		(*data) [ row*(*w)*(*d) + j*(*d) + i ] =
		    (unsigned char) rbuf[j];
	    }
	}
    }

    delete [] rbuf;

    iclose( image_in );
}
    


void
TextureOps::reset()
{
    int	debugstack;
    Boolean redrawstack;
    
    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }

    set_texture_1d( _texture_1d );
    set_texture_2d( _texture_2d );
    set_glTexParameter_1d_min_filter( _glTexParameter_1d_min_filter );
    set_glTexParameter_1d_mag_filter( _glTexParameter_1d_mag_filter );
    set_glTexParameter_1d_wrap_s( _glTexParameter_1d_wrap_s );
    set_glTexParameter_1d_wrap_t( _glTexParameter_1d_wrap_t );
    set_glTexParameter_1d_border_color( _glTexParameter_1d_border_color.r,
				       _glTexParameter_1d_border_color.g,
				       _glTexParameter_1d_border_color.b,
				       _glTexParameter_1d_border_color.a );
    set_glTexParameter_2d_min_filter( _glTexParameter_2d_min_filter );
    set_glTexParameter_2d_mag_filter( _glTexParameter_2d_mag_filter );
    set_glTexParameter_2d_wrap_s( _glTexParameter_1d_wrap_s );
    set_glTexParameter_2d_wrap_t( _glTexParameter_1d_wrap_t );
    set_glTexParameter_2d_border_color( _glTexParameter_2d_border_color.r,
				       _glTexParameter_2d_border_color.g,
				       _glTexParameter_2d_border_color.b,
				       _glTexParameter_2d_border_color.a );
    set_glTexEnv_mode( _glTexEnv_mode );

    if( _texture_1d ) {
	build_1d_mipmaps();
    }
    if( _texture_2d ) {
	build_2d_mipmaps();
    }

    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}

////////////////////////////////////////////////////////////////////
//  callbacks
////////////////////////////////////////////////////////////////////

void
TextureOps::cb_set_texture_1d( VkCallbackObject *, void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int	state = (int) call;

    self->set_texture_1d( state );
}

void
TextureOps::cb_set_texture_2d( VkCallbackObject *, void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int	state = (int) call;

    self->set_texture_2d( state );
}

void
TextureOps::cb_set_glTexParameter_1d_min_filter( VkCallbackObject *,
					    void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int	val = (int) call;

    self->set_glTexParameter_1d_min_filter( val );
}

void
TextureOps::cb_set_glTexParameter_1d_mag_filter( VkCallbackObject *,
					    void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int val = (int) call;

    self->set_glTexParameter_1d_mag_filter( val );
}

void
TextureOps::cb_set_glTexParameter_1d_wrap_s( VkCallbackObject *,
				     void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int val = (int) call;

    self->set_glTexParameter_1d_wrap_s( val );
}

void
TextureOps::cb_set_glTexParameter_1d_wrap_t( VkCallbackObject *,
					void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int	val = (int) call;

    self->set_glTexParameter_1d_wrap_t( val );
}

void
TextureOps::cb_set_glTexParameter_1d_border_color( VkCallbackObject *,
					      void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glTexParameter_1d_border_color( c->r, c->g, c->b, c->a );
}

void
TextureOps::cb_set_glTexParameter_2d_min_filter( VkCallbackObject *,
					    void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int	val = (int) call;

    self->set_glTexParameter_2d_min_filter( val );
}

void
TextureOps::cb_set_glTexParameter_2d_mag_filter( VkCallbackObject *,
					    void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int val = (int) call;

    self->set_glTexParameter_2d_mag_filter( val );
}

void
TextureOps::cb_set_glTexParameter_2d_wrap_s( VkCallbackObject *,
				     void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int val = (int) call;

    self->set_glTexParameter_2d_wrap_s( val );
}

void
TextureOps::cb_set_glTexParameter_2d_wrap_t( VkCallbackObject *,
					void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int	val = (int) call;

    self->set_glTexParameter_2d_wrap_t( val );
}

void
TextureOps::cb_set_glTexParameter_2d_border_color( VkCallbackObject *,
					      void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    RGBAColor *c = (RGBAColor *) call;

    self->set_glTexParameter_2d_border_color( c->r, c->g, c->b, c->a );
}

void
TextureOps::cb_set_glTexEnv_mode( VkCallbackObject *,
				 void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    int val = (int) call;

    self->set_glTexEnv_mode( val );
}

void
TextureOps::cb_read_1d_image_file( VkCallbackObject *,
				  void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    char *fname = (char *) call;

    self->read_1d_image_file( fname );
    self->build_1d_mipmaps();
}

void
TextureOps::cb_read_2d_image_file( VkCallbackObject *,
				  void *client, void *call )
{
    TextureOps *self = (TextureOps *) client;
    char *fname = (char *) call;

    self->read_2d_image_file( fname );
    self->build_2d_mipmaps();
}
