/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

#ifndef _TEXTUREOPS_H
#define _TEXTUREOPS_H

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Mon Jan 10 11:00:06 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:14:20 1994 by George Kyriazis
 *
 * Texturing operations
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "IfaceGenerator.h"

class TextureOps : public IfaceGenerator {
  public:
    TextureOps( GlWindow * );
    virtual ~TextureOps();

    // texture application
    void	set_texture_1d( Boolean );
    void	get_texture_1d( Boolean * );
    void	set_texture_2d( Boolean );
    void	get_texture_2d( Boolean * );
    void	set_glTexParameter_1d_min_filter( GLenum );
    void	get_glTexParameter_1d_min_filter( GLenum * );
    void	set_glTexParameter_1d_mag_filter( GLenum );
    void	get_glTexParameter_1d_mag_filter( GLenum * );
    void	set_glTexParameter_1d_wrap_s( GLenum );
    void	get_glTexParameter_1d_wrap_s( GLenum * );
    void	set_glTexParameter_1d_wrap_t( GLenum );
    void	get_glTexParameter_1d_wrap_t( GLenum * );
    void	set_glTexParameter_1d_border_color( GLfloat r,
						   GLfloat g,
						   GLfloat b,
						   GLfloat a );
    void	get_glTexParameter_1d_border_color( GLfloat *r,
						   GLfloat *g,
						   GLfloat *b,
						   GLfloat *a );
    void	set_glTexParameter_2d_min_filter( GLenum );
    void	get_glTexParameter_2d_min_filter( GLenum * );
    void	set_glTexParameter_2d_mag_filter( GLenum );
    void	get_glTexParameter_2d_mag_filter( GLenum * );
    void	set_glTexParameter_2d_wrap_s( GLenum );
    void	get_glTexParameter_2d_wrap_s( GLenum * );
    void	set_glTexParameter_2d_wrap_t( GLenum );
    void	get_glTexParameter_2d_wrap_t( GLenum * );
    void	set_glTexParameter_2d_border_color( GLfloat r,
						   GLfloat g,
						   GLfloat b,
						   GLfloat a );
    void	get_glTexParameter_2d_border_color( GLfloat *r,
						   GLfloat *g,
						   GLfloat *b,
						   GLfloat *a );
    void	set_glTexEnv_mode( GLenum );
    void	get_glTexEnv_mode( GLenum * );
    
    void	read_1d_image_file( char *fname );
    void	read_2d_image_file( char *fname );
    void	build_1d_mipmaps();	// to be called each frame
    void	build_2d_mipmaps();	// to be called each frame
    

    virtual	void	reset();

  protected:
    static	int	iface[];
    virtual	int	get_this() { return (int) this; }
    void	install_values();

    virtual	void	remember();

    // our state
    Boolean	_texture_1d;
    Boolean	_texture_2d;
    GLenum	_glTexParameter_1d_min_filter;
    GLenum	_glTexParameter_1d_mag_filter;
    GLenum	_glTexParameter_1d_wrap_s;
    GLenum	_glTexParameter_1d_wrap_t;
    RGBAColor	_glTexParameter_1d_border_color;
    GLenum	_glTexParameter_2d_min_filter;
    GLenum	_glTexParameter_2d_mag_filter;
    GLenum	_glTexParameter_2d_wrap_s;
    GLenum	_glTexParameter_2d_wrap_t;
    RGBAColor	_glTexParameter_2d_border_color;
    GLenum	_glTexEnv_mode;
    

    // texture related stuff
    unsigned	char	*data_1d;
    int	w_1d;	// width
    int	h_1d;	// height
    int	d_1d;	// depth
    unsigned	char	*data_2d;
    int	w_2d;	// width
    int	h_2d;	// height
    int	d_2d;	// depth
    void	readImageFile( char *fname, unsigned char **data,
			      int *w, int *h, int *d );

    // callbacks
    static	void	cb_set_texture_1d( CB_ARGS );
    static	void	cb_set_texture_2d( CB_ARGS );
    static	void	cb_set_glTexParameter_1d_min_filter( CB_ARGS );
    static	void	cb_set_glTexParameter_1d_mag_filter( CB_ARGS );
    static	void	cb_set_glTexParameter_1d_wrap_s( CB_ARGS );
    static	void	cb_set_glTexParameter_1d_wrap_t( CB_ARGS );
    static	void	cb_set_glTexParameter_1d_border_color( CB_ARGS );
    static	void	cb_set_glTexParameter_2d_min_filter( CB_ARGS );
    static	void	cb_set_glTexParameter_2d_mag_filter( CB_ARGS );
    static	void	cb_set_glTexParameter_2d_wrap_s( CB_ARGS );
    static	void	cb_set_glTexParameter_2d_wrap_t( CB_ARGS );
    static	void	cb_set_glTexParameter_2d_border_color( CB_ARGS );
    static	void	cb_set_glTexEnv_mode( CB_ARGS );
    static	void	cb_read_1d_image_file( CB_ARGS );
    static	void	cb_read_2d_image_file( CB_ARGS );
};

#endif /* _TEXTUREOPS_H */
