/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA 94039
 *
 * Created:       Tue Mar 15 14:20:33 1994 by George Kyriazis
 * Last Modified: Wed Mar 23 11:13:35 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "Transforms.h"

#include "GlWindow.h"
#include "MatrixItem.h"

#include "IfaceGeneratorP.h"

int Transforms::iface[] = {
    IF_BEGIN,
    IF_WINDOW, IF_THIS, NULL,
    (int) "Transformation Matrices", 1,

    IF_MATRIX, IF_THIS, IF_VALUE+1000,
    (int) "Model/View matrix",
    IF_VALUE+0, IF_VALUE+1, IF_VALUE+2, IF_VALUE+3,
    IF_VALUE+4, IF_VALUE+5, IF_VALUE+6, IF_VALUE+7,
    IF_VALUE+8, IF_VALUE+9, IF_VALUE+10, IF_VALUE+11,
    IF_VALUE+12, IF_VALUE+13, IF_VALUE+14, IF_VALUE+15,
    (int) &Transforms::cb_set_model_view,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_MATRIX, IF_THIS, IF_VALUE+1001,
    (int) "Projection matrix",
    IF_VALUE+100, IF_VALUE+101, IF_VALUE+102, IF_VALUE+103,
    IF_VALUE+104, IF_VALUE+105, IF_VALUE+106, IF_VALUE+107,
    IF_VALUE+108, IF_VALUE+109, IF_VALUE+110, IF_VALUE+111,
    IF_VALUE+112, IF_VALUE+113, IF_VALUE+114, IF_VALUE+115,
    (int) &Transforms::cb_set_proj,

    IF_SEPARATOR, IF_THIS, NULL,
    (int) "separator",

    IF_MATRIX, IF_THIS, IF_VALUE+1002,
    (int) "Texture matrix",
    IF_VALUE+200, IF_VALUE+201, IF_VALUE+202, IF_VALUE+203,
    IF_VALUE+204, IF_VALUE+205, IF_VALUE+206, IF_VALUE+207,
    IF_VALUE+208, IF_VALUE+209, IF_VALUE+210, IF_VALUE+211,
    IF_VALUE+212, IF_VALUE+213, IF_VALUE+214, IF_VALUE+215,
    (int) &Transforms::cb_set_proj,

    IF_END
};


Transforms::Transforms( GlWindow *dwa )
: IfaceGenerator( dwa )
{
    install_this( iface );
    install_values();
    parse( iface );
    remember();
}

Transforms::~Transforms()
{
}


///////////////////////////////////////////////////////////////////


void
Transforms::install_values()
{
    GLfloat	*m;
    int	i;

    get_model_view( &m );
    for( i = 0; i < 16; i++ ) {
	install( iface, IF_VALUE + i, (int) ftoa(m[i]) );
    }

    get_proj( &m );
    for( i = 0; i < 16; i++ ) {
	install( iface, IF_VALUE + 100 + i, (int) ftoa(m[i]) );
    }

    get_texture( &m );
    for( i = 0; i < 16; i++ ) {
	install( iface, IF_VALUE + 200 + i, (int) ftoa(m[i]) );
    }

    // install instance handles
    install( iface, IF_VALUE+1000, (int) &model_view_thing );
    install( iface, IF_VALUE+1001, (int) &proj_thing );
    install( iface, IF_VALUE+1002, (int) &texture_thing );
}

void
Transforms::remember()
{
    
}


void
Transforms::set_model_view( GLfloat *m )
{
    if( debuglevel > 0 ) {
	printf("glMatrixMode GL_MODELVIEW\n");

	printf("glLoadMatrix\t[ %f %f %f %f ]\n",
	       m[0], m[1], m[2], m[3] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[4], m[5], m[6], m[7] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[8], m[9], m[10], m[11] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[12], m[13], m[14], m[15] );
    }

    glMatrixMode( GL_MODELVIEW );
    glLoadMatrixf( m );

    int	i;
    for( i = 0; i < 16; i++ ) {
	_model_view[i] = m[i];
    }

    redraw();
}

void
Transforms::get_model_view( GLfloat **m )
{
    glGetFloatv( GL_MODELVIEW_MATRIX, _model_view );
    
    *m = _model_view;
}


void
Transforms::set_model_view_text( GLfloat *m )
{
    model_view_thing->set_matrix( m );
}


void
Transforms::set_proj( GLfloat *m )
{
    if( debuglevel > 0 ) {
	printf("glMatrixModel GL_PROJECTION\n" );

	printf("glLoadMatrix\t[ %f %f %f %f ]\n",
	       m[0], m[1], m[2], m[3] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[4], m[5], m[6], m[7] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[8], m[9], m[10], m[11] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[12], m[13], m[14], m[15] );
    }

    glMatrixMode( GL_PROJECTION );
    glLoadMatrixf( m );

    int	i;
    for( i = 0; i < 16; i++ ) {
	_proj[i] = m[i];
    }

    redraw();
}

void
Transforms::get_proj( GLfloat **m )
{
    glGetFloatv( GL_PROJECTION_MATRIX, _proj );

    *m = _proj;
}

void
Transforms::set_proj_text( GLfloat *m )
{
    proj_thing->set_matrix( m );
}


void
Transforms::set_texture( GLfloat *m )
{
    if( debuglevel > 0 ) {
	printf("glMatrixMode GL_TEXTURE\n");

	printf("glLoadMatrix\t[ %f %f %f %f ]\n",
	       m[0], m[1], m[2], m[3] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[4], m[5], m[6], m[7] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[8], m[9], m[10], m[11] );
	printf("\t\t[ %f %f %f %f ]\n",
	       m[12], m[13], m[14], m[15] );
    }

    glMatrixMode( GL_TEXTURE );
    glLoadMatrixf( m );

    int	i;
    for( i = 0; i < 16; i++ ) {
	_texture[i] = m[i];
    }

    redraw();
}

void
Transforms::get_texture( GLfloat **m )
{
    glGetFloatv( GL_TEXTURE_MATRIX, _texture );
    
    *m = _texture;
}


void
Transforms::set_texture_text( GLfloat *m )
{
    texture_thing->set_matrix( m );
}



void
Transforms::reset()
{
    int	debugstack;
    Boolean redrawstack;

    if( debuglevel <= 1 ) {
	debugstack = debuglevel;
	set_debug_level( 0 );
    }
    if( redrawStatus ) {
	redrawstack = redrawStatus;
	setRedraw( FALSE );
    }




    if( debuglevel == 0 ) {
	set_debug_level( debugstack );
    }
    if( redrawstack == TRUE ) {
	setRedraw( TRUE );
    }
}


///// callbacks

void
Transforms::cb_set_model_view( VkCallbackObject *,
			     void *client, void *call )
{
    Transforms *self = (Transforms *) client;
    GLfloat *m = (GLfloat *) call;

    // don't do anything...  We would have to extract all the
    // necessery info from the matrix and pass it to Inventor.
    // Not a fun thing to do on a Tuesday night.
#if 0
    self->set_model_view( m );
    self->glWindow->set_model_view( m );
#endif
}

void
Transforms::cb_set_proj( VkCallbackObject *,
			     void *client, void *call )
{
    Transforms *self = (Transforms *) client;
    GLfloat *m = (GLfloat *) call;

}

void
Transforms::cb_set_texture( VkCallbackObject *,
			     void *client, void *call )
{
    Transforms *self = (Transforms *) client;
    GLfloat *m = (GLfloat *) call;

}
