/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */

/************************************************************************
 *
 * File: %M%
 *
 * Copyright (c) 1994 by Silicon Graphics Inc., Mt. View, CA
 *
 * Created:       Fri Jan  7 18:58:55 1994 by George Kyriazis
 * Last Modified: Wed Sep 21 16:21:02 1994 by George Kyriazis
 *
 * < General description goes here > 
 *
 ************************************************************************/

#ident "$Revision: 1.1 $"

#include "BasicDefs.h"
#include "GlWindow.h"

#include <Vk/VkApp.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/SoDB.h>
#include <Inventor/SoInput.h>

#include "GlCone.h"
#include "GlSphere.h"
#include "GlCube.h"
#include "GlCylinder.h"
#include "GlFaceSet.h"
#include "GlIndexedFaceSet.h"
#include "GlUserDef1.h"

void
help( char *argv0 )
{
    printf( "Usage: %s [-ci][-nr][-sb][-fps][-h]\n", argv0 );
    printf( "\t-ci:  start in a color index visual\n");
    printf( "\t-nr:  do not redraw after user interaction\n");
    printf( "\t-sb:  start using a single buffer visual\n");
    printf( "\t-fps: display frames per second after each frame\n");
    printf( "\t-h:   help. this screen.\n");
    printf( "\n");
}

main( int argc, char **argv )
{
    char *progName = argv[0];
    char *fileName = NULL;
    VkApp	app( "Oat", &argc, argv );

    SoXt::init( app.baseWidget() );

    // init our own node classes
    GlCone::initClass();
    GlSphere::initClass();
    GlCube::initClass();
    GlCylinder::initClass();
    GlFaceSet::initClass();
    GlIndexedFaceSet::initClass();
    GlUserDef1::initClass();

    GlWindow	viewer( &app, "OpenGL Attribute Tester" );

    viewer.setUpdateMode( TRUE );
    viewer.showFPS( FALSE );

    while( argc >= 2 ) {
	if( strcmp( argv[1], "-sb" ) == 0 ) {
	   argv++;
	   argc--;

	   // do into single-buffer mode
	   viewer.setDbufMode( FALSE );
	   continue;
	}

	if( strcmp( argv[1], "-ci" ) == 0 ) {
	    argv++;
	    argc--;
	    
	    // enable color index mode
	    viewer.setRgbaMode( FALSE );
	    continue;
	}

	if( strcmp( argv[1], "-nr" ) == 0 ) {
	    argv++;
	    argc--;

	    // disable quick interaction
	    viewer.setUpdateMode( FALSE );
	    continue;
	}

	if( strcmp( argv[1], "-fps" ) == 0 ) {
	    argv++;
	    argc--;

	    // tell the viewer to display frames per second
	    viewer.showFPS( TRUE );
	    continue;
	}

	if( strcmp( argv[1], "-h" ) == 0 ) {
	    help( progName );
	    exit(0);
	}

	fileName = argv[1];
	argv++;
	argc--;
    }

    viewer.show();

    if( fileName ) {
	viewer.readFile( fileName );
    }

    app.run();

    return 0;
}

